@@ -114,7 +114,6 @@
 #include        "mcom.h"
 #include        "cmds.h"
   
-#define	DOWN	0
 #define	LIST	1
 #define	CLEAR	2
 #define	OFFLINE	3
@@ -127,6 +126,7 @@
 enum note_flags {unused, set, list};
 
 int quiet = 0;
+char *progname;
 
 
 /* globals */
@@ -140,12 +140,14 @@ mbool_t DisplayXML = FALSE;
  * set_note - set the note attribute for a node
  *
  */
+
 static int set_note(
-  int   con, 
+
+  int    con, 
   char  *name,
-  char  *msg
-)
-{
+  char  *msg)
+
+  {
   char	         *errmsg;
   struct attropl  new;
   int             rc;
@@ -166,73 +168,45 @@ static int set_note(
 
   if (rc && !quiet) 
     {
-    fprintf(stderr,"Error setting note attribute for %s - ", name);
+    fprintf(stderr,"Error setting note attribute for %s - ", 
+      name);
 
     if ((errmsg = pbs_geterrmsg(con)) != NULL)
-      fprintf(stderr,"%s\n", errmsg);
+      fprintf(stderr,"%s\n", 
+        errmsg);
     else
-      fprintf(stderr,"(error %d)\n", pbs_errno);
+      fprintf(stderr,"(error %d)\n", 
+        pbs_errno);
     }
 
-  return rc;
-}
-
-
-/*
- * cmp_node_name - compare two node names, allow the second to match the
- *	first if the same up to a dot ('.') in the second; i.e.
- *	"foo" == "foo.bar"
- *
- *	return 0 if match, 1 if not
- */
-
-static int cmp_node_name(
-
-  char *n1, 
-  char *n2)
-
-  {
-  while ((*n1 != '\0') && (*n2 != '\0')) 
-    {
-    if (*n1 != *n2)
-      break;
-
-    n1++;
-    n2++;
-    }
-
-  if (*n1 == *n2) 
-    {
-    return(0);
-    }
-
-  if ((*n1 == '.') && (*n2 == '\0'))
-    {
-    return(0);
-    }
-
-  return(1);
+  return(rc);
   }
 
 
 
-
 static void prt_node_attr(
 
-  struct batch_status *pbs)  /* I */
+  struct batch_status *pbs,         /* I */
+  int                  IsVerbose)   /* I */
 
   {
   struct attrl *pat;
 
   for (pat = pbs->attribs;pat;pat = pat->next) 
     {
+    if ((pat->value == NULL) || (pat->value[0] == '?'))
+      {
+      if (IsVerbose == 0)
+        continue;
+      }
+
     printf("     %s = %s\n", 
       pat->name, 
       pat->value);
-    }
+    }  /* END for (pat) */
 
   return;
-  }
+  }  /* END prt_node_attr() */
 
 
 
@@ -258,64 +232,6 @@ static char *get_nstate(
 
 
 
-
-/*
- * is_down - returns indication if node is marked down or not
- *
- *	returns 1 if node is down and 0 if not down
- */
-
-static int is_down(
-
-  struct batch_status *pbs)  /* I */
-
-  {
-
-  if (strstr(get_nstate(pbs),ND_down) != NULL)
-    {
-    return(1);
-    }
-
-  return(0);
-  }
-
-
-
-
-
-static int is_offline(
-
-  struct batch_status *pbs)
-
-  {
-
-  if (strstr(get_nstate(pbs),ND_offline) != NULL)
-    {
-    return(1);
-    }
-
-  return(0);
-  }
-
-
-
-
-static int is_unknown(
-
-  struct batch_status *pbs)
-
-  {
-
-  if (strstr(get_nstate(pbs),ND_state_unknown) != NULL)
-    {
-    return(1);
-    }
-
-  return(0);
-  }
-
-
-
 /* returns a pointer to the note if there is one, otherwise NULL */
 static char *get_note(
 
@@ -339,12 +255,12 @@ static char *get_note(
 
 static int marknode(
 
-  int   con, 
-  char *name, 
-  char *state1, 
-  enum batch_op op1,
-  char *state2, 
-  enum batch_op op2)
+  int            con, 
+  char          *name, 
+  char          *state1, 
+  enum batch_op  op1,
+  char          *state2, 
+  enum batch_op  op2)
 
   {
   char	         *errmsg;
@@ -395,20 +311,213 @@ static int marknode(
   }  /* END marknode() */
 
 
+struct batch_status *statnode(int con,char *nodearg)
+  {
+  struct batch_status *bstatus;
+  char	         *errmsg;
+
+  bstatus = pbs_statnode(con,nodearg,NULL,NULL);
+
+  if (bstatus == NULL) 
+    {
+    if (pbs_errno) 
+      {
+      if (!quiet) 
+        {
+        if ((errmsg = pbs_geterrmsg(con)) != NULL)
+          {
+          fprintf(stderr,"%s: %s\n",
+            progname,
+            errmsg);
+          }
+        else
+          {
+          fprintf(stderr,"%s: Error %d\n",
+            progname,
+            pbs_errno);
+          }
+        }
+
+      exit(1);
+      }
+
+    if (!quiet)
+      fprintf(stderr,"%s: No nodes found\n", 
+        progname);
+
+    exit(2);
+    }
+
+  return bstatus;
+  }    /* END statnode() */
 
 
+void addxmlnode(
+  mxml_t *DE,
+  struct batch_status *pbstat)
+  {
+  mxml_t *NE;
+  mxml_t *AE;
+  struct attrl *pat;
+
+  NE = NULL;
+
+  MXMLCreateE(&NE,"Node");
+
+  MXMLAddE(DE,NE);
+
+  /* add nodeid */
+
+  AE = NULL;
+  MXMLCreateE(&AE,"name");
+  MXMLSetVal(AE,pbstat->name,mdfString);
+  MXMLAddE(NE,AE);
+
+  for (pat = pbstat->attribs;pat;pat = pat->next)
+    {
+    AE = NULL;
+
+    if (pat->value == NULL)
+      continue;
+
+    MXMLCreateE(&AE,pat->name);
+
+    MXMLSetVal(AE,pat->value,mdfString);
+
+    MXMLAddE(NE,AE);
+    }
+} /* END addxmlnode() */
+
+
+enum NStateEnum {
+  tnsNONE = 0, /* default behavior - show down, offline, and unknown nodes */
+  tnsActive,   /* one or more jobs running on node */
+  tnsAll,      /* list all nodes */
+  tnsBusy,     /* node cannot accept additional workload */
+  tnsDown,     /* node is down */
+  tnsFree,     /* node is idle/free */
+  tnsOffline,  /* node is offline */
+  tnsUnknown,  /* node is unknown - no contact recieved */
+  tnsUp,       /* node is healthy */
+  tnsLAST };
+
+const char *NState[] = {
+  "NONE",
+  "active",
+  "all",
+  "busy",
+  "down",
+  "free",
+  "offline",
+  "unknown",
+  "up",
+  NULL };
+
+
+int filterbystate(
+  struct batch_status * pbstat,
+  enum NStateEnum ListType,
+  char *S)
+  {
+  int Display;
+
+  Display = 0;
+
+  switch (ListType)
+    {
+    case tnsNONE:  /* display down, offline, and unknown nodes */
+    default:
+
+      if (strstr(S,ND_down) || strstr(S,ND_offline) || strstr(S,ND_state_unknown)) 
+        {
+        Display = 1;
+        }
+
+      break;
+
+    case tnsActive:   /* one or more jobs running on node */
+
+      if (strstr(S,ND_busy) || strstr(S,ND_job_exclusive) || strstr(S,ND_job_sharing))
+        {
+        Display = 1;
+        }
+
+      break;
+
+    case tnsAll:      /* list all nodes */
+
+      Display = 1;
+
+      break;
+
+    case tnsBusy:     /* node cannot accept additional workload */
+
+      if (strstr(S,ND_busy))
+        {
+        Display = 1;
+        }
+
+      break;
+
+    case tnsDown:     /* node is down or unknown */
+
+      if (strstr(S,ND_down) || strstr(S,ND_state_unknown))
+        {
+        Display = 1;
+        }
+
+      break;
+
+    case tnsFree:     /* node is idle/free */
+
+      if (strstr(S,ND_free))
+        {
+        Display = 1;
+        }
+
+      break;
+
+    case tnsOffline:  /* node is offline */
+
+      if (strstr(S,ND_offline))
+        {
+        Display = 1;
+        }
+
+      break;
+
+    case tnsUnknown:  /* node is unknown - no contact recieved */
+
+      if (strstr(S,ND_state_unknown))
+        {
+        Display = 1;
+        }
+
+      break;
+
+    case tnsUp:       /* node is healthy */
+
+      if (!strstr(S,ND_down) && !strstr(S,ND_offline) && !strstr(S,ND_state_unknown))
+        {
+        Display = 1;
+        }
+
+      break;
+    }  /* END switch (ListType) */
+
+  return Display;
+}
 
 int main(
 
-  int	 argc,
-  char **argv)
+  int	 argc,  /* I */
+  char **argv)  /* I */
 
   {
   struct batch_status *bstatus = NULL;
   int	 con;
-  char	*def_server;
+  char	*specified_server = NULL;
   int	 errflg = 0;
-  char	*errmsg;
   int	 i;
   extern char	*optarg;
   extern int	 optind;
@@ -416,14 +525,15 @@ int main(
   struct batch_status *pbstat;
   int	flag = ALLI;
   char	*note = NULL;
-  enum note_flags note_flag = unused;
+  enum  note_flags note_flag = unused;
+  char **nodeargs=NULL;
+          int lindex;
+
+  enum NStateEnum ListType = tnsNONE;
 
   /* get default server, may be changed by -s option */
 
-  def_server = pbs_default();
-
-  if (def_server == NULL)
-    def_server = "";
+  progname=strdup(argv[0]);
 
   while ((i = getopt(argc,argv,"acdlopqrs:x-:N:n")) != EOF)
     {
@@ -479,7 +589,7 @@ int main(
 
       case 's':
    
-        def_server = optarg;
+        specified_server = optarg;
 
         break;
 
@@ -496,6 +606,7 @@ int main(
         /* preserve any previous option other than the default,
          * to allow -N to be combined with -o, -c, etc
          */
+
         if (flag == ALLI)
           flag = NOTE;
 
@@ -504,12 +615,14 @@ int main(
         if (note == NULL)
           {
           perror("Error: strdup() returned NULL");
+          exit(1);
           }
         else
           {
           note_flag = set;
 
           /* -N n is the same as -N ""  -- it clears the note */
+
           if (!strcmp(note,"n"))
             *note = '\0';
 
@@ -519,7 +632,6 @@ int main(
 
           if (strchr(note,'\n') != NULL)
             fprintf(stderr,"Warning: note contains a newline - server may reject it...\n");
-
           }
 
         break;
@@ -539,6 +651,12 @@ int main(
 
           exit(0);
           }
+        else if ((optarg != NULL) && !strcmp(optarg,"about"))
+          {
+          TShowAbout();
+
+          exit(0);
+          }
 
         errflg = 1;
 
@@ -559,7 +677,7 @@ int main(
     errflg = 1;
     }
 
-  for (pa = argv + optind;*pa;pa++) 
+  for (pa = argv + optind;*pa;pa++)
     {
     if (strlen(*pa) == 0)
       {
@@ -572,83 +690,74 @@ int main(
     if (!quiet)
       {
       fprintf(stderr,"usage:\t%s [-{c|d|l|o|p|r}] [-s server] [-n] [-N \"note\"] [-q] node ...\n",
-        argv[0]);
+        progname);
 
       fprintf(stderr,"\t%s [-{a|x}] [-s server] [-q] [node]\n",
-        argv[0]);
+        progname);
       }
 
     exit(1);
     }
 
-  con = cnt2server(def_server);
+  con = cnt2server(specified_server);
 
   if (con <= 0) 
     {
     if (!quiet)
       {
       fprintf(stderr, "%s: cannot connect to server %s, error=%d\n",
-        argv[0],           
-        def_server, 
+        progname,           
+        (specified_server) ? specified_server : pbs_default(), 
         pbs_errno);
       }
 
     exit(1);
     }
 
-  /* if flag is ALLI, DOWN or LIST, get status of all nodes */
+  /* if flag is ALLI, LIST, get status of all nodes */
 
-  if ((flag == ALLI) || (flag == DOWN) || (flag == LIST) || (flag == DIAG)) 
+  if ((flag == ALLI) || (flag == LIST) || (flag == DIAG)) 
     {
     if ((flag == ALLI) || (flag == LIST) || (flag == DIAG))
       {
-      /* allow node specification */
+      if (flag == LIST)
+        {
+        /* allow state specification */
+
+        if (argv[optind] != NULL)
+          {
+
+          for (lindex = 1;lindex < tnsLAST;lindex++)
+            {
+            if (!strcasecmp(NState[lindex],argv[optind]))
+              {
+              ListType = lindex;
+
+              optind++;
+
+              break;
+              }
+            }
+          } 
+        }
+
+      /* allow node specification (if none, then create an empty list) */
 
       if (argv[optind] != NULL)
-        bstatus = pbs_statnode(con,argv[optind],NULL,NULL);
-      else
-        bstatus = pbs_statnode(con,"",NULL,NULL);
-      }
-    else
-      {
-      /* node specification not allowed */
-
-      bstatus = pbs_statnode(con,"",NULL,NULL);
-      }
-
-    if (bstatus == NULL) 
-      {
-      if (pbs_errno) 
         {
-        if (!quiet) 
-          {
-          if ((errmsg = pbs_geterrmsg(con)) != NULL)
-            {
-            fprintf(stderr,"%s: %s\n",
-              argv[0],
-              errmsg);
-            }
-          else
-            {
-            fprintf(stderr,"%s: Error %d\n",
-              argv[0],
-              pbs_errno);
-            }
-          }
-
-        exit(1);
+        nodeargs=argv + optind;
+        }
+      else
+        {
+        nodeargs=malloc(2*sizeof(char **));
+        nodeargs[0]=strdup("");
+        nodeargs[1]='\0';
         }
- 
-      if (!quiet)
-        fprintf(stderr,"%s: No nodes found\n", 
-          argv[0]);
-
-      exit(0);
       }
-    }    /* END if ((flag == ALLI) || (flag == DOWN) || (flag == LIST) || (flag == DIAG)) */
+    }
 
 
-  if (note_flag == set)
+  if ((note_flag == set) && (note != NULL))
     {
     /* set the note attrib string on specified nodes */
     for (pa = argv + optind;*pa;pa++) 
@@ -665,50 +774,6 @@ int main(
 
       break;
 
-    case DOWN:
-
-      /*
-       * loop through the list of nodes returned above:
-       *   if node is up and is in argv list, mark it down;
-       *   if node is down and not in argv list, mark it up;
-       * for all changed nodes, send in request to server
-       */
-
-      for (pbstat = bstatus;pbstat != NULL;pbstat = pbstat->next) 
-        {
-        for (pa = argv + optind;*pa;pa++) 
-          {
-          if (cmp_node_name(*pa,pbstat->name) == 0) 
-            {
-            if (is_down(pbstat) == 0) 
-              {
-              marknode(
-                con, 
-                pbstat->name,
-                ND_down, 
-                INCR, 
-                NULL, 
-                INCR);
-
-              break;
-              }
-            }
-          }
-
-        if (*pa == NULL) 
-          {
-          /* node not in list, if down now, set up */
-
-          if (is_down(pbstat) == 1)
-            {
-            marknode(con,pbstat->name, 
-              ND_down,DECR,NULL,DECR);
-            }
-          }
-        }
-
-      break;
-
     case CLEAR:
 
       /* clear  OFFLINE from specified nodes */
@@ -754,46 +819,27 @@ int main(
 
       if (DisplayXML == TRUE)
         {
-        struct attrl *pat;
 
         char *tmpBuf=NULL, *tail=NULL;
         int  bufsize;
 
         mxml_t *DE;
-        mxml_t *NE;
-        mxml_t *AE;
 
         DE = NULL;
 
         MXMLCreateE(&DE,"Data");
 
-        for (pbstat = bstatus;pbstat;pbstat = pbstat->next)
+        for (lindex=0;nodeargs[lindex]!='\0';lindex++) 
           {
-          NE = NULL;
+          bstatus=statnode(con,nodeargs[lindex]);
 
-          MXMLCreateE(&NE,"Node");
-
-          MXMLAddE(DE,NE);
-
-          AE = NULL;
-          MXMLCreateE(&AE,"name");
-          MXMLSetVal(AE,pbstat->name,mdfString);
-          MXMLAddE(NE,AE);
-
-          for (pat = pbstat->attribs;pat;pat = pat->next)
+          for (pbstat = bstatus;pbstat;pbstat = pbstat->next)
             {
-            AE = NULL;
+            addxmlnode(DE,pbstat);
+            }    /* END for (pbstat) */
 
-            if (pat->value == NULL)
-              continue;
-
-            MXMLCreateE(&AE,pat->name);
-
-            MXMLSetVal(AE,pat->value,mdfString);
-
-            MXMLAddE(NE,AE);
-            }
-          }    /* END for (pbstat) */
+          pbs_statfree(pbstat);
+          }
 
         MXMLToXString(DE,&tmpBuf,&bufsize,INT_MAX,&tail,TRUE);
 
@@ -804,13 +850,21 @@ int main(
         }
       else
         {
-        for (pbstat = bstatus;pbstat;pbstat = pbstat->next) 
+        for (lindex=0;nodeargs[lindex]!='\0';lindex++) 
           {
-          printf("%s\n", 
-            pbstat->name);
-            prt_node_attr(pbstat);
+          bstatus=statnode(con,nodeargs[lindex]);
 
-          putchar('\n');
+          for (pbstat = bstatus;pbstat;pbstat = pbstat->next) 
+            {
+            printf("%s\n", 
+              pbstat->name);
+
+            prt_node_attr(pbstat,0);
+
+            putchar('\n');
+            }  /* END for (bpstat) */
+
+          pbs_statfree(pbstat);
           }
         }
 
@@ -820,26 +874,36 @@ int main(
 
       /* list any node that is DOWN, OFFLINE, or UNKNOWN */
 
-      for (pbstat = bstatus; pbstat; pbstat = pbstat->next) 
+      for (lindex=0;nodeargs[lindex]!='\0';lindex++) 
         {
-        if (is_down(pbstat) || is_offline(pbstat) || is_unknown(pbstat)) 
-          {
-          char *n;
+        bstatus=statnode(con,nodeargs[lindex]);
 
-          if ((note_flag == list) && (n = get_note(pbstat)))
+        for (pbstat = bstatus;pbstat != NULL;pbstat = pbstat->next) 
+          {
+          char *S;
+
+          S = get_nstate(pbstat);
+
+          if (filterbystate(pbstat,ListType,S))
             {
-            printf("%-20.20s %-26.26s %s\n",
-              pbstat->name,
-              get_nstate(pbstat),
-              n);
-            }
-          else
-            {
-            printf("%-20.20s %s\n",
-              pbstat->name,
-              get_nstate(pbstat));
+            char *n;
+
+            if ((note_flag == list) && (n = get_note(pbstat)))
+              {
+              printf("%-20.20s %-26.26s %s\n",
+                pbstat->name,
+                S,
+                n);
+              }
+            else
+              {
+              printf("%-20.20s %s\n",
+                pbstat->name,
+                S);
+              }
             }
           }
+        pbs_statfree(pbstat);
         }
 
       break;
