@@ -94,6 +94,8 @@
 #include "cmds.h"
 #include <pbs_config.h>   /* the master config generated by configure */
 
+void qdel_all(char *extend);
+
 /* qdel */
 
 int main(
@@ -103,8 +105,8 @@ int main(
 
   {
   int c;
-  int errflg=0;
-  int any_failed=0;
+  int errflg = 0;
+  int any_failed = 0;
   char *pc;
 
   char job_id[PBS_MAXCLTJOBID];	/* from the command line */
@@ -129,6 +131,8 @@ int main(
 
         if (extend[0] != '\0')
           {
+          /* extension option already specified */
+
           errflg++;
 
           break;
@@ -136,6 +140,8 @@ int main(
 
         if (strchr(optarg,'='))
           {
+          /* message cannot contain '=' character */
+
           errflg++;
 
           break;
@@ -208,7 +214,7 @@ int main(
 
   if ((errflg != 0) || (optind >= argc)) 
     {
-    static char usage[] = "usage: qdel [{ -p | -W delay | -m message}] [<JOBID>[ <JOBID>]...\n";
+    static char usage[] = "usage: qdel [{ -p | -W delay | -m message}] [<JOBID>[<JOBID>]| 'all']...\n";
 
     fprintf(stderr,usage);
 
@@ -223,9 +229,15 @@ int main(
     int stat=0;
     int located = FALSE;
 
-    strcpy(job_id,argv[optind]);
-
-    if (get_server(job_id,job_id_out,server_out)) 
+    /* check to see if user specified 'all' to delete all jobs */
+    
+    strcpy(job_id, argv[optind]);
+    if (strcmp("all", job_id) == 0)
+      {
+      qdel_all(extend);
+      continue;
+      }
+    else if (get_server(job_id,job_id_out,server_out)) 
       {
       fprintf(stderr,"qdel: illegally formed job identifier: %s\n", 
         job_id);
@@ -282,4 +294,96 @@ cnt:
   exit(any_failed);
   }  /* END main() */
 
+/* if a user requested deleting 'all' then this routine will get the list of
+ * jobs from the server and try to delete all jobs that are not in a
+ * 'C'omplete or 'E'xiting state
+ */  
+void qdel_all(
+  char *extend)   /* I */
+
+  {
+  char *jobid;
+  char *state = 0;
+  int connect;
+  int stat;
+  int retries;
+  struct batch_status *p_status;
+  struct batch_status *p;
+  struct attropl *p_atropl = 0;
+  struct attrl *a;
+
+  connect = cnt2server('\0');
+
+  if (connect <= 0) 
+    {
+    fprintf(stderr,"qdel: cannot connect to default server (errno=%d)\n",
+      pbs_errno);
+      
+    return;
+    }
+    
+    p_status = pbs_selstat(connect,p_atropl,NULL);
+    if ( p_status == NULL )
+      {
+      fprintf(stderr,"qdel: cannot find any jobs to delete\n");
+      }
+
+  for (p = p_status;p != NULL;p = p->next)
+    {
+    jobid = p->name;
+    a = p->attribs;
+    while (a != NULL) 
+      {
+      if ((a->name != NULL) && (!strcmp(a->name,ATTR_state)))
+        {
+        state = a->value;
+        break;
+        }
+      a = a->next;
+      }
+      
+    /* Don't bother deleting jobs that are 'C'omplete or 'E'xiting */
+    
+    if ((*state != 'E') && (*state != 'C'))
+      {
+      retries = 0;
+
+redo:
+      stat = pbs_deljob(connect,jobid,extend);
+      
+      /*
+       * if MOM is too slow to respond, we will retry a few times before
+       * before giving up
+       */
+
+      if (stat && (pbs_errno == PBSE_NORELYMOM) && (retries < 3)) 
+        {
+        sleep(1);
+        retries++;
+        goto redo;
+        }
+
+      if (stat &&
+        (pbs_errno != PBSE_UNKJOBID) &&
+        (pbs_errno != PBSE_BADSTATE)) 
+        {
+        printf("Deletion Error: %d\n", pbs_errno);
+        prt_job_err("qdel",connect,jobid);
+        /*
+         * if we don't have permission for this job, we probably don't have
+         * permission for any other jobs
+        */
+        if (pbs_errno == PBSE_PERM)
+          {
+          break;
+          }
+        }
+      }  
+
+    }
+    
+    pbs_disconnect(connect);
+ 
+  return;
+  }
 
