@@ -143,8 +143,8 @@
 #endif /* END HAVE_READLINE */
 
 /* PBS include files */
-#include <cmds.h>
-#include <qmgr.h>
+#include "cmds.h"
+#include "qmgr.h"
 
 
 
@@ -367,9 +367,9 @@ int main(
 
 int attributes(
 
-  char *attrs,
+  char            *attrs,
   struct attropl **attrlist,
-  int doper)
+  int              doper)
 
   {
   char *c;            /* Pointer into the attrs text */
@@ -715,8 +715,9 @@ struct server *make_connection(
     }
   else
     {
-    if( ! zopt ) fprintf(stderr,"qmgr: cannot connect to server %s\n", 
-      name);
+    if (!zopt) 
+      fprintf(stderr,"qmgr: cannot connect to server %s\n", 
+        name);
     }
 
   return(svr);
@@ -1377,6 +1378,7 @@ int execute(
 	else
 	  if( ! zopt ) fprintf(stderr,"No Active Nodes, nothing done.\n");
         break;
+/* DIAGTODO: handle new diag type */
       }
     }
   else
@@ -1417,6 +1419,8 @@ int execute(
 	  case MGR_OBJ_NODE:
 	    ss = pbs_statnode(sp -> s_connect, pname -> obj_name, sa, NULL);
 	  break;
+/* DIAGTODO: handle new diag type for list */
+/* DIAGTODO: create a pbs_statdiag() */
 	}
 	freeattrl(sa);
 	perr = (ss == NULL);
@@ -1445,6 +1449,7 @@ int execute(
 	  case MGR_OBJ_NODE:
 	    ss = pbs_statnode(sp->s_connect,pname->obj_name,sa,NULL);
 	  break;
+/* DIAGTODO: handle new diag type for print */
 	}
 
 	freeattrl(sa);
@@ -1965,6 +1970,7 @@ int is_attr(
     NULL
   };
 
+/* DIAGTODO: need a list of public and read-only diag attrs */
   char **attr_public = NULL;
   char **attr_readonly = NULL;
   int ret = FALSE;
@@ -2035,13 +2041,13 @@ void show_help(
   if( str[0] == '\0' )
     printf( HELP_DEFAULT );
   else if( strncmp(str, "active", 6) == 0 )
-    printf( HELP_ACTIVE );
+    printf( "%s%s", HELP_ACTIVE,HELP_ACTIVE2 );
   else if( strncmp(str, "create", 6) == 0 )
     printf( HELP_CREATE );
   else if( strncmp(str, "delete", 6) == 0 )
     printf( HELP_DELETE );
   else if( strncmp(str, "set", 3) == 0 )
-    printf( HELP_SET );
+    printf( "%s%s", HELP_SET,HELP_SET2 );
   else if( strncmp(str, "unset", 5) == 0 )
     printf( HELP_UNSET );
   else if( strncmp(str, "list", 4) == 0 )
@@ -2061,11 +2067,11 @@ void show_help(
   else if( strncmp(str, "attribute", 9) == 0 )
     printf( HELP_ATTRIBUTE );
   else if( strncmp(str, "serverpublic", 12) == 0 )
-    printf( HELP_SERVERPUBLIC );
+    printf( "%s%s%s", HELP_SERVERPUBLIC,HELP_SERVERPUBLIC2,HELP_SERVERPUBLIC3 );
   else if( strncmp(str, "serverro", 8) == 0 )
     printf( HELP_SERVERRO );
   else if( strncmp(str, "queuepublic", 11) == 0 )
-    printf( HELP_QUEUEPUBLIC );
+    printf( "%s%s%s", HELP_QUEUEPUBLIC,HELP_QUEUEPUBLIC2,HELP_QUEUEPUBLIC3 );
   else if( strncmp(str, "queueexec", 9) == 0)
     printf( HELP_QUEUEEXEC );
   else if( strncmp(str, "queueroute", 10) == 0 )
@@ -2190,6 +2196,7 @@ int parse(
       *type = MGR_OBJ_QUEUE;
     else if (strncmp(req[1],"node",lp) == 0) 
       *type = MGR_OBJ_NODE;
+/* DIAGTODO: allow new object type */
     else 
       {
       if( ! zopt ) fprintf(stderr,"qmgr: Illegal object type: %s.\n", 
