@@ -86,96 +86,153 @@
 #include <pbs_config.h>   /* the master config generated by configure */
 
 
-int main(int argc, char **argv)
-{
-    char job_id1[PBS_MAXCLTJOBID+1];		/* from the command line */
-    char job_id2[PBS_MAXCLTJOBID+1];		/* from the command line */
-    char job_id1_out[PBS_MAXCLTJOBID+1];
-    char job_id2_out[PBS_MAXCLTJOBID+1];
-    char *pn;
-    int  port1 = 0;
-    int  port2 = 0;
-    char server_out1[MAXSERVERNAME+1];
-    char server_out2[MAXSERVERNAME+1];
-    char svrtmp[MAXSERVERNAME+1];
-    int connect;
-    int stat=0;
-    int rc = 0;
+int main(
 
+  int    argc,   /* I */
+  char **argv)   /* I */
+ 
+  {
+  char job_id1[PBS_MAXCLTJOBID+1];		/* from the command line */
+  char job_id2[PBS_MAXCLTJOBID+1];		/* from the command line */
+  char job_id1_out[PBS_MAXCLTJOBID+1];
+  char job_id2_out[PBS_MAXCLTJOBID+1];
+  char *pn;
+  int  port1 = 0;
+  int  port2 = 0;
+  char server_out1[MAXSERVERNAME+1];
+  char server_out2[MAXSERVERNAME+1];
+  char svrtmp[MAXSERVERNAME+1];
+  int connect;
+  int stat = 0;
+  int rc = 0;
 
-    if (argc != 3) {
-        static char usage[]="usage: qorder job_identifier job_identifier\n";
-        fprintf(stderr, usage);
-        exit (2);
+  if (argc != 3) 
+    {
+    static char usage[] = "usage: qorder job_identifier job_identifier\n";
+
+    fprintf(stderr, 
+      usage);
+
+    exit(2);
     }
 
-        strcpy(job_id1, argv[1]);
-        strcpy(job_id2, argv[2]);
-	svrtmp[0] = '\0';
-        if ( get_server(job_id1, job_id1_out, svrtmp) ) {
-            fprintf(stderr, "qorder: illegally formed job identifier: %s\n", job_id1);
-            exit(1);
-        }
-	if (*svrtmp == '\0') {
-	    if ((pn = pbs_default())) {
-		(void)strcpy(svrtmp, pn);
-	    } else {
-		fprintf(stderr, "qorder: could not get default server: %s\n", job_id1);
-		exit(1);
-	    }
-	}
+  strcpy(job_id1,argv[1]);
+  strcpy(job_id2,argv[2]);
+
+  svrtmp[0] = '\0';
+
+  if (get_server(job_id1,job_id1_out,svrtmp)) 
+    {
+    fprintf(stderr,"qorder: illegally formed job identifier: %s\n", 
+      job_id1);
+
+    exit(1);
+    }
+
+  if (*svrtmp == '\0') 
+    {
+    if ((pn = pbs_default())) 
+      {
+      strcpy(svrtmp,pn);
+      } 
+    else 
+      {
+      fprintf(stderr,"qorder: could not get default server: %s\n", 
+        job_id1);
+
+      exit(1);
+      }
+    }
 		
-	if ((pn = strchr(svrtmp, (int)':')) != 0) {
-	    *pn = '\0';
-	    port1 = atoi(pn+1);
-	}
-	if (get_fullhostname(svrtmp, server_out1, MAXSERVERNAME) != 0) {
-	    fprintf(stderr, "qorder: invalid server name: %s\n", job_id1);
-	    exit(1);
-	}
+  if ((pn = strchr(svrtmp,(int)':')) != 0) 
+    {
+    *pn = '\0';
+
+    port1 = atoi(pn + 1);
+    }
+
+  if (get_fullhostname(svrtmp,server_out1,MAXSERVERNAME,NULL) != 0) 
+    {
+    fprintf(stderr,"qorder: invalid server name: %s\n", 
+      job_id1);
+
+    exit(1);
+    }
 	
-	svrtmp[0] = '\0';
-        if ( get_server(job_id2, job_id2_out, svrtmp) ) {
-            fprintf(stderr, "qorder: illegally formed job identifier: %s\n", job_id2);
-            exit(1);
-        }
-	if (*svrtmp == '\0') {
-	    if ((pn = pbs_default())) {
-		(void)strcpy(svrtmp, pn);
-	    } else {
-		fprintf(stderr, "qorder: could not get default server: %s\n", job_id1);
-		exit(1);
-	    }
-	}
-	if ((pn = strchr(svrtmp, (int)':')) != 0) {
-	    *pn = '\0';
-	    port2 = atoi(pn+1);
-	}
-	if (get_fullhostname(svrtmp, server_out2, MAXSERVERNAME) != 0) {
-	    fprintf(stderr, "qorder: invalid server name: %s\n", job_id2);
-	    exit(1);
-	}
-	if ( (strcmp(server_out1, server_out2) != 0) || (port1 != port2) ) {
-	    fprintf(stderr, "qorder: both jobs ids must specify the same server\n");
-	    exit(1);
-	}
-	if (pn)
-		*pn = ':';	/* restore : if it was present */
+  svrtmp[0] = '\0';
 
+  if (get_server(job_id2,job_id2_out,svrtmp)) 
+    {
+    fprintf(stderr,"qorder: illegally formed job identifier: %s\n", 
+      job_id2);
 
-        connect = cnt2server(svrtmp);
-        if ( connect <= 0 ) {
-            fprintf(stderr, "qorder: cannot connect to server %s (errno=%d)\n",
-                    pbs_server, pbs_errno);
-            exit(1);;
-        }
+    exit(1);
+    }
 
-        stat = pbs_orderjob(connect, job_id1_out, job_id2_out, NULL);
-        if ( stat ) {
-	    prt_job_err("qorder", connect, "");
-            rc = pbs_errno;
-	}
+  if (*svrtmp == '\0') 
+    {
+    if ((pn = pbs_default())) 
+      {
+      strcpy(svrtmp,pn);
+      } 
+    else 
+      {
+      fprintf(stderr,"qorder: could not get default server: %s\n", 
+        job_id1);
+
+      exit(1);
+      }
+    }
+
+  if ((pn = strchr(svrtmp,(int)':')) != 0) 
+    {
+    *pn = '\0';
+
+    port2 = atoi(pn + 1);
+    }
+
+  if (get_fullhostname(svrtmp,server_out2,MAXSERVERNAME,NULL) != 0) 
+    {
+    fprintf(stderr,"qorder: invalid server name: %s\n", 
+      job_id2);
+
+    exit(1);
+    }
+
+  if ((strcmp(server_out1,server_out2) != 0) || (port1 != port2)) 
+    {
+    fprintf(stderr,"qorder: both jobs ids must specify the same server\n");
+
+    exit(1);
+    }
+
+  if (pn != NULL)
+    *pn = ':';	/* restore : if it was present */
+
+  connect = cnt2server(svrtmp);
+
+  if (connect <= 0) 
+    {
+    fprintf(stderr, "qorder: cannot connect to server %s (errno=%d)\n",
+      pbs_server,  
+      pbs_errno);
+
+    exit(1);
+    }
+
+  stat = pbs_orderjob(connect,job_id1_out,job_id2_out,NULL);
+
+  if (stat != 0) 
+    {
+    prt_job_err("qorder",connect,"");
+
+    rc = pbs_errno;
+    }
+
+  pbs_disconnect(connect);
+
+  exit(rc);
+  }  /* END main() */
+
+/* END qorder.c */
 
-        pbs_disconnect(connect);
-        exit(rc);
-}
