@@ -95,58 +95,160 @@
 #include <pbs_config.h>   /* the master config generated by configure */
 
 
-int main(int argc, char **argv) /* qrerun */
-{
-    int any_failed=0;
+int main(
 
-    char job_id[PBS_MAXCLTJOBID];       /* from the command line */
+  int    argc, 
+  char **argv) /* qrerun */
 
-    char job_id_out[PBS_MAXCLTJOBID];
-    char server_out[PBS_MAXSERVERNAME];
-    char rmt_server[MAXSERVERNAME];
+  {
+  int any_failed = 0;
 
-    if (argc < 2) {
-        static char usage[]="usage: qrerun job_identifier...\n";
-        fprintf(stderr, usage);
-        exit (2);
+  char job_id[PBS_MAXCLTJOBID];       /* from the command line */
+
+  char job_id_out[PBS_MAXCLTJOBID];
+  char server_out[PBS_MAXSERVERNAME];
+  char rmt_server[MAXSERVERNAME];
+
+  char extend[1024];
+
+  int  errflg = 0;
+
+  char c;
+
+  static char usage[] = "usage: qrerun [-f] <JOBID>[ <JOBID>]...\n";
+
+  if (argc < 2) 
+    {
+    fprintf(stderr,usage);
+
+    exit(2);
     }
 
-    for ( optind=1; optind < argc; optind++) {
-        int connect;
-        int stat=0;
-	int located = FALSE;
+#define GETOPT_ARGS "m:f"
+
+  extend[0] = '\0';
+
+  while ((c = getopt(argc,argv,GETOPT_ARGS)) != EOF)
+    {
+    switch (c)
+      {
+      case 'm':
+
+        /* add delete message */
+
+        if (extend[0] != '\0')
+          {
+          /* extension option already specified */
+
+          errflg++;
+
+          break;
+          }
+
+        if (strchr(optarg,'='))
+          {
+          /* message cannot contain '=' character */
+
+          errflg++;
+
+          break;
+          }
+
+        strncpy(extend,optarg,sizeof(extend));
+
+        break;
+
+      case 'f':
+
+        if (extend[0] != '\0')
+          {
+          errflg++;
+
+          break;
+          }
+
+        strcpy(extend,RERUNFORCE);
+
+        break;
+
+      }  /* END switch (c) */
+    }    /* END while ((c = getopt(argc,argv,GETOPT_ARGS)) != EOF) */
+
+  if ((errflg != 0) || (optind >= argc))
+    {
+    fprintf(stderr,usage);
+
+    exit(2);
+    }
+
+  for (;optind < argc;optind++) 
+    {
+    int connect;
+    int stat = 0;
+    int located = FALSE;
+
+    strcpy(job_id,argv[optind]);
+
+    if (get_server(job_id,job_id_out,server_out)) 
+      {
+      fprintf(stderr,"qrerun: illegally formed job identifier: %s\n", 
+        job_id);
+
+      any_failed = 1;
+
+      continue;
+      }
 
-        strcpy(job_id, argv[optind]);
-        if ( get_server(job_id, job_id_out, server_out) ) {
-            fprintf(stderr, "qrerun: illegally formed job identifier: %s\n", job_id);
-            any_failed = 1;
-            continue;
-        }
 cnt:
-        connect = cnt2server(server_out);
-        if ( connect <= 0 ) {
-            fprintf(stderr, "qrerun: cannot connect to server %s (errno=%d)\n",
-                    pbs_server, pbs_errno);
-            any_failed = pbs_errno;
-            continue;
+
+    connect = cnt2server(server_out);
+
+    if (connect <= 0) 
+      {
+      fprintf(stderr,"qrerun: cannot connect to server %s (errno=%d)\n",
+        pbs_server, 
+        pbs_errno);
+
+      any_failed = pbs_errno;
+
+      continue;
+      }
+
+    if (extend[0] != '\0')
+      stat = pbs_rerunjob(connect,job_id_out,extend);
+    else
+      stat = pbs_rerunjob(connect,job_id_out,NULL);
+
+    if (stat && (pbs_errno != PBSE_UNKJOBID)) 
+      {
+      prt_job_err("qrerun",connect,job_id_out);
+
+      any_failed = pbs_errno;
+      } 
+    else if (stat && (pbs_errno == PBSE_UNKJOBID) && !located) 
+      {
+      located = TRUE;
+
+      if (locate_job(job_id_out,server_out,rmt_server)) 
+        {
+        pbs_disconnect(connect);
+
+        strcpy(server_out,rmt_server);
+
+        goto cnt;
         }
 
-        stat = pbs_rerunjob(connect, job_id_out, NULL);
-        if ( stat && (pbs_errno != PBSE_UNKJOBID) ) {
-	    prt_job_err("qrerun", connect, job_id_out);
-            any_failed = pbs_errno;
-        } else if ( stat && (pbs_errno == PBSE_UNKJOBID) && !located ) {
-	    located = TRUE;
-	    if ( locate_job(job_id_out, server_out, rmt_server) ) {
-	        pbs_disconnect(connect);
-		strcpy(server_out, rmt_server);
-		goto cnt;
-	    }
-	    prt_job_err("qrerun", connect, job_id_out);
-	    any_failed = pbs_errno;
-	}
+      prt_job_err("qrerun",connect,job_id_out);
+
+      any_failed = pbs_errno;
+      }
+
+    pbs_disconnect(connect);
+    }
+
+  exit(any_failed);
+  }  /* END main() */
+
+/* END qrerun.c */
+
 
-        pbs_disconnect(connect);
-   }
-    exit(any_failed);
-}
