@@ -101,14 +101,14 @@
 #include <pbs_config.h>   /* the master config generated by configure */
 
 int exitstatus = 0; /* Exit Status */
-static void execute ();
+static void execute();
 
 
 
 int main(
 
-  int    argc,
-  char **argv)
+  int    argc,  /* I */
+  char **argv)  /* I */
 
   {
   /*
@@ -121,10 +121,11 @@ int main(
   char server[MAXSERVERNAME];	  /* Server name */
   char *location = NULL;          /* Where to run the job */
     
-  static char opts[] = "H:";     /* See man getopt */
-  static char *usage = "Usage: qrun [-H host] job_id ...\n";
+  static char opts[] = "aH:";     /* See man getopt */
+  static char *usage = "Usage: qrun [-a] [-H host] job_id ...\n";
   int s;
   int errflg = 0;
+  int runAsync = FALSE;
     
   /* Command line options */
 
@@ -132,6 +133,12 @@ int main(
     {
     switch(s) 
       {
+      case 'a':	/* Async job start */
+
+        runAsync = TRUE;
+
+        break;
+        
       case 'H':
 
 	if (strlen(optarg) == 0) 
@@ -173,7 +180,7 @@ int main(
       continue;
       }
 
-    execute(job,server,location);
+    execute(job,server,location,runAsync);
     }  /* END for (;optind) */
 
   exit(exitstatus);
@@ -203,9 +210,10 @@ int main(
 
 static void execute(
 
-  char *job,
-  char *server,
-  char *location)
+  char *job,      /* I */
+  char *server,   /* I */
+  char *location, /* I */
+  int  async)     /* I */
 
   {
   int ct;         /* Connection to the server */
@@ -217,9 +225,24 @@ cnt:
  
   if ((ct = cnt2server(server)) > 0) 
     {
-    err = pbs_runjob(ct,job,location,NULL);  /* see lib/Libifl/pbsD_runjob.c */
+    if (async == TRUE)
+    {
+      err = pbs_asyrunjob(ct,job,location,NULL);  /* see lib/Libifl/pbsD_runjob.c */
+    }
+    else
+    {
+      err = pbs_runjob(ct,job,location,NULL);  /* see lib/Libifl/pbsD_runjob.c */
+    }
 
-    if (err && (pbs_errno != PBSE_UNKJOBID)) 
+    if (err && (pbs_errno == PBSE_UNKNODE))
+      {
+      fprintf(stderr,"qrun: Unknown node in hostlist '%.16s...' for job %s\n",
+        location,
+        job);
+
+      exitstatus = 2;
+      }
+    else if (err && (pbs_errno != PBSE_UNKJOBID)) 
       {
       prt_job_err("qrun",ct,job);
 
