@@ -95,17 +95,21 @@
 #include <pbs_config.h>   /* the master config generated by configure */
 
 
-int main(int argc, char **argv) /* qsig */
-{
-    int c;
-    int errflg=0;
-    int any_failed=0;
+int main(
+  
+  int    argc, 
+  char **argv) /* qsig */
 
-    char job_id[PBS_MAXCLTJOBID];       /* from the command line */
+  {
+  int c;
+  int errflg = 0;
+  int any_failed = 0;
 
-    char job_id_out[PBS_MAXCLTJOBID];
-    char server_out[MAXSERVERNAME];
-    char rmt_server[MAXSERVERNAME];
+  char job_id[PBS_MAXCLTJOBID];       /* from the command line */
+
+  char job_id_out[PBS_MAXCLTJOBID];
+  char server_out[MAXSERVERNAME];
+  char rmt_server[MAXSERVERNAME];
 
 #define MAX_SIGNAL_TYPE_LEN 32
     static char sig_string[MAX_SIGNAL_TYPE_LEN+1] = "SIGTERM";
@@ -148,7 +152,7 @@ cnt:
 
         stat = pbs_sigjob(connect, job_id_out, sig_string, NULL);
         if ( stat && (pbs_errno != PBSE_UNKJOBID) ) {
-	    prt_job_err("qsig", connect, job_id_out);
+	    prt_job_err("qsig",connect,job_id_out);
             any_failed = pbs_errno;
         } else if ( stat && (pbs_errno == PBSE_UNKJOBID) && !located ) {
 	    located = TRUE;
@@ -157,7 +161,7 @@ cnt:
 		strcpy(server_out, rmt_server);
 		goto cnt;
 	    }
-	    prt_job_err("qsig", connect, job_id_out);
+	    prt_job_err("qsig",connect,job_id_out);
 	    any_failed = pbs_errno;
 	}
 
