@@ -93,10 +93,6 @@
 
 #include <pbs_config.h>   /* the master config generated by configure */
 
-#include "cmds.h"
-#include "mcom.h"
-
-
 #include <pwd.h>
 #include <limits.h>
 
@@ -108,6 +104,9 @@
 #endif
 #endif
 
+#include "cmds.h"
+#include "mcom.h"
+
 static void states();
 
 #if !defined(PBS_NO_POSIX_VIOLATION)
@@ -136,11 +135,18 @@ mbool_t DisplayXML = FALSE;
 #define maxlinesize 65536
 int   linesize = 77;
 
+int   tasksize;
 
 /* END globals */
 
 
+#ifdef TXT
+#define DEFTASKSIZE 5
+#else
+#define DEFTASKSIZE 3
+#endif
 
+int   tasksize = DEFTASKSIZE;
 
 
 
@@ -162,26 +168,29 @@ int isjobid(
     result = 0;  /* who knows - probably a syntax error */
 
   return(result);
-  }
+  }  /* END isjobid() */
+
 
 
 
 int istrue(
 
-  char *string)
+  char *string)  /* I */
 
   {
-  if (strcmp(string,"TRUE") == 0) 
+  if (!strcasecmp(string,"TRUE")) 
     {
     return(TRUE);
     }
 
-  if ( strcmp(string,"True") == 0 ) return TRUE;
-  if ( strcmp(string,"true") == 0 ) return TRUE;
-  if ( strcmp(string,"1") == 0 ) return TRUE;
+  if (!strcmp(string,"1"))
+    {
+    return(TRUE);
+    }
 
   return(FALSE);
-  }
+  }  /* END istrue() */
+
 
 
 
@@ -257,7 +266,7 @@ static void states(
     }
 
   return;
-  }
+  }  /* END states() */
 
 
 
@@ -269,9 +278,9 @@ static void states(
 
 void prt_attr(
 
-  char *n,
-  char *r,
-  char *v)
+  char *n,  /* I name */
+  char *r,  /* I resource (optional) */
+  char *v)  /* I value */
 
   {
   char *c;
@@ -337,7 +346,7 @@ void prt_attr(
     }
 
   return;
-  }
+  }  /* END prt_attr() */
 
 
 
@@ -475,47 +484,65 @@ static void prt_nodes(
  *	gb    for	  -G
  */
 
-static char *cnv_size(value, opt)
-        char *value;
-        int   opt;
-{
-        static int sift_factor[3][5] = {
-		{ -20, -10, 0, 10, 20 },	/* mb conversion */
-		{ -23, -13, -3, 7, 17 },	/* mw conversion */
-		{ -30, -20, -10, 0, 10 } };	/* gb conversion */
-	int in;
-	int out;
-	int sft;
-	unsigned long nval;
-	char *pc;
-	char *suffix;
-	static char outbuf[25];
+static char *cnv_size(
 
-	if (opt & ALT_DISPLAY_Mb) {
-		out = 0;
-		suffix = "mb";
-	} else if (opt & ALT_DISPLAY_Mw) {
-		out = 1;
-		suffix = "mw";
-	} else if (opt & ALT_DISPLAY_G) {
-		out = 2;
-		suffix = "gb";
-	} else {
-		return (value);		/* return unmodified */
-	}
+  char *value,
+  int   opt)
 
-	nval = strtol(value, &pc, 10);
-	if (*pc == 'k')	in = 1;
-	else if (*pc == 'm') in = 2;
-	else if (*pc == 'g') in = 3;
-	else if (*pc == 't') in = 4;
-	else in = 0;
+  {
+  static int sift_factor[3][5] = {
+    { -20, -10, 0, 10, 20 },	/* mb conversion */
+    { -23, -13, -3, 7, 17 },	/* mw conversion */
+    { -30, -20, -10, 0, 10 } };	/* gb conversion */
 
-	sft = sift_factor[out][in];
+  int in;
+  int out;
+  int sft;
+  unsigned long nval;
+  char *pc;
+  char *suffix;
+  static char outbuf[25];
 
-	if ((*pc == 'w') || (*(pc+1) == 'w'))
-		nval = nval << 3;	/* convert to bytes */
+  if (opt & ALT_DISPLAY_Mb) 
+    {
+    out = 0;
 
+    suffix = "mb";
+    } 
+  else if (opt & ALT_DISPLAY_Mw) 
+    {
+    out = 1;
+
+    suffix = "mw";
+    } 
+  else if (opt & ALT_DISPLAY_G) 
+    {
+    out = 2;
+
+    suffix = "gb";
+    } 
+  else 
+    {
+    return (value);		/* return unmodified */
+    }
+
+  nval = strtol(value,&pc,10);
+
+  if (*pc == 'k')
+    in = 1;
+  else if (*pc == 'm')
+    in = 2;
+  else if (*pc == 'g')
+    in = 3;
+  else if (*pc == 't')
+    in = 4;
+  else 
+    in = 0;
+
+  sft = sift_factor[out][in];
+
+  if ((*pc == 'w') || (*(pc+1) == 'w'))
+    nval = nval << 3;	/* convert to bytes */
 	
 	if (sft < 0) {
 		nval = nval + ((1 << -sft) - 1);  /* round up (ceiling) */
@@ -524,9 +551,12 @@ static char *cnv_size(value, opt)
 		nval = nval << sft;
 	}
 
-	(void)sprintf(outbuf, "%lu%s", nval, suffix);
-        return outbuf;
-}
+  sprintf(outbuf,"%lu%s", 
+    nval, 
+    suffix);
+
+  return(outbuf);
+  }
 
 
 
@@ -538,9 +568,9 @@ static char *cnv_size(value, opt)
 
 static void altdsp_statjob(
 
-  struct batch_status *pstat,
-  struct batch_status *prtheader,
-  int                  alt_opt)
+  struct batch_status *pstat,     /* I */
+  struct batch_status *prtheader, /* I */
+  int                  alt_opt)   /* I */
 
   {
   char *comment;
@@ -583,7 +613,15 @@ static void altdsp_statjob(
       printf("Job ID               Username Queue    NDS   TSK Memory Time  S Time   BIG  FAST   PFS\n");
 
       printf("-------------------- -------- -------- ----- --- ------ ----- - ----- ----- ----- -----\n"); 
-      } 
+      }
+    else if (tasksize == 5)
+      {
+      printf("\n                                                                     Req'd  Req'd   Elap\n");
+
+      printf("Job ID               Username Queue    Jobname    SessID NDS   Tasks Memory Time  S Time\n");
+
+      printf("-------------------- -------- -------- ---------- ------ ----- ----- ------ ----- - -----\n");
+      }
     else 
       {
       printf("\n                                                                   Req'd  Req'd   Elap\n");
@@ -659,6 +697,11 @@ static void altdsp_statjob(
           if (strcmp(pat->value,"0"))
             tasks = pat->value;
           } 
+        else if (!strcmp(pat->resource,"size")) 
+          {
+          if (strcmp(pat->value,"0"))
+            tasks = pat->value;
+          } 
         else if (!strcmp(pat->resource,"mem")) 
           {
           strncpy(rqmem,cnv_size(pat->value,alt_opt),SIZEL);
@@ -715,8 +758,10 @@ static void altdsp_statjob(
 
     if (alt_opt & ALT_DISPLAY_R) 
       {
-      printf("%5.5s %3.3s %6.6s %5.5s %1.1s %5.5s %5.5s %5.5s %5.5s", 
+      printf("%5.5s %*.*s %6.6s %5.5s %1.1s %5.5s %5.5s %5.5s %5.5s", 
         nodect, 
+        tasksize,
+        tasksize,
         tasks, 
         rqmem, 
         usecput ? rqtimecpu : rqtimewal,
@@ -728,10 +773,12 @@ static void altdsp_statjob(
       } 
     else 
       {
-      printf("%-10.10s %6.6s %5.5s %3.3s %6.6s %5.5s %1.1s %5.5s",
+      printf("%-10.10s %6.6s %5.5s %*.*s %6.6s %5.5s %1.1s %5.5s",
         jobn, 
         sess, 
         nodect, 
+        tasksize,
+        tasksize,
         tasks, 
         rqmem,
         usecput ? rqtimecpu : rqtimewal,
@@ -771,24 +818,38 @@ static void altdsp_statjob(
  * get_ct - get count of jobs in queue/run state
  *	support function for altdsp_statque()
  */
-static void get_ct(str, jque, jrun)
-	char *str;
-	int  *jque;
-	int  *jrun;
-{
-	char *ps;
-	int   colon = (int)':';
 
-	ps = strchr(str, colon);	/* Transit - skip */
-	ps = strchr(ps+1, colon);	/* Queued  - add to jque */
-	*jque += atoi(ps+1);
-	ps = strchr(ps+1, colon);	/* Held    - add to jque  */
-	*jque += atoi(ps+1);
-	ps = strchr(ps+1, colon);	/* Waiting - add to jque  */
-	*jque += atoi(ps+1);
-	ps = strchr(ps+1, colon);	/* Running - add to jrun  */
-	*jrun += atoi(ps+1);
-}
+static void get_ct(
+
+  char *str,
+  int  *jque,
+  int  *jrun)
+
+  {
+  char *ps;
+  int   colon = (int)':';
+
+  ps = strchr(str,colon);	/* Transit - skip */
+
+  ps = strchr(ps + 1,colon);	/* Queued  - add to jque */
+
+  *jque += atoi(ps + 1);
+
+  ps = strchr(ps + 1,colon);	/* Held    - add to jque  */
+
+  *jque += atoi(ps + 1);
+
+  ps = strchr(ps + 1,colon);	/* Waiting - add to jque  */
+
+  *jque += atoi(ps + 1);
+
+  ps = strchr(ps+1,colon);	/* Running - add to jrun  */
+
+  *jrun += atoi(ps + 1);
+
+  return;
+  }  /* END get_ct() */
+
 
 
 
@@ -799,9 +860,9 @@ static void get_ct(str, jque, jrun)
 
 static void altdsp_statque(
 
-  char *serv,
+  char                *serv,
   struct batch_status *pstat,
-  int  opt)
+  int                  opt)
 
   {
   char  rmem[SIZEL];
@@ -852,36 +913,60 @@ static void altdsp_statque(
         } 
       else if (strcmp(pat->name,ATTR_start) == 0) 
         {
-				if (*pat->value == 'T')
-					qstarted = 'R';
-				else
-					qstarted = 'S';
-			} else if (strcmp(pat->name, ATTR_count) == 0) {
-				get_ct(pat->value, &jque, &jrun);
-				tot_jque += jque;
-				tot_jrun += jrun;
-			} else if (strcmp(pat->name, ATTR_rescmax) == 0) {
-				if (strcmp(pat->resource, "mem") == 0) {
-				    (void)strncpy(rmem,
-					  cnv_size(pat->value, opt), SIZEL);
-				} else if (strcmp(pat->resource, "cput") == 0) {
-				    cput = pat->value;
-			 	} else if (strcmp(pat->resource,"walltime")==0){
-				    wallt = pat->value;
-				} else if (strcmp(pat->resource, "nodect")==0) {
-				    nodect = pat->value;
-				}
-			}
-			pat = pat->next;
-		}
+        if (*pat->value == 'T')
+          qstarted = 'R';
+        else
+          qstarted = 'S';
+        } 
+     else if (strcmp(pat->name,ATTR_count) == 0) 
+        {
+        get_ct(pat->value,&jque,&jrun);
 
-		printf("%-16.16s %6.6s %8.8s %8.8s %5.5s ",
-		       pstat->name, rmem, cput, wallt, nodect);
-		printf("%3d %3d %2.2s   %c %c\n",
-		       jrun, jque, jmax, qenabled, qstarted);
+        tot_jque += jque;
+        tot_jrun += jrun;
+        } 
+      else if (strcmp(pat->name,ATTR_rescmax) == 0) 
+        {
+        if (strcmp(pat->resource, "mem") == 0) 
+          {
+          strncpy(
+            rmem,
+            cnv_size(pat->value,opt), 
+            SIZEL);
+          } 
+        else if (strcmp(pat->resource,"cput") == 0) 
+          {
+          cput = pat->value;
+          } 
+        else if (strcmp(pat->resource,"walltime") == 0)
+          {
+          wallt = pat->value;
+          } 
+        else if (strcmp(pat->resource,"nodect") == 0) 
+          {
+          nodect = pat->value;
+          }
+        }
 
-		pstat = pstat->next;
-	}
+      pat = pat->next;
+      }
+
+    printf("%-16.16s %6.6s %8.8s %8.8s %5.5s ",
+      pstat->name, 
+      rmem, 
+      cput, 
+      wallt, 
+      nodect);
+
+    printf("%3d %3d %2.2s   %c %c\n",
+      jrun, 
+      jque, 
+      jmax, 
+      qenabled, 
+      qstarted);
+
+    pstat = pstat->next;
+    }  /* END while (pstat != NULL) */
 
   printf("                                               ----- -----\n");
   printf("                                               %5d %5d\n", 
@@ -889,7 +974,7 @@ static void altdsp_statque(
     tot_jque);
 
   return;
-  }
+  }  /* END altdsp_statque() */
 
 
 
@@ -897,27 +982,36 @@ static void altdsp_statque(
 
 /* build and add an attropl struct to the list */
 
-static void add_atropl(list, name, resc, value, op)
-	struct attropl **list;
-	char		*name;
-	char		*resc;
-	char		*value;
-	enum batch_op	 op;
-{
-	struct attropl *patro;
+static void add_atropl(
 
-	patro = (struct attropl *)malloc(sizeof (struct attropl));
-	if (patro == 0) {
-		fprintf(stderr, "cannot malloc space\n");
-		exit (1);
-	}
-	patro->next     = *list;
-	patro->name     = name;
-	patro->resource = resc;
-	patro->value    = value;
-	patro->op       = op;
-	*list = patro;
-}
+  struct attropl **list,
+  char            *name,
+  char            *resc,
+  char            *value,
+  enum batch_op	   op)
+
+  {
+  struct attropl *patro;
+
+  patro = (struct attropl *)malloc(sizeof (struct attropl));
+
+  if (patro == NULL) 
+    {
+    fprintf(stderr,"cannot malloc space\n");
+
+    exit(1);
+    }
+
+  patro->next     = *list;
+  patro->name     = name;
+  patro->resource = resc;
+  patro->value    = value;
+  patro->op       = op;
+  
+  *list = patro;
+
+  return;
+  }  /* END add_atropl() */
 
 
 
@@ -957,7 +1051,7 @@ void display_statjob(
   if (!full)
     {
     sprintf(format,"%%-%ds %%-%ds %%-%ds %%%ds %%%ds %%-%ds\n", 
-      PBS_MAXSEQNUM + 11, 
+      PBS_MAXSEQNUM + PBS_MAXJOBARRAYLEN + 11, 
       NAMEL, 
       OWNERL, 
       TIMEUL, 
@@ -966,9 +1060,10 @@ void display_statjob(
 
     if (prtheader) 
       {
-      /* display summary header */
-      printf("Job id              Name             User            Time Use S Queue\n");
-      printf("------------------- ---------------- --------------- -------- - -----\n");
+      /* display summary header TODO - the sizes of these fields should be determined from 
+         #defines in pbs_ifl.h */
+      printf("Job id                    Name             User            Time Use S Queue\n");
+      printf("------------------------- ---------------- --------------- -------- - -----\n");
       }
     }    /* END if (!full) */
 
@@ -1011,7 +1106,6 @@ void display_statjob(
       a = p->attribs;
 
       RE1 = NULL;
-
       while (a != NULL) 
         {
         if (a->name != NULL) 
@@ -1046,6 +1140,7 @@ void display_statjob(
                 !strcmp(a->name,ATTR_etime) ||
                 !strcmp(a->name,ATTR_mtime) ||
                 !strcmp(a->name,ATTR_qtime) ||
+                !strcmp(a->name,ATTR_start_time) ||
                 !strcmp(a->name,ATTR_a)) 
               {
               epoch = (time_t)atoi(a->value);
@@ -1084,11 +1179,11 @@ void display_statjob(
 
         l = strlen(p->name);
 
-        if (l > (PBS_MAXSEQNUM + 8)) 
+        if (l > (PBS_MAXSEQNUM + PBS_MAXJOBARRAYLEN + 8)) 
           {
           /* truncate job name */
 
-          c = p->name + PBS_MAXSEQNUM + 14;
+          c = p->name + PBS_MAXSEQNUM + PBS_MAXJOBARRAYLEN + 14;
 
           *c = '\0';
           }
@@ -1098,7 +1193,6 @@ void display_statjob(
 
       a = p->attribs;
 
-      RE1 = NULL;
       while (a != NULL) 
         {
         if (a->name != NULL) 
@@ -1482,19 +1576,22 @@ void display_statserver(
     printf(format,"----------------","---","---","---","---","---","---","---","---","----------");
     }
 
-    p = status;
-    while ( p != NULL ) {
-        name = NULL;
-        max = "0";
-        tot = "0";
-        strcpy(que, "0");
-        strcpy(run, "0");
-        strcpy(hld, "0");
-        strcpy(wat, "0");
-        strcpy(trn, "0");
-        strcpy(ext, "0");
-        stats = "";
-        if (full) {
+  p = status;
+
+  while (p != NULL) 
+    {
+    name = NULL;
+    max = "0";
+    tot = "0";
+    strcpy(que, "0");
+    strcpy(run, "0");
+    strcpy(hld, "0");
+    strcpy(wat, "0");
+    strcpy(trn, "0");
+    strcpy(ext, "0");
+    stats = "";
+
+    if (full) {
             printf("Server: %s\n", p->name);
             a = p->attribs;
             while ( a != NULL ) {
@@ -1531,7 +1628,7 @@ void display_statserver(
                         }
                         tot = a->value;
                     } else if ( strcmp(a->name,ATTR_count) == 0 ) {
-                        states(a->value, que, run, hld, wat, trn, ext, NUML);
+                        states(a->value,que,run,hld,wat,trn,ext,NUML);
                     } else if ( strcmp(a->name,ATTR_status) == 0 ) {
                         l = strlen(a->value);
                         if ( l > STATUSL ) {
@@ -1564,32 +1661,50 @@ char *attrlist(
   struct attrl *ap)
 
   {
-	char	nameres[256];
-	char	*argv[ARGNUM];
-	char	*ret;
-	int	i, num = 0;
+  char	nameres[256];
+  char	*argv[ARGNUM];
+  char	*ret;
+  int	i, num = 0;
+
+  while (ap) 
+    {
+    char *twol[2];
+
+    if (ap->resource != NULL) 
+      {
+      sprintf(nameres,"%s%s%s",
+        ap->name, 
+        TCL_ATRSEP, 
+        ap->resource);
+
+      twol[0] = nameres;
+      }
+    else
+      {
+      twol[0] = ap->name;
+      }
+
+    twol[1] = ap->value;
+
+    argv[num++] = Tcl_Merge(2,(const char **)twol);
+
+    if (num == ARGNUM)
+      break;
+
+    ap = ap->next;
+    }
+
+  ret = Tcl_Merge(num,(const char **)argv);
+
+  for (i = 0;i < num;i++)
+    free(argv[i]);
+
+  return(ret);
+  }
+
+
 
-	while (ap) {
-		char	*twol[2];
 
-		if (ap->resource) {
-			sprintf(nameres, "%s%s%s",
-					ap->name, TCL_ATRSEP, ap->resource);
-			twol[0] = nameres;
-		}
-		else
-			twol[0] = ap->name;
-		twol[1] = ap->value;
-		argv[num++] = Tcl_Merge(2, twol);
-		if (num == ARGNUM)
-			break;
-		ap = ap->next;
-	}
-	ret = Tcl_Merge(num, argv);
-	for (i=0; i<num; i++)
-		free(argv[i]);
-	return (ret);
-}
 
 Tcl_Interp	*interp = NULL;
 char		script[200];
@@ -1651,50 +1766,69 @@ tcl_addarg(name, arg)
 			TCL_APPEND_VALUE);
 }
 
-int
-tcl_stat(type, bs, f_opt)
-    char	*type;
-    struct	batch_status	*bs;
-    int		f_opt;
-{
-	struct	batch_status	*bp;
-	char	*twol[2];
-	char	*argv[ARGNUM];
-	int	i, num = 0;
-	char	*result;
 
-	if (interp == NULL)
-		return 1;
 
-	if (f_opt == 0)
-		return 1;
 
-	twol[0] = type;
-	for (bp=bs; bp; bp=bp->next) {
-		char	*threel[3];
 
-		threel[0] = bp->name;
-		threel[1] = attrlist(bp->attribs);
-		threel[2] = bp->text;
+int tcl_stat(
 
-		argv[num++] = Tcl_Merge(3, threel);
-		free(threel[1]);	/* malloc'ed in attrlist() */
-		if (num == ARGNUM)
-			break;
-	}
-	twol[1] = Tcl_Merge(num, argv);
-	for (i=0; i<num; i++)
-		free(argv[i]);
+  char                *type,
+  struct batch_status *bs,
+  int                  f_opt)
 
-	result = Tcl_Merge(2, twol);
-	Tcl_SetVar(interp, "objects", result,
-				TCL_GLOBAL_ONLY|
-				TCL_LIST_ELEMENT|
-				TCL_APPEND_VALUE);
-	free(twol[1]);
-	free(result);
-	return 0;
-}
+  {
+  struct	batch_status	*bp;
+  char	*twol[2];
+  char	*argv[ARGNUM];
+  int	i, num = 0;
+  char	*result;
+
+  if (interp == NULL)
+    {
+    return(1);
+    }
+
+  if (f_opt == 0)
+    {
+    return(1);
+    }
+
+  twol[0] = type;
+
+  for (bp = bs;bp != NULL;bp = bp->next) 
+    {
+    char *threel[3];
+
+    threel[0] = bp->name;
+    threel[1] = attrlist(bp->attribs);
+    threel[2] = bp->text;
+
+    argv[num++] = Tcl_Merge(3,(const char **)threel);
+
+    free(threel[1]);	/* malloc'ed in attrlist() */
+
+    if (num == ARGNUM)
+      break;
+    }
+
+  twol[1] = Tcl_Merge(num,(const char **)argv);
+
+  for (i = 0;i < num;i++)
+    free(argv[i]);
+
+  result = Tcl_Merge(2,(const char **)twol);
+
+  Tcl_SetVar(
+    interp, 
+    "objects", 
+    result,
+    TCL_GLOBAL_ONLY|TCL_LIST_ELEMENT|TCL_APPEND_VALUE);
+
+  free(twol[1]);
+  free(result);
+
+  return(0);
+  }
 
 
 
@@ -1713,7 +1847,7 @@ void tcl_run(
     {
     char *trace;
 
-    trace = Tcl_GetVar(interp,"errorInfo",0);
+    trace = (char *)Tcl_GetVar(interp,"errorInfo",0);
 
     if (trace == NULL)
       trace = interp->result;
@@ -1743,8 +1877,8 @@ void tcl_run(
 
 int main(
 
-  int    argc,
-  char **argv)
+  int    argc,  /* I */
+  char **argv)  /* I */
 
   {
   int c;
@@ -1781,7 +1915,7 @@ int main(
   struct batch_status *p_server;
   struct attropl *p_atropl = 0;
   char *errmsg;
-  int exec_only=0;
+  int exec_only = 0;
 
 #ifndef mbool
 #define mbool char
@@ -1820,8 +1954,8 @@ int main(
     {
     option[1] = (char)c;
 
-    tcl_addarg(flags, option);
-    tcl_addarg(flags, optarg);
+    tcl_addarg(flags,option);
+    tcl_addarg(flags,optarg);
 
     switch (c) 
       {
@@ -1871,7 +2005,7 @@ int main(
 
         alt_opt |= ALT_DISPLAY_r;
 
-        add_atropl(&p_atropl, ATTR_state, (char *)0, "RS", EQ);
+        add_atropl(&p_atropl,ATTR_state,NULL,"RS",EQ);
 
         break;
 
@@ -1885,7 +2019,7 @@ int main(
 
         alt_opt |= ALT_DISPLAY_u;
 
-        add_atropl(&p_atropl, ATTR_u, (char *)0, optarg, EQ);
+        add_atropl(&p_atropl,ATTR_u,NULL,optarg,EQ);
 
         break;
 
@@ -1970,6 +2104,13 @@ int main(
           exit(0);
           }
 
+        if ((optarg != NULL) && !strcmp(optarg,"about"))
+          {
+          TShowAbout();
+
+          exit(0);
+          }
+
         /* unexpected '--' option received */
 
         errflg = 1;
@@ -2044,7 +2185,7 @@ int main(
 		++pc;
 	    }
 #endif	/* (TCL_QSTAT == 0) */
-	    break;
+        break;
 
       case '?':
       default:
@@ -2150,7 +2291,7 @@ qstat -B [-f [-1]] [-W site_specific] [ server_name... ]\n";
         server_out[0] = '@';
         strcpy(&server_out[1],def_server);
 
-        tcl_addarg(ops, server_out);
+        tcl_addarg(ops,server_out);
 
         queue_name_out = NULL;
 
@@ -2189,9 +2330,9 @@ qstat -B [-f [-1]] [-W site_specific] [ server_name... ]\n";
 
     located = FALSE;
 
-    strcpy(operand, argv[optind]);
+    strcpy(operand,argv[optind]);
 
-    tcl_addarg(ops, operand);
+    tcl_addarg(ops,operand);
 
     switch (mode) 
       {
@@ -2207,7 +2348,7 @@ qstat -B [-f [-1]] [-W site_specific] [ server_name... ]\n";
 
           if (get_server(job_id,job_id_out,server_out)) 
             {
-            fprintf(stderr, "qstat: illegally formed job identifier: %s\n", 
+            fprintf(stderr,"qstat: illegally formed job identifier: %s\n", 
               job_id);
 
             ret = tcl_stat(error,NULL,f_opt);
@@ -2223,15 +2364,21 @@ qstat -B [-f [-1]] [-W site_specific] [ server_name... ]\n";
 
           stat_single_job = 0;
 
-                strcpy(destination, operand);
-                if ( parse_destination_id(destination,
-                                          &queue_name_out,
-                                          &server_name_out) ) {
-                    fprintf(stderr, "qstat: illegally formed destination: %s\n", destination);
-		    ret = tcl_stat(error, NULL, f_opt);
-                    any_failed = 1;
-                    break;
-                } else {
+          strcpy(destination,operand);
+
+          if (parse_destination_id(
+                destination,
+                &queue_name_out,
+                &server_name_out)) 
+            {
+            fprintf(stderr, "qstat: illegally formed destination: %s\n", destination);
+
+            ret = tcl_stat(error, NULL, f_opt);
+            any_failed = 1;
+            break;
+            } 
+          else 
+            {
                     if ( notNULL(server_name_out) ) {
                         strcpy(server_out, server_name_out);
                     } else {
@@ -2256,7 +2403,7 @@ job_no_args:
             pbs_server, 
             pbs_errno);
 
-          ret = tcl_stat(error, NULL, f_opt);
+          ret = tcl_stat(error,NULL,f_opt);
 
           any_failed = connect;
 
