@@ -128,7 +128,7 @@
  */
 
 struct svrattrl {
-	tlist_link	al_link;
+	list_link	al_link;
 	struct attropl	al_atopl;    /* name,resource,value, see pbs_ifl.h   */
 	int		al_tsize;    /* size of this structure (variable)    */
 	int		al_nameln;   /* len of name string (including null)  */
@@ -272,6 +272,7 @@ typedef struct attribute_def attribute_def;
 #define PARENT_TYPE_QUE_PULL	 5
 #define PARENT_TYPE_SERVER	 6
 #define PARENT_TYPE_NODE	 7
+/* DIAGTODO: define PARENT_TYPE_DIAG, but atrpart field isn't wide enough */
 
 /*
  * values for the "actmode" parameter to at_action()
@@ -335,6 +336,7 @@ extern int  decode_size   A_((attribute *patr,char *name, char *rn, char *val));
 extern int  decode_str   A_((attribute *patr,char *name, char *rn, char *val));
 extern int  decode_time  A_((attribute *patr,char *name, char *rn, char *val));
 extern int  decode_arst  A_((attribute *patr,char *name, char *rn, char *val));
+extern int  decode_arst_direct  A_((attribute *patr, char *val));
 extern int  decode_resc  A_((attribute *patr,char *name, char *rn, char *val));
 extern int  decode_depend A_((attribute *patr,char *name, char *rn, char *val));
 extern int  decode_hold A_((attribute *patr,char *name, char *rn, char *val));
@@ -437,6 +439,10 @@ extern int      node_status_list A_(( attribute*, void*, int));
 extern int      node_note        A_(( attribute*, void*, int));
 extern int      set_note_str     A_((attribute *attr,attribute *new, enum batch_op));
 
+/* Token manipulation functions */
+
+extern int  decode_tokens A_((attribute *,char *, char *, char *));
+
 /* "type" to pass to acl_check() */
 #define ACL_Host  1
 #define ACL_User  2
