@@ -89,6 +89,10 @@
 
 #ifndef JOB_H
 #define JOB_H 1
+
+/* anything including job.h also needs array.h so lets just include it this way*/
+#include "array.h"
+
 /*
  * Dependent Job Structures
  *
@@ -102,12 +106,12 @@
  */
 
 struct depend {
-	tlist_link dp_link;	/* link to next dependency, if any       */
-	short	  dp_type;	/* type of dependency (all) 	         */
-	short	  dp_numexp;	/* num jobs expected (on or syncct only) */
-	short	  dp_numreg;	/* num jobs registered (syncct only)     */
-	short	  dp_released;	/* This job released to run (syncwith)   */
-	tlist_head dp_jobs;	/* list of related jobs  (all)           */
+  list_link dp_link;	/* link to next dependency, if any       */
+  short	  dp_type;	/* type of dependency (all) 	         */
+  short	  dp_numexp;	/* num jobs expected (on or syncct only) */
+  short	  dp_numreg;	/* num jobs registered (syncct only)     */
+  short	  dp_released;	/* This job released to run (syncwith)   */
+  tlist_head dp_jobs;	/* list of related jobs  (all)           */
 };
 
 /*
@@ -116,7 +120,7 @@ struct depend {
  */
 
 struct depend_job {
-	tlist_link dc_link;
+	list_link dc_link;
 	short	dc_state;	/* released / ready to run (syncct)	 */
 	long	dc_cost;	/* cost of this child (syncct)		 */
 	char	dc_child[PBS_MAXSVRJOBID+1]; /* child (dependent) job	 */
@@ -152,7 +156,7 @@ struct depend_job {
  * status back, see svr_movejob.c.
  */
 typedef struct	badplace {
-  tlist_link	bp_link;
+  list_link	bp_link;
   char		bp_dest[PBS_MAXROUTEDEST + 1];
 } badplace;
 
@@ -204,64 +208,70 @@ struct grpcache {
 /* sync w/XXX */
 
 enum job_atr {
-	JOB_ATR_jobname,	/* this set appears first as they show */
-	JOB_ATR_job_owner,	/* in a basic job status display       */
-	JOB_ATR_resc_used,
-	JOB_ATR_state,
-	JOB_ATR_in_queue,
-	JOB_ATR_at_server,
+  JOB_ATR_jobname,	/* this set appears first as they show */
+  JOB_ATR_job_owner,	/* in a basic job status display       */
+  JOB_ATR_resc_used,
+  JOB_ATR_state,
+  JOB_ATR_in_queue,
+  JOB_ATR_at_server,    /* (5) */
 
-	JOB_ATR_account,	/* the bulk of the attributes are in   */
-	JOB_ATR_chkpnt,		/* alphabetic order for no good reason */
-	JOB_ATR_ctime,
-	JOB_ATR_depend,
-	JOB_ATR_errpath,
-	JOB_ATR_exec_host,
-	JOB_ATR_exectime,
-	JOB_ATR_grouplst,
-	JOB_ATR_hold,
-	JOB_ATR_interactive,
-	JOB_ATR_join,
-	JOB_ATR_keep,
-	JOB_ATR_mailpnts,
-	JOB_ATR_mailuser,
-	JOB_ATR_mtime,
-	JOB_ATR_outpath,
-	JOB_ATR_priority,
-	JOB_ATR_qtime,
-	JOB_ATR_rerunable,
-	JOB_ATR_resource,
-	JOB_ATR_session_id,
-	JOB_ATR_shell,
-	JOB_ATR_stagein,
-	JOB_ATR_stageout,
-	JOB_ATR_substate,
-	JOB_ATR_userlst,
-	JOB_ATR_variables,
-				/* this set contains private attributes,  */
-				/* as such not sent to clients (status)   */
+  JOB_ATR_account,	/* the bulk of the attributes are in   */
+  JOB_ATR_chkpnt,	/* alphabetic order for no good reason */
+  JOB_ATR_ctime,
+  JOB_ATR_depend,
+  JOB_ATR_errpath,
+  JOB_ATR_exec_host,
+  JOB_ATR_exectime,
+  JOB_ATR_grouplst,
+  JOB_ATR_hold,
+  JOB_ATR_interactive,
+  JOB_ATR_join,
+  JOB_ATR_keep,
+  JOB_ATR_mailpnts,
+  JOB_ATR_mailuser,
+  JOB_ATR_mtime,         /* (20) */
+  JOB_ATR_outpath,
+  JOB_ATR_priority,
+  JOB_ATR_qtime,
+  JOB_ATR_rerunable,
+  JOB_ATR_resource,
+  JOB_ATR_session_id,
+  JOB_ATR_shell,
+  JOB_ATR_stagein,
+  JOB_ATR_stageout,
+  JOB_ATR_substate,
+  JOB_ATR_userlst,
+  JOB_ATR_variables,    /* (32) */
+			/* this set contains private attributes,  */
+			/* as such not sent to clients (status)   */
 
-	JOB_ATR_euser,		/* execution user name for MOM		  */
-	JOB_ATR_egroup,		/* execution group name for MOM		  */
-	JOB_ATR_hashname,	/* job name hashed into 14 characters	  */
-	JOB_ATR_hopcount,
-	JOB_ATR_qrank,
-	JOB_ATR_queuetype,
-	JOB_ATR_sched_hint,     /* 39 */
-	JOB_ATR_security,	
-	JOB_ATR_Comment,
-	JOB_ATR_Cookie,
-	JOB_ATR_altid,		/* alternate job id, for irix6 = array id */
-	JOB_ATR_etime,		/* time job became eligible to run	  */
-	JOB_ATR_exitstat,	/* exit status of job			  */
-	JOB_ATR_forwardx11,
-	JOB_ATR_submit_args,
-        JOB_ATR_umask,
+  JOB_ATR_euser,	/* execution user name for MOM		  */
+  JOB_ATR_egroup,	/* execution group name for MOM		  */
+  JOB_ATR_hashname,	/* job name hashed into 14 characters	  */
+  JOB_ATR_hopcount,
+  JOB_ATR_qrank,
+  JOB_ATR_queuetype,
+  JOB_ATR_sched_hint,   /* 39 */
+  JOB_ATR_security,	
+  JOB_ATR_Comment,
+  JOB_ATR_Cookie,
+  JOB_ATR_altid,	/* alternate job id, for irix6 = array id */
+  JOB_ATR_etime,	/* time job became eligible to run	  */
+  JOB_ATR_exitstat,	/* exit status of job			  */
+  JOB_ATR_forwardx11,
+  JOB_ATR_submit_args,
+  JOB_ATR_job_array_id,
+  JOB_ATR_job_array_request,
+  JOB_ATR_umask,
+  JOB_ATR_start_time,  /* time when job was first started */
+  JOB_ATR_start_count, /* number of times the job has been started */
+  JOB_ATR_chkptdir,    /* directory where job checkpoint file is stored */
+  JOB_ATR_chkptname,   /* name of checkpoint file */
 #include "site_job_attr_enum.h"
 
-	JOB_ATR_UNKN,		/* the special "unknown" type		  */
-	JOB_ATR_LAST		/* This MUST be LAST	*/
-};
+  JOB_ATR_UNKN,		/* the special "unknown" type		  */
+  JOB_ATR_LAST		/* This MUST be LAST	*/
+  };
 
 /*
  * The "definitions" for the job attributes are in the following array,
@@ -279,7 +289,7 @@ extern attribute_def job_attr_def[];
 */
 typedef	struct	hnodent {
 	int		hn_node;	/* host (node) identifier (index) */
-	char	       *hn_host;	/* hostname of node */
+	char	*hn_host;	/* hostname of node */
 	int		hn_stream;	/* stream to MOM on node */
 	int		hn_sister;	/* save error for KILL_JOB event */
 	tlist_head	hn_events;	/* pointer to list of events */
@@ -374,6 +384,10 @@ typedef struct {
   int       mjspipe[2];     /* MOM to job starter for ack */
   int       downfds;
   } pjobexec_t;
+  
+  
+  
+  
 
 
 
@@ -382,8 +396,8 @@ struct job {
 	/* Note: these members, up to ji_qs, are not saved to disk
            (except for ji_stdout, ji_stderr) */
 
-	tlist_link       ji_alljobs;	/* links to all jobs in server */
-	tlist_link       ji_jobque;	/* SVR: links to jobs in same queue */
+	list_link       ji_alljobs;	/* links to all jobs in server */
+	list_link       ji_jobque;	/* SVR: links to jobs in same queue */
 					/* MOM: links to polled jobs */
 	time_t		ji_momstat;	/* SVR: time of last status from MOM */
 					/* MOM: time job suspend (Cray)	*/
@@ -395,7 +409,7 @@ struct job {
 	time_t		ji_chkptnext;	/* next checkpoint time */
 	time_t		ji_sampletim;	/* last usage sample time, irix only */
 	pid_t		ji_momsubt;	/* pid of mom subtask   */
-	void	      (*ji_mompost)();	/* ptr to post processing func  */
+	int	      (*ji_mompost)();	/* ptr to post processing func  */
 	struct batch_request *ji_preq;	/* hold request until finish_exec */
 	int		ji_numnodes;	/* number of nodes (at least 1) */
 	int		ji_numvnod;	/* number of virtual nodes */
@@ -417,6 +431,9 @@ struct job {
 	int		ji_lastdest;	/* last destin tried by route */
 	int		ji_retryok;	/* ok to retry, some reject was temp */
 	tlist_head	ji_rejectdest;	/* list of rejected destinations */
+	list_link	ji_arrayjobs;	/* links to all jobs in same array */
+	job_array	*ji_arraystruct;/* pointer to job_array for this array */
+        int		ji_isparent;    /* set to TRUE if this is a "parent job"*/
 #endif					/* END SERVER ONLY */
 
 	/*
@@ -426,6 +443,7 @@ struct job {
 	 */
 
 	struct jobfix {
+	    int     qs_version;		/* quick save version */
 	    int	    ji_state;		/* internal copy of state */
 	    int	    ji_substate;	/* job sub-state */
 	    int	    ji_svrflags;	/* server flags */
@@ -471,6 +489,7 @@ struct job {
 
 typedef struct job job;
 
+
 #ifdef	PBS_MOM
 /*
 **	Tasks are sessions belonging to a job, running on one of the
@@ -478,7 +497,7 @@ typedef struct job job;
 */
 typedef struct	task {
 	job		*ti_job;	/* pointer to owning job */
-	tlist_link	ti_jobtask;	/* links to tasks for this job */
+	list_link	ti_jobtask;	/* links to tasks for this job */
 	int		ti_fd;		/* DIS file descriptor to task */
 	int		ti_flags;	/* task internal flags */
 	tm_event_t	ti_register;	/* event if task registers */
@@ -498,15 +517,17 @@ typedef struct	task {
 	} ti_qs;
 } task;
 
+
+
 /*
 **	Events need to be linked to either a task or another event
 **	waiting at another MOM.  This is the information needed so
 **	we can forward the event to another MOM.
 */
-typedef struct	fwdevent {
-	tm_node_id	fe_node;	/* where does notification go */
-	tm_event_t	fe_event;	/* event number */
-	tm_task_id	fe_taskid;	/* which task id */
+typedef struct fwdevent {
+  tm_node_id fe_node;	/* where does notification go */
+  tm_event_t fe_event;	/* event number */
+  tm_task_id fe_taskid;	/* which task id */
 } fwdevent;
 
 /*
@@ -520,7 +541,7 @@ typedef struct	eventent {
 	fwdevent	ee_forward;	/* event to get notified */
 	char		**ee_argv;	/* save args for spawn */
 	char		**ee_envp;	/* save env for spawn */
-	tlist_link	ee_next;	/* link to next one */
+	list_link	ee_next;	/* link to next one */
 } eventent;
 
 /*
@@ -529,7 +550,7 @@ typedef struct	eventent {
 */
 typedef struct	obitent {
 	fwdevent	oe_info;	/* who gets the event */
-	tlist_link	oe_next;	/* link to next one */
+	list_link	oe_next;	/* link to next one */
 } obitent;
 
 /*
@@ -540,7 +561,7 @@ typedef struct	infoent {
   char		*ie_name;	/* published name */
   void		*ie_info;	/* the glop */
   size_t	ie_len;		/* how much glop */
-  tlist_link	ie_next;	/* link to next one */
+  list_link	ie_next;	/* link to next one */
   } infoent;
 
 #define	TI_FLAGS_INIT		1		/* task has called tm_init */
@@ -573,16 +594,19 @@ typedef struct	infoent {
 
 #define IM_MAX          12
 
-eventent	*event_alloc	A_((	int		command,
-					hnodent		*pnode,
-					tm_event_t	event,
-					tm_task_id	taskid));
+eventent *event_alloc A_((
+  int		command,
+  hnodent	*pnode,
+  tm_event_t	event,
+  tm_task_id	taskid));
 
-task		*pbs_task_create	A_((	job		*pjob,
-					tm_task_id	taskid));
+task *pbs_task_create A_((	
+  job *pjob,
+  tm_task_id	taskid));
 
-task		*task_find	A_((	job		*pjob,
-					tm_task_id	taskid));
+task *task_find A_((
+  job *pjob,
+  tm_task_id	taskid));
 
 #endif	/* MOM */
 
@@ -621,15 +645,15 @@ task		*task_find	A_((	job		*pjob,
 #define MAIL_NORMAL 0
 #define MAIL_FORCE  1
 
-#define JOB_FILE_COPY      ".JC"	/* tmp copy while updating */
-#define JOB_FILE_SUFFIX    ".JB"	/* job control file */
-#define JOB_SCRIPT_SUFFIX  ".SC"	/* job script file  */
-#define JOB_STDOUT_SUFFIX  ".OU"	/* job standard out */
-#define JOB_STDERR_SUFFIX  ".ER"	/* job standard error */
-#define JOB_CKPT_SUFFIX    ".CK"	/* job checkpoint file */
-#define JOB_TASKDIR_SUFFIX ".TK"	/* job task directory */
-#define JOB_BAD_SUFFIX	   ".BD"	/* save bad job file */
-
+#define JOB_FILE_COPY       ".JC"	/* tmp copy while updating */
+#define JOB_FILE_SUFFIX     ".JB"	/* job control file */
+#define JOB_SCRIPT_SUFFIX   ".SC"	/* job script file  */
+#define JOB_STDOUT_SUFFIX   ".OU"	/* job standard out */
+#define JOB_STDERR_SUFFIX   ".ER"	/* job standard error */
+#define JOB_CKPT_SUFFIX     ".CK"	/* job checkpoint file */
+#define JOB_TASKDIR_SUFFIX  ".TK"	/* job task directory */
+#define JOB_BAD_SUFFIX	    ".BD"	/* save bad job file */
+#define JOB_FILE_TMP_SUFFIX ".TA"	/* temporary job array parent file suffix */
 /*
  * Job states are defined by POSIX as:
  */
@@ -681,6 +705,7 @@ task		*task_find	A_((	job		*pjob,
 #define JOB_SUBSTATE_STAGEDEL	52	/* job deleteing staged out files  */
 #define JOB_SUBSTATE_EXITED	53	/* job exit processing completed   */
 #define JOB_SUBSTATE_ABORT      54	/* job is being aborted by server  */
+#define JOB_SUBSTATE_PREOBIT    57	/* (MOM) preobit jobstat sent */
 #define JOB_SUBSTATE_OBIT       58	/* (MOM) job obit notice sent */
 #define JOB_SUBSTATE_COMPLETE   59	/* job is complete */
 
@@ -702,6 +727,9 @@ task		*task_find	A_((	job		*pjob,
 #define HOLD_u 1
 #define HOLD_o 2
 #define HOLD_s 4
+/* jobs in a job array are held until the whole array is ready for now this is 
+ * a system hold, but it may be a special hold in the future */
+#define HOLD_a HOLD_s  
 
 /* Special Job Exit Values,  Set by the job starter (child of MOM)   */
 /* see server/req_jobobit() & mom/start_exec.c			     */
@@ -728,14 +756,15 @@ extern int   init_chkmom A_((job *));
 extern void  issue_track A_((job *));
 extern int   job_abt A_((job **,char *));
 extern job  *job_alloc();
+extern job  *job_clone A_((job *,int));
 extern void  job_free A_((job *));
 extern void  job_purge A_((job *));
 extern job  *job_recov A_((char *));
 extern int   job_save A_((job *,int));
 extern int   modify_job_attr A_((job *,svrattrl *,int,int *));
 extern char *prefix_std_file A_((job *,int));
-extern int   set_jobexid A_((job *,attribute *));
-extern int   site_check_user_map A_((job *,char *));
+extern int   set_jobexid A_((job *,attribute *,char *));
+extern int   site_check_user_map A_((job *,char *,char *));
 extern void  svr_dequejob A_((job *));
 extern int   svr_enquejob A_((job *));
 extern void  svr_evaljobstate A_((job *,int *,int *,int));
@@ -759,6 +788,7 @@ extern int   site_acl_check A_((job *,pbs_queue *));
 #endif	/* QUEUE_H */
 
 #ifdef WORK_TASK_H
+extern void  job_clone_wt A_((struct work_task *));
 extern int   issue_signal A_((job *,char *,void(*)(struct work_task *),void *));
 extern void   on_job_exit A_((struct work_task *));
 #endif	/* WORK_TASK_H */
