@@ -78,7 +78,7 @@
 */
 
 /*
- * @(#) $Id: libpbs.h 1484 2007-07-18 22:09:16Z garrick $
+ * @(#) $Id: libpbs.h 1735 2008-01-12 02:01:16Z garrick $
  */
 
 /*	libpbs.h
@@ -156,7 +156,7 @@ struct brp_select {		/* reply to Select Job Request */
 };
 
 struct brp_status {		/* reply to Status Job/Queue/Server Request */
-	tlist_link brp_stlink;
+	list_link brp_stlink;
 	int	  brp_objtype;
 	char	  brp_objname[(PBS_MAXSVRJOBID > PBS_MAXDEST ? PBS_MAXSVRJOBID:PBS_MAXDEST)+1];
 	tlist_head brp_attr;		/* head of svrattrlist */
@@ -251,18 +251,17 @@ enum PBatchReqTypeEnum {
   PBS_BATCH_MvJobFile	= 57,
   PBS_BATCH_StatusNode	= 58,
   PBS_BATCH_Disconnect	= 59 };
-
+/* DIAGTODO: define PBS_BATCH_StatusDiag */
 #define PBS_BATCH_FileOpt_Default	0
 #define PBS_BATCH_FileOpt_OFlg		1
 #define PBS_BATCH_FileOpt_EFlg		2
 
 #define PBS_credentialtype_none 0
+const char *reqtype_to_txt(int);
 
 void
 PBS_free_aopl A_(( struct attropl * aoplp ));
 
-void
-advise ();
 
 int
 PBSD_commit A_((int connect, char *jobid));
