@@ -112,6 +112,17 @@ struct cphosts {
 };
 extern int cphosts_num;
 
+/* holds the varattrs */
+
+struct varattr {
+	list_link   va_link;
+	char       *va_name;
+        int         va_ttl;
+        char       *va_cmd;
+        char       *va_value;
+        time_t      va_lasttime;
+};
+
 /* public funtions within MOM */
 
 #ifdef _CRAY
@@ -127,7 +138,7 @@ extern pid_t fork_me A_((int sock));
 extern int   get_la A_((double *));
 extern void  init_abort_jobs A_((int));
 extern int   init_groups A_((char *, int, int, int *));
-extern int   kill_job A_((job *, int sig));
+extern int   kill_job A_((job *, int sig, char *killer_id_name, char *why_killed_reason));
 extern void  mom_deljob A_((job *));
 extern void  mom_freenodes A_((job *));
 extern void  scan_for_exiting();
@@ -151,19 +162,21 @@ extern int   site_mom_postchk A_((job *, int));
 extern int   site_mom_prerst A_((job *));
 extern int   reader A_((int, int));
 extern int   writer A_((int, int));
-extern int   conn_qsub A_((char *,unsigned int));
+extern int   conn_qsub A_((char *,unsigned int,char *));  /* NOTE:  should be moved out of here to job_func proto header */
 extern int   run_pelog A_((int, char *,job *,int));
 extern int   is_joined A_((job *));
 extern void  check_busy A_((double));
 extern void  state_to_server A_((int,int));
-extern void end_proc(void);
-extern int message_job(job *,enum job_file,char *);
-extern void term_job(job *);
+extern void  dep_main_loop_cycle(void);
+extern int   message_job(job *,enum job_file,char *);
+extern void  term_job(job *);
 extern struct passwd *check_pwd(job *);
-extern int send_sisters(job *,int);
-extern int task_save(task *) ;
+extern int   send_sisters(job *,int);
+extern int   task_save(task *) ;
 extern void DIS_rpp_reset A_((void));
-extern void checkret A_((char **,int));
+extern void  checkret A_((char **,int));
+extern char *get_job_envvar A_((job *,char *));
+extern int   mom_open_socket_to_jobs_server A_((job* pjob, char *id, void (*message_hander) A_((int))));
 
 /* defined in mach-dependant mom_mach.c */
 extern int kill_task A_((struct task *,int,int));
