@@ -118,7 +118,7 @@
 #define PBSE_QUENBIG	15028		/* Queue name too long */
 #define PBSE_NOSUP	15029		/* Feature/function not supported */
 #define PBSE_QUENOEN	15030		/* Cannot enable queue,needs add def */
-#define PBSE_PROTOCOL	15031		/* Protocol (ASN.1) error */
+#define PBSE_PROTOCOL	15031		/* Protocol error */
 #define PBSE_BADATLST	15032		/* Bad attribute list structure */
 #define PBSE_NOCONNECTS	15033		/* No free connections */
 #define PBSE_NOSERVER	15034		/* No server to connect to */
@@ -161,13 +161,15 @@
 #define PBSE_NOTSNODE	15071		/* no time-shared nodes */
 #define PBSE_JOBTYPE	15072		/* wrong job type (batch or interactive) */
 #define PBSE_BADACLHOST	15073		/* bad entry in ACL host list */
-#define PBSE_MAXUSERQUED	15074	/* max number of jobs queued for user */
-#define PBSE_BADDISALLOWTYPE	15075	/* bad type in disallowed_types queue attribute */
-#define PBSE_NOINTERACTIVE	15076	/* interactive jobs not allowed in queue */
-#define PBSE_NOBATCH	15077		/* batch jobs not allowed in queue */
-#define PBSE_NORERUNABLE        15078	/* rerunable jobs not allowed in queue */
-#define PBSE_NONONRERUNABLE	15079	/* nonrerunable jobs not allowed in queue */
-
+#define PBSE_MAXUSERQUED     15074	/* max number of jobs queued for user */
+#define PBSE_BADDISALLOWTYPE 15075	/* bad type in disallowed_types queue attribute */
+#define PBSE_NOINTERACTIVE   15076	/* interactive jobs not allowed in queue */
+#define PBSE_NOBATCH    15077		/* batch jobs not allowed in queue */ 
+#define PBSE_NORERUNABLE     15078	/* rerunable jobs not allowed in queue */
+#define PBSE_NONONRERUNABLE  15079	/* nonrerunable jobs not allowed in queue */
+#define PBSE_UNKARRAYID 15080		/* unknown array id */
+#define PBSE_BAD_ARRAY_REQ   15081      /* bad array request */
+#define PBSE_TIMEOUT	15082		/* Time out */
 /*
 ** 	Resource monitor specific
 */
