@@ -32,10 +32,10 @@ struct pfwdsock {
   char buff[BUF_SIZE];
 };
 
-int port_forwarder(struct pfwdsock *,int(*connfunc)(char *phost,int pport),char*,int);
+void port_forwarder(struct pfwdsock *,int(*connfunc)(char *phost,int pport,char *),char*,int,char *);
 void set_nodelay(int);
 int connect_local_xsocket(u_int);
-int x11_connect_display(char *,int);
+int x11_connect_display(char *,int,char *);
 
 
 #endif /* END PORT_FORWARDING_H */
