@@ -107,6 +107,7 @@ ATTR_maxrun,
 ATTR_maxuserrun,
 ATTR_maxgrprun,
 ATTR_nodepack,
+ATTR_nodesuffix,
 ATTR_operators,
 ATTR_queryother,
 ATTR_rescavail,
@@ -122,20 +123,24 @@ ATTR_tcptimeout,
 ATTR_jobstatrate,
 ATTR_polljobs,
 ATTR_downonerror,
+ATTR_disableserveridcheck,
 ATTR_jobnanny,
 ATTR_ownerpurge,
 ATTR_qcqlimits,
 ATTR_momjobsync,
 ATTR_maildomain,
-ATTR_pbsversion,
 ATTR_killdelay,
 ATTR_acllogic,
 ATTR_aclgrpslpy,
 ATTR_keepcompleted,
 ATTR_submithosts,
 ATTR_allownodesubmit,
+ATTR_allowproxyuser,
 ATTR_servername,
 ATTR_autonodenp,
 ATTR_logfilemaxsize,
 ATTR_logfilerolldepth,
 ATTR_nextjobnum,
+ATTR_tokens,
+ATTR_extraresc,
+ATTR_schedversion,
