@@ -172,7 +172,7 @@ char *msg_svrdown	= "Server shutdown completed";
 
 /* sync w/enum PBatchReqTypeEnum in libpbs.h */
 
-const char *PBatchReqType[] = {
+static const char *PBatchReqType[] = {
   "Connect",
   "QueueJob",
   "JobCred",
@@ -235,6 +235,8 @@ const char *PBatchReqType[] = {
   "Disconnect",
   NULL };
 
+#define NPBatchReqType (sizeof(PBatchReqType)/sizeof(PBatchReqType[0]))
+
 /*
  * This next set of messages are returned to the client on an error.
  * They may also be logged.
@@ -326,10 +328,11 @@ char *msg_badaclhost	= "Bad ACL entry in host list";
 char *msg_baddisallowtype = "Bad type in disallowed_types list";
 char *msg_nointeractive = "Queue does not allow interactive jobs";
 char *msg_nobatch       = "Queue does not allow batch jobs";
-char *msg_norerunable   = "Queue does not allow rerunable jobs"; 
+char *msg_norerunable   = "Queue does not allow rerunable jobs";
 char *msg_nononrerunable = "Queue does not allow nonrerunable jobs";
-
-
+char *msg_unkarrayid    = "Unknown Array ID";
+char *msg_bad_array_req = "Bad Job Array Request";
+char *msg_timeout 	= "Time out";
 /*
  * The following table connects error numbers with text
  * to be returned to the client.  Each is guaranteed to be pure text.
@@ -419,8 +422,11 @@ struct pbs_err_to_txt pbs_err_to_txt[] = {
         { PBSE_BADDISALLOWTYPE, &msg_baddisallowtype },
         { PBSE_NOINTERACTIVE, &msg_nointeractive },
         { PBSE_NOBATCH, &msg_nobatch },
-        { PBSE_NORERUNABLE, &msg_norerunable },
+	{ PBSE_NORERUNABLE, &msg_norerunable },
 	{ PBSE_NONONRERUNABLE, &msg_nononrerunable },
+	{ PBSE_UNKARRAYID, &msg_unkarrayid },
+	{ PBSE_BAD_ARRAY_REQ, &msg_bad_array_req },
+	{ PBSE_TIMEOUT, &msg_timeout },
         { PBSE_NONE, &msg_none },
 	{ 0, (char **)0 }		/* MUST be the last entry */
 };
@@ -453,4 +459,17 @@ char *pbse_to_txt(
   }  /* END pbse_to_txt() */
 
 
+/*
+ * rqtype_to_txt()	- Return the printable name of a request type
+ */
 
+const char *reqtype_to_txt(
+
+  int reqtype)
+
+  {
+  if (reqtype >= 0 && reqtype < (int)NPBatchReqType && PBatchReqType[reqtype])
+    return(PBatchReqType[reqtype]);
+  else
+    return("NONE");
+  }  /* END reqtype_to_txt() */
