@@ -80,6 +80,7 @@
 
 #include <sys/types.h>
 #include <sys/socket.h>
+#include <rpc/rpc.h>
 #include <netinet/in.h>
 #include <stdio.h>
 #include <errno.h>
@@ -95,6 +96,13 @@
 #include <sys/time.h>
 
 
+#ifdef __APPLE__
+/* this is a hack for the missing bindresvport declaration on OS X 
+   the function works fine but its use will generate a compiler warning 
+   if -Wall is used with gcc */
+int bindresvport(int sd, struct sockaddr_in *sin);
+#endif
+
 /*
 ** wait for connect to complete.  We use non-blocking sockets,
 ** so have to wait for completion this way.
@@ -146,6 +154,8 @@ static int await_connect(
 
 
 
+#define TORQUE_MAXCONNECTTIMEOUT  5
+
 /*
  * client_to_svr - connect to a server
  *
@@ -155,30 +165,44 @@ static int await_connect(
  *		 PBS_NET_RC_FATAL (-1) if fatal error, just quit, or
  *		 PBS_NET_RC_RETRY (-2) if temp error, should retry
  *
- * Note, the server's host address and port are was chosen as parameters
+ * NOTE: the server's host address and port were chosen as parameters
  * rather than their names to possibly save extra look-ups.  It seems likely
  * that the caller "might" make several calls to the same host or different
- * hosts with the same port.  Let the caller keep the addresses arround
+ * hosts with the same port.  Let the caller keep the addresses around
  * rather than look it up each time.
+ *
+ * NOTE:  will wait up to TORQUE_MAXCONNECTTIMEOUT seconds for transient network failures
  */
 
+/* NOTE:  create new connection on reserved port to validate root/trusted authority */
+
 int client_to_svr(
 
-  pbs_net_t     hostaddr,	/* Internet addr  of host */
-  unsigned int  port,		/* port to which to connect */
-  int           local_port)	/* BOOLEAN:  not 0 if use local reserved port */
+  pbs_net_t     hostaddr,	/* I - internet addr of host */
+  unsigned int  port,		/* I - port to which to connect */
+  int           local_port,	/* I - BOOLEAN:  not 0 to use local reserved port */
+  char         *EMsg)           /* O (optional,minsize=1024) */
 
   {
+  const char id[] = "client_to_svr";
+
   struct sockaddr_in local;
   struct sockaddr_in remote;
   int                sock;
   unsigned short     tryport;
   int                flags;
-  int               one = 1;
-  
-  local.sin_family = AF_INET;
+  int                one = 1;
+
+  int                trycount;
+ 
+  if (EMsg != NULL)
+    EMsg[0] = '\0';
+
+  errno = 0;
+ 
+  local.sin_family      = AF_INET;
   local.sin_addr.s_addr = 0;
-  local.sin_port = 0;
+  local.sin_port        = 0;
 
   tryport = IPPORT_RESERVED - 1;
 
@@ -190,27 +214,38 @@ retry:  /* retry goto added (rentec) */
 
   if (sock < 0) 
     {
+    if (EMsg != NULL)
+      sprintf(EMsg,"cannot create socket in %s - errno: %d %s",
+        id,
+        errno,
+        strerror(errno));
+
     return(PBS_NET_RC_FATAL);
     } 
 
   if (sock >= PBS_NET_MAX_CONNECTIONS) 
     {
+    if (EMsg != NULL)
+      sprintf(EMsg,"PBS_NET_MAX_CONNECTIONS exceeded in %s",
+        id);
+
     close(sock);		/* too many connections */
 
     return(PBS_NET_RC_RETRY);
     }
 
 #ifndef NOPRIVPORTS
-
   flags = fcntl(sock,F_GETFL);
   flags |= O_NONBLOCK;
-  fcntl(sock,F_SETFL,flags);
 
+  fcntl(sock,F_SETFL,flags);
 #endif /* !NOPRIVPORTS */
 
   /* If local privilege port requested, bind to one */
   /* must be root privileged to do this	*/
 
+  trycount = 0;
+
   if (local_port != FALSE) 
     {
     /* set REUSEADDR (rentec) */
@@ -228,7 +263,7 @@ retry:  /* retry goto added (rentec) */
     /*
      * bindresvport seems to cause connect() failures in some odd corner case when
      * talking to a local daemon.  So we'll only try this once and fallback to
-     * the slow loop around bind() if connect() failes with EADDRINUSE
+     * the slow loop around bind() if connect() fails with EADDRINUSE
      * or EADDRNOTAVAIL.
      * http://www.supercluster.org/pipermail/torqueusers/2006-June/003740.html
      */
@@ -237,6 +272,10 @@ retry:  /* retry goto added (rentec) */
       {
       if (bindresvport(sock,&local) < 0)
         {
+        if (EMsg != NULL)
+          sprintf(EMsg,"cannot bind to reserved port in %s",
+            id);
+
         close(sock);
 
         return(PBS_NET_RC_FATAL);
@@ -244,41 +283,56 @@ retry:  /* retry goto added (rentec) */
       }
     else
       {
-
-#endif
-
-    local.sin_port = htons(tryport);
-
-    while (bind(sock,(struct sockaddr *)&local,sizeof(local)) < 0) 
-      {
-#ifdef NDEBUG2
-      fprintf(stderr,"INFO:  cannot bind to port %d, errno: %d - %s\n",
-        tryport,
-        errno,
-        strerror(errno));
-#endif /* NDEBUG2 */
-
-      if ((errno != EADDRINUSE) && (errno != EADDRNOTAVAIL)) 
-        {
-        close(sock);
-
-        return(PBS_NET_RC_FATAL);
-        } 
-   
-      if (--tryport < (unsigned short)(IPPORT_RESERVED / 2)) 
-        {
-        close(sock);
-
-        return(PBS_NET_RC_RETRY);
-        }
+#endif /* HAVE_BINDRESVPORT */
 
       local.sin_port = htons(tryport);
-      }  /* END while (bind() < 0) */
+
+      while (bind(sock,(struct sockaddr *)&local,sizeof(local)) < 0) 
+        {
+#ifdef NDEBUG2
+        fprintf(stderr,"INFO:  cannot bind to port %d, errno: %d - %s\n",
+          tryport,
+          errno,
+          strerror(errno));
+#endif /* NDEBUG2 */
+
+        if ((errno != EADDRINUSE) && (errno != EADDRNOTAVAIL)) 
+          {
+          if (EMsg != NULL)
+            sprintf(EMsg,"cannot bind to port %d in %s - errno: %d %s",
+              tryport,
+              id,
+              errno,
+              strerror(errno));
+
+          close(sock);
+
+          return(PBS_NET_RC_FATAL);
+          } 
+
+        trycount++;
+   
+        if (--tryport < (unsigned short)(IPPORT_RESERVED / 2)) 
+          {
+          if (EMsg != NULL)
+            sprintf(EMsg,"cannot bind to port %d in %s - too many retries",
+              tryport,
+              id);
+
+          close(sock);
+
+          return(PBS_NET_RC_RETRY);
+          }
+
+        local.sin_port = htons(tryport);
+        }  /* END while (bind() < 0) */
 #ifdef HAVE_BINDRESVPORT
-      } /* END if (tryport == (IPPORT_RESERVED - 1)) else */
-#endif /* HAVE_BINDRESVPORT */
-#endif /* !NOPRIVPORTS */
-    }    /* END if (local_port != FALSE) */
+      }    /* END if (tryport == (IPPORT_RESERVED - 1)) else */
+#endif     /* HAVE_BINDRESVPORT */
+#endif     /* !NOPRIVPORTS */
+    }      /* END if (local_port != FALSE) */
+
+  /* bind successful!!! */
 			
   /* connect to specified server host and port	*/
 
@@ -318,7 +372,7 @@ retry:  /* retry goto added (rentec) */
 
         goto retry;
         }
-#endif
+#endif /* NOPRIVPORTS */
 
       /* fall through to next case */
 
@@ -326,9 +380,9 @@ retry:  /* retry goto added (rentec) */
     case ETIMEDOUT:
     case EINPROGRESS:   
 
-      if (await_connect(5,sock) == 0)
+      if (await_connect(TORQUE_MAXCONNECTTIMEOUT,sock) == 0)
         {
-        /* socket not ready for writing after 5 second timeout */
+        /* socket not ready for writing after TORQUE_MAXCONNECTTIMEOUT second timeout */
         /* no network failures detected */
 
         break;
@@ -338,6 +392,11 @@ retry:  /* retry goto added (rentec) */
 
     case ECONNREFUSED:
 
+      if (EMsg != NULL)
+        sprintf(EMsg,"cannot bind to port %d in %s - connection refused",
+          tryport,
+          id);
+
       close(sock);
 
       return(PBS_NET_RC_RETRY);
@@ -348,6 +407,13 @@ retry:  /* retry goto added (rentec) */
 
     default:
 
+      if (EMsg != NULL)
+        sprintf(EMsg,"cannot bind to port %d in %s - errno:%d %s",
+          tryport,
+          id,
+          errno,
+          strerror(errno));
+
       close(sock);
 
       return(PBS_NET_RC_FATAL);
@@ -355,7 +421,7 @@ retry:  /* retry goto added (rentec) */
       /*NOTREACHED*/
 
       break;
-    }  /* END switch(errno) */
+    }  /* END switch (errno) */
 
   /* SUCCESS */
 			
