@@ -80,6 +80,7 @@
 
 #include <sys/types.h>
 #include <unistd.h>
+#include <stdio.h>
 #include <string.h>
 #include <netdb.h>
 #include "portability.h"
@@ -117,8 +118,9 @@ extern char *msg_orighost;	/* error message: no PBS_O_HOST */
 
 int site_check_user_map(
 
-  job  *pjob,	
-  char *luser)
+  job  *pjob,  /* I */	
+  char *luser, /* I */
+  char *EMsg)  /* O (optional,minsize=1024) */
 
   {
   char *orighost;
@@ -127,8 +129,15 @@ int site_check_user_map(
   char *p2;
   int   rc;
  
+  int   ProxyAllowed = 0;
+  int   ProxyRequested = 0;
+  int   HostAllowed = 0;
+
   char  *dptr;
 
+  if (EMsg != NULL)
+    EMsg[0] = '\0';
+
   /* get just the owner name, without the "@host" */
 	
   p1 = pjob->ji_wattr[JOB_ATR_job_owner].at_val.at_str;
@@ -151,14 +160,35 @@ int site_check_user_map(
       pjob->ji_qs.ji_jobid, 
       msg_orighost);
 
+    if (EMsg != NULL)
+      strcpy(EMsg,"source host not specified");
+
     return(-1);
     }
 
+  if ((server.sv_attr[(int)SRV_ATR_AllowProxyUser].at_flags & ATR_VFLAG_SET) && \
+      (server.sv_attr[(int)SRV_ATR_AllowProxyUser].at_val.at_long == 1))
+    {
+    ProxyAllowed = 1;
+    }
+
+  if (strcmp(owner,luser) != 0)
+    {
+    ProxyRequested = 1;
+    }
+
   if (!strcmp(orighost,server_host) && !strcmp(owner,luser))
     {
     /* submitting from server host, access allowed */
 
-    return(0);
+    if ((ProxyRequested == 0) || (ProxyAllowed == 1))
+      {
+      return(0);
+      }
+
+    /* host is fine, must validate proxy via ruserok() */
+
+    HostAllowed = 1;
     }
 
   /* make short host name */
@@ -168,8 +198,9 @@ int site_check_user_map(
     *dptr = '\0';
     }
 
-  if ((server.sv_attr[(int)SRV_ATR_AllowNodeSubmit].at_flags & ATR_VFLAG_SET) && \
-      (server.sv_attr[(int)SRV_ATR_AllowNodeSubmit].at_val.at_long == 1) && \
+  if ((HostAllowed == 0) &&
+      (server.sv_attr[SRV_ATR_AllowNodeSubmit].at_flags & ATR_VFLAG_SET) && 
+      (server.sv_attr[SRV_ATR_AllowNodeSubmit].at_val.at_long == 1) &&
       (find_nodebyname(orighost) != NULL))
     {
     /* job submitted from compute host, access allowed */
@@ -177,10 +208,18 @@ int site_check_user_map(
     if (dptr != NULL)
        *dptr = '.';
 
-    return(0);
+    if ((ProxyRequested == 0) || (ProxyAllowed == 1))
+      {
+      return(0);
+      }
+
+    /* host is fine, must validate proxy via ruserok() */
+
+    HostAllowed = 1;
     }
 
-  if (server.sv_attr[(int)SRV_ATR_SubmitHosts].at_flags & ATR_VFLAG_SET)
+  if ((HostAllowed == 0) &&
+      (server.sv_attr[(int)SRV_ATR_SubmitHosts].at_flags & ATR_VFLAG_SET))
     {
     struct array_strings *submithosts = NULL;
     char                 *testhost;
@@ -199,19 +238,34 @@ int site_check_user_map(
         if (dptr != NULL)
           *dptr = '.';
 
-        return(0);
+        if ((ProxyRequested == 0) || (ProxyAllowed == 1))
+          {
+          return(0);
+          }
+
+        /* host is fine, must validate proxy via ruserok() */
+
+        HostAllowed = 1;
+
+        break;
         }
-    } /* END for (hostnum) */
-  } /* END if (SRV_ATR_SubmitHosts) */
+      }  /* END for (hostnum) */
+    }    /* END if (SRV_ATR_SubmitHosts) */
 
   if (dptr != NULL)
     *dptr = '.';
 
   rc = ruserok(orighost,0,owner,luser);
 
+  if (EMsg != NULL)
+    snprintf(EMsg,1024,"ruserok failed validating %s/%s from %s",
+      owner,
+      luser,
+      orighost);
+
 #ifdef sun
-  /* broken Sun ruserok() sets process so it appears to be owned	*/
-  /* by the luser, change it back for cosmetic reasons		*/
+  /* broken Sun ruserok() sets process so it appears to be owned */
+  /* by the luser, change it back for cosmetic reasons           */
 
   setuid(0);
 #endif	/* sun */
@@ -232,9 +286,11 @@ int site_check_user_map(
  *    Return -1 for access denied, otherwise 0 for ok.
  */
 
-int site_acl_check(pjob, pque)
-	job		*pjob;
-	pbs_queue	*pque;
-{
-	return (0);
-}
+int site_acl_check(
+
+  job       *pjob,
+  pbs_queue *pque)
+
+  {
+  return(0);
+  }
