@@ -226,13 +226,15 @@ struct	config	dependent_config[] = {
 	{ NULL,		{nullproc} },
 };
 	
+
 /*
-**	Don't need any periodic procsessing.
-*/
+ * This routine is called on each cycle of the main loop.
+ */
+
 void
-end_proc()
+dep_main_loop_cycle()
 {
-	return;
+  /* No periodic functions. */
 }
 
 void
@@ -246,8 +248,6 @@ dep_initialize()
 		log_err(errno, id, "opendir");
 		return;
 	}
-
-	return;
 }
 
 void
