@@ -142,7 +142,7 @@
 **		quota		quota information (sizes in KB)
 */
 
-static char ident[] = "@(#) sgi/$RCSfile$ $Revision: 983 $";
+static char ident[] = "@(#) sgi/$RCSfile$ $Revision: 1894 $";
 
 #define SGI_ZOMBIE_WRONG 1
 #define COMPLEX_MEM_CALC 0
@@ -288,12 +288,13 @@ dep_cleanup()
 }
 
 /*
-**	Don't need any periodic procsessing.
-*/
+ * This routine is called on each cycle of the main loop.
+ */
+
 void
-end_proc()
+dep_main_loop_cycle()
 {
-	return;
+  /* No periodic functions. */
 }
 
 /*
