@@ -91,6 +91,7 @@
 #include <unistd.h>
 #include <assert.h>
 #include <dirent.h>
+#include <time.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
 #include "dis.h"
@@ -107,6 +108,7 @@
 #include "mom_func.h"
 #include "log.h"
 #include "rpp.h"
+#include "resmon.h"
 #include "net_connect.h"
 #ifdef _CRAY
 #include <sys/category.h>
@@ -116,8 +118,7 @@
 #include <wordexp.h>
 
 extern struct var_table vtable;      /* see start_exec.c */
-extern        char          **environ;
-extern unsigned int alarm_time;
+extern char           **environ;
 
 extern int InitUserEnv(
       
@@ -141,13 +142,13 @@ extern unsigned int	default_server_port;
 extern int		exiting_tasks;
 extern tlist_head	svr_alljobs;
 extern char		mom_host[];
-extern char		*msg_err_unlink;
-extern char		*path_checkpoint;
-extern char		*path_spool;
-extern char		*path_undeliv;
+extern char            *msg_err_unlink;
+extern char            *path_checkpoint;
+extern char            *path_spool;
+extern char            *path_undeliv;
 extern attribute_def	job_attr_def[];
-extern char		*msg_jobmod;
-extern char		*msg_manager;
+extern char            *msg_jobmod;
+extern char            *msg_manager;
 extern time_t		time_now;
 extern int		resc_access_perm;	/* see encode_resc() */
 						/* in attr_fn_resc.c */
@@ -156,6 +157,7 @@ extern char             MOMUNameMissing[];
 extern int              pbs_rm_port;
 extern char             rcp_path[];
 extern char             rcp_args[];
+extern char            *TNoSpoolDirList[];
 
 /* Local Data Items */
 
@@ -163,14 +165,17 @@ static uid_t  useruid;
 static gid_t  usergid;
 static int    ngroup;
 static int   *groups;
+#if NO_SPOOL_OUTPUT == 0
 static char  *output_retained = "Output retained on that host in: ";
+#endif /* !NO_SPOOL_OUTPUT */
+
 static char   rcperr[MAXPATHLEN];	/* file to contain rcp error */
 
 extern char PBSNodeMsgBuf[1024];
 extern int  LOGLEVEL;
 
 extern int im_compose A_((int,char *,char *,int,tm_event_t,tm_task_id));
-
+extern int mom_open_socket_to_jobs_server A_(( job *, char *, void (*) A_((int))));
 
 /* prototypes */
 
@@ -446,7 +451,13 @@ static pid_t fork_to_user(
     setgroups(ngroup,(gid_t *)groups);
 
     setgid(usergid);
-    setuid(useruid);   /* run as the user */
+    
+    if (setuid(useruid) == -1)
+      {
+      /* cannot run as the user */
+
+      return(-PBSE_BADUSER); 
+      }
 
     if (chdir(hdir) == -1)
       {
@@ -456,52 +467,55 @@ static pid_t fork_to_user(
       }
     }
 
-    
 #ifdef HAVE_WORDEXP
   {
-    /* set some useful env variables */
+  /* set some useful env variables */
 
-    char *envstr;
+  char *envstr;
 
-    envstr=malloc((strlen("HOME=") + strlen(hdir) + 1) * sizeof(char));
+  envstr = malloc((strlen("HOME=") + strlen(hdir) + 1) * sizeof(char));
 
-    if (envstr == NULL)
-      {
-      sprintf(log_buffer,"malloc failed, errno=%d (%s)",
-        errno,
-        strerror(errno));
+  if (envstr == NULL)
+    {
+    sprintf(log_buffer,"malloc failed, errno=%d (%s)",
+      errno,
+      strerror(errno));
 
-      log_err(-1,id,log_buffer);
+    log_err(-1,id,log_buffer);
 
-      if (EMsg != NULL)
-        strncpy(EMsg,log_buffer,1024);
+    if (EMsg != NULL)
+      strncpy(EMsg,log_buffer,1024);
 
-      return(-PBSE_SYSTEM);
-      }
+    return(-PBSE_SYSTEM);
+    }
 
-    sprintf(envstr,"HOME=%s",hdir);
-    putenv(envstr);
+  sprintf(envstr,"HOME=%s",
+    hdir);
 
-    envstr=malloc((strlen("PBS_JOBID=") + strlen(preq->rq_ind.rq_cpyfile.rq_jobid) + 1) * sizeof(char));
+  putenv(envstr);
 
-    if (envstr == NULL)
-      {
-      sprintf(log_buffer,"malloc failed, errno=%d (%s)",
-        errno,
-        strerror(errno));
+  envstr = malloc((strlen("PBS_JOBID=") + strlen(preq->rq_ind.rq_cpyfile.rq_jobid) + 1) * sizeof(char));
 
-      log_err(-1,id,log_buffer);
+  if (envstr == NULL)
+    {
+    sprintf(log_buffer,"malloc failed, errno=%d (%s)",
+      errno,
+      strerror(errno));
 
-      if (EMsg != NULL)
-        strncpy(EMsg,log_buffer,1024);
+    log_err(-1,id,log_buffer);
 
-      return(-PBSE_SYSTEM);
-      }
+    if (EMsg != NULL)
+      strncpy(EMsg,log_buffer,1024);
 
-    sprintf(envstr,"PBS_JOBID=%s",preq->rq_ind.rq_cpyfile.rq_jobid);
-    putenv(envstr);
+    return(-PBSE_SYSTEM);
+    }
+
+  sprintf(envstr,"PBS_JOBID=%s",
+    preq->rq_ind.rq_cpyfile.rq_jobid);
+
+  putenv(envstr);
   }
-#endif
+#endif /* END HAVE_WORDEXP */
 
   return(pid);
   }  /* END fork_to_user() */
@@ -716,6 +730,7 @@ static int wchost_match(
 
 
 
+/* Determine which local path to copy to if specified using $usecp */
 
 static int told_to_cp(
 
@@ -727,6 +742,8 @@ static int told_to_cp(
   int    i;
   int    nh;
 
+  const char *id = "told_to_cp";
+
   static char newp[MAXPATHLEN + 1];
   extern struct cphosts *pcphosts;
 
@@ -749,10 +766,24 @@ static int told_to_cp(
         return(1);
         }
       }
-    }
+    }    /* END for (nh) */
 
    /* failure */
 
+  if (LOGLEVEL >= 3)
+    {
+    sprintf(log_buffer,"no local path matches host '%s' and path '%s' (%d paths checked)\n",
+      host,
+      oldpath,
+      nh);
+
+    log_record(
+      PBSEVENT_SYSTEM,
+      PBS_EVENTCLASS_SERVER,
+      (char *)id,
+      log_buffer);
+    }
+
   return(0);
   }  /* END told_to_cp() */
 
@@ -866,7 +897,7 @@ static int is_file_same(
 
 void req_deletejob(
 
-  struct batch_request *preq)
+  struct batch_request *preq)  /* I */
 
   {
   job *pjob;
@@ -875,6 +906,15 @@ void req_deletejob(
 
   if (pjob != NULL) 
     {
+    if (LOGLEVEL >= 3)
+      {
+      log_record(
+        PBSEVENT_JOB,
+        PBS_EVENTCLASS_JOB,
+        pjob->ji_qs.ji_jobid, 
+        "deleting job");
+      }
+
     /* assume success? */
 
     mom_deljob(pjob);
@@ -918,7 +958,7 @@ void req_holdjob(
     } 
   else 
     {
-    if ((rc = start_checkpoint(pjob,1,preq)) != 0)
+    if ((rc = start_checkpoint(pjob,1,preq)) != PBSE_NONE)
       req_reject(rc,0,preq,mom_host,"cannot checkpoint job");    /* unable to start chkpt */
     }
 
@@ -1008,14 +1048,12 @@ int message_job(
 
   rc = PBSE_NONE;
 
-  alarm(alarm_time);
   if (write(fds,text,len) != len)
     {
     log_err(errno,"message_job","unable to write message to job");
 
     rc = PBSE_INTERNAL;
     }
-  alarm(0);
 
   if (close(fds) != 0)
     {
@@ -1119,6 +1157,8 @@ const char *TJobAttr[] = {
   "hopcount",
   "qrank",
   "queuetype",
+  "start_time",
+  "start_count",
   NULL };
 
 
@@ -1157,6 +1197,15 @@ void req_modifyjob(
     return;
     }
 
+  if (LOGLEVEL >= 3)
+    {
+    log_record(
+      PBSEVENT_JOB,
+      PBS_EVENTCLASS_JOB,
+      pjob->ji_qs.ji_jobid,
+      "modifying job");
+    }
+
   plist = (svrattrl *)GET_NEXT(preq->rq_ind.rq_modify.rq_attr);
 
   if (plist == NULL) 
@@ -1423,9 +1472,15 @@ int MUSleep(
 
 
 /* send a signal to all tasks on sisters */
-int sigalltasks_sisters(job *pjob, int signum)
+
+int sigalltasks_sisters(
+
+  job *pjob, 
+  int  signum)
+
   {
   char      id[] = "sigalltasks_sisters";
+
   char     *cookie;
   eventent *ep;
   int i;
@@ -1488,8 +1543,10 @@ int sigalltasks_sisters(job *pjob, int signum)
       }
     }
 
-    return 0;
-  }
+  return(0);
+  }  /* END sigalltasks_sisters() */
+
+
 
 
 static void resume_suspend( 
@@ -1572,6 +1629,7 @@ static void resume_suspend(
       /* couldn't send signal, don't signal more tasks */
 
       savederr = errno;
+
       break;
       }  /* END if (stat < 0) */
     }    /* END for (tp) */
@@ -1706,9 +1764,9 @@ void req_signaljob(
   struct batch_request *preq) /* I */
 
   {
+  char           id[]= "req_signaljob";
   job            *pjob;
   int             sig;
-  int		  numprocs=0;
   char           *sname;
   struct sig_tbl *psigt;
   extern struct sig_tbl sig_tbl[];
@@ -1732,7 +1790,7 @@ void req_signaljob(
     log_record(
       PBSEVENT_JOB,
       PBS_EVENTCLASS_JOB,
-      (pjob != NULL) ? pjob->ji_qs.ji_jobid : "N/A",
+      pjob->ji_qs.ji_jobid,
       log_buffer);
     }
 
@@ -1782,14 +1840,14 @@ void req_signaljob(
     }
   else 
     {
-    if (!strncmp("SIG",sname,3))
+    if (!strncasecmp("SIG",sname,3))
       sname += 3;
 
     psigt = sig_tbl;
 
     while (psigt->sig_name != NULL) 
       {
-      if (!strcmp(sname,psigt->sig_name)) 
+      if (!strcasecmp(sname,psigt->sig_name)) 
         break;
 
       psigt++;
@@ -1805,14 +1863,21 @@ void req_signaljob(
     return;
     }
 
-  numprocs=kill_job(pjob,sig);
+  if ((sig == SIGTERM) && (pjob->ji_qs.ji_substate == JOB_SUBSTATE_SUSPEND))
+    {
+    /* if job is suspended, resume, and then kill - allow job to clean up on sigterm */
 
-  if ((numprocs == 0) && ((sig == 0)||(sig == SIGKILL))) 
+    kill_job(pjob,SIGCONT,id,"job is suspended, resume and kill");
+
+    sleep(1);
+    }
+
+  if ((kill_job(pjob,sig,id,"job was suspended, now killing") == 0) && (sig == 0)) 
     {
     /* SIGNUL and no procs found, force job to exiting */
     /* force issue of (another) job obit */
 
-    sprintf(log_buffer,"job recycled into exiting on SIGNULL/KILL from substate %d",
+    sprintf(log_buffer,"job recycled into exiting on SIGNULL from substate %d",
       pjob->ji_qs.ji_substate);
 
     LOG_EVENT(
@@ -1828,6 +1893,21 @@ void req_signaljob(
     exiting_tasks = 1;
     }
 
+  if ((sig == SIGKILL) && (pjob->ji_qs.ji_substate == JOB_SUBSTATE_EXITING)) 
+    {
+    /* force issue of (another) job obit */
+
+    sprintf(log_buffer,"job recycled into exiting on SIGKILL from substate exiting");
+
+    LOG_EVENT(
+      PBSEVENT_ERROR, 
+      PBS_EVENTCLASS_JOB,
+      pjob->ji_qs.ji_jobid, 
+      log_buffer);
+
+    exiting_tasks = 1;
+    }
+
   reply_ack(preq);
 
   return;
@@ -1837,7 +1917,7 @@ void req_signaljob(
 
 void encode_used(
 
-  job       *pjob,   /* I */
+  job        *pjob,   /* I */
   tlist_head *phead)  /* O */
 
   {
@@ -1914,6 +1994,50 @@ void encode_used(
   }  /* END encode_used() */
 
 
+void encode_flagged_attrs(
+
+  job        *pjob,   /* I */
+  tlist_head *phead)  /* O */
+
+  {
+  int index;
+  attribute		*at;
+  attribute_def		*ad;
+
+  for (index = 0;(int)index < JOB_ATR_LAST;++index)
+    {
+    at  = &pjob->ji_wattr[index];
+    ad  = &job_attr_def[index];
+
+    if (at->at_flags & ATR_VFLAG_SEND)
+      {
+      /* turn off "need to send" flag */
+
+      at->at_flags &= ~ATR_VFLAG_SEND;
+
+      if (LOGLEVEL >= 4)
+        {
+        sprintf(log_buffer,"encoding \"send flagged\" attr: %s",
+          ad->at_name);
+
+        LOG_EVENT(
+          PBSEVENT_DEBUG,
+          PBS_EVENTCLASS_JOB,
+          pjob->ji_qs.ji_jobid,
+          log_buffer);
+        }
+
+      ad->at_encode(
+        at,
+        phead,
+        ad->at_name,
+        NULL,
+        ATR_ENCODE_CLIENT);
+      }
+    }
+  }
+
+
 
 
 /*
@@ -1923,17 +2047,14 @@ void encode_used(
 
 void req_stat_job(
 
-  struct batch_request *preq)
+  struct batch_request *preq)  /* I */
 
   {
   int			all;
   char			*name;
   job			*pjob;
-  int			index;
   struct batch_reply	*preply = &preq->rq_reply;
   struct brp_status	*pstat;
-  attribute		*at;
-  attribute_def		*ad;
 
   /*
    * first, validate the name of the requested object, either
@@ -1994,43 +2115,9 @@ void req_stat_job(
 
     append_link(&preply->brp_un.brp_status,&pstat->brp_stlink,pstat);
 
-    /* add attributes to the status reply */
+    encode_used(pjob,&pstat->brp_attr);  /* adds resources_used attr */
 
-    for (index = 0;(int)index < JOB_ATR_LAST;++index) 
-      {
-      at  = &pjob->ji_wattr[index];
-      ad  = &job_attr_def[index];
-
-      if (at->at_flags & ATR_VFLAG_SEND) 
-        {
-        /* turn off "need to send" flag */
- 
-        at->at_flags &= ~ATR_VFLAG_SEND;
-
-        if (LOGLEVEL >= 4)
-          {
-          sprintf(log_buffer,"sending \"send flagged\" attr: %s",
-            ad->at_name);
-
-          LOG_EVENT(
-            PBSEVENT_DEBUG, 
-            PBS_EVENTCLASS_JOB,
-            pjob->ji_qs.ji_jobid, 
-            log_buffer);
-          }
-
-        ad->at_encode(
-          at, 
-          &pstat->brp_attr,
-          ad->at_name, 
-          NULL,
-          ATR_ENCODE_CLIENT);
-        }
-      }
-
-    /* now do resources used */
-
-    encode_used(pjob,&pstat->brp_attr);
+    encode_flagged_attrs(pjob,&pstat->brp_attr);  /* adds other flagged attrs */
     }
 
   reply_send(preq);
@@ -2092,7 +2179,7 @@ static int del_files(
     {
     /* ignore failure */
 
-    chdir(HDir);
+    if (chdir(HDir) == -1) {}
     }
 
   for (pair = (struct rqfpair *)GET_NEXT(preq->rq_ind.rq_cpyfile.rq_pair);
@@ -2124,7 +2211,7 @@ static int del_files(
         {
         /* ignore failure of chdir */
 
-        chdir(HDir);
+        if (chdir(HDir) == -1) {}
         }
 
 #ifdef HAVE_WORDEXP
@@ -2154,16 +2241,30 @@ static int del_files(
       }
 
 #ifdef HAVE_WORDEXP
-    switch (wordexp(path,&pathexp, WRDE_NOCMD|WRDE_UNDEF))
+    switch (wordexp(path,&pathexp,WRDE_NOCMD|WRDE_UNDEF))
       {
       case 0:
+
         break; /* Successful */
+
       case WRDE_NOSPACE:
+
         wordfree(&pathexp);
+
+        /* fall through */
+
       default:
-        sprintf(log_buffer,">>> failed to delete files, expansion of %s failed",path);
+
+        sprintf(log_buffer,">>> failed to delete files, expansion of %s failed",
+          path);
+
         add_bad_list(pbadfile,log_buffer,1);
+
         return(-1);
+
+        /*NOTREACHED*/
+
+        break;
       }
 
     strcpy(path,pathexp.we_wordv[0]);
@@ -2275,16 +2376,14 @@ static int del_files(
 
 void req_rerunjob(
 
-  struct batch_request *preq)
+  struct batch_request *preq)  /* I */
 
   {
   static char   *id = "req_rerunjob";
 
   job		*pjob;
-  unsigned int	 port;
+  int        sock;
   int		 rc;
-  int		 sock;
-  char		*svrport;
 
   pjob = find_job(preq->rq_ind.rq_rerun);
 
@@ -2295,6 +2394,15 @@ void req_rerunjob(
     return;
     }
 
+  if (LOGLEVEL >= 3)
+    {
+    log_record(
+      PBSEVENT_JOB,
+      PBS_EVENTCLASS_JOB,
+      pjob->ji_qs.ji_jobid,
+      "rerunning job");
+    }
+
   /* fork to send files back */
 
   if ((rc = fork_me(preq->rq_conn)) > 0) 
@@ -2310,28 +2418,17 @@ void req_rerunjob(
     return;
     }
 
-  /* Child process ...  for each standard file generate and */
-  /* send a Job Files request(s).				  */
+  /* Child process ...  for each standard file generate and send a Job Files request(s).
+   * No message handler function is needed because return_file blocks and waits for reply.
+   * This is acceptable because we are a child process, not pbs_mom.
+   */
 
-  svrport = strchr(pjob->ji_wattr[(int)JOB_ATR_at_server].at_val.at_str,(int)':');
-
-  if (svrport)
-    port = atoi(svrport+1);
-  else
-    port = default_server_port;
-
-  sock = client_to_svr(pjob->ji_qs.ji_un.ji_momt.ji_svraddr,port,1);
+  sock = mom_open_socket_to_jobs_server(pjob,id,NULL);
 
   if (sock < 0) 
     {
     /* FAILURE */
 
-    LOG_EVENT(
-      PBSEVENT_ERROR,
-      PBS_EVENTCLASS_REQUEST,
-      id, 
-      "no contact with the server");
-
     req_reject(PBSE_NOSERVER,0,preq,NULL,NULL);
 
     exit(0);
@@ -2379,8 +2476,8 @@ void req_rerunjob(
 static int sys_copy(
 
   int   rmtflg,  /* I */
-  char *ag2,     /* I */
-  char *ag3,     /* I */
+  char *ag2,     /* I (is this source or destination?) */
+  char *ag3,     /* I (is this source or destination?) */
   int   conn)    /* I */
 
   {
@@ -2482,6 +2579,8 @@ static int sys_copy(
         close(fd);
         }
 
+      /* NOTE:  arg2 should be source, arg3 should be destination */
+
       execl(ag0,ag0,ag1,ag2,ag3,NULL);
 
       /* reached only if execl() fails */
@@ -2496,12 +2595,12 @@ static int sys_copy(
       log_err(errno,id,log_buffer);
 
       exit(13);	/* 13, an unlucky number */
-      }
+      }    /* END else ((rc = fork()) > 0) */
 
     /* copy did not work, try again */
 
     if ((loop % 2) == 0)
-      sleep(loop/2 * 3 + 1);
+      sleep(loop / 2 * 3 + 1);
     }  /* END for (loop) */
 
   /* tried a bunch of times, just give up */
@@ -2568,38 +2667,43 @@ void req_cpyfile(
   int		 bad_files = 0;
   char		*bad_list = NULL;
   int		 dir;		
-  int		 from_spool;
+  int		 from_spool;  /* boolean - set if file must be removed from spool after copy */
   int		 len;
   char		 localname[MAXPATHLEN + 1];  /* used only for in-bound */
-  struct rqfpair  *pair;
+  struct rqfpair *pair;
   char		*prmt;
   int		 rc;
   int		 rmtflag;
+#if NO_SPOOL_OUTPUT == 0
   char		 undelname[MAXPATHLEN + 1];
+#endif /* !NO_SPOOL_OUTPUT */
+
 #ifdef  _CRAY
   char		 tmpdirname[MAXPATHLEN + 1];
 #endif 	/* _CRAY */
-#if NO_SPOOL_OUTPUT == 1
   char		 localname_alt[MAXPATHLEN + 1];
   struct stat	 myspooldir;
   int		 rcstat;
   char           homespool[MAXPATHLEN + 1];
-  int            havehomespool = 0;
-#endif
+  int            havehomespool;
 
   char           EMsg[1024];
   char           HDir[1024];
 
+  job           *pjob = NULL;
+
 #ifdef HAVE_WORDEXP
-  int		 madefaketmpdir=0;
-  int		 usedfaketmpdir=0;
+  int		 madefaketmpdir = 0;
+  int		 usedfaketmpdir = 0;
   wordexp_t	 arg2exp, arg3exp;
   int            arg2index = -1;
   char		 faketmpdir[1024];
-  job 		*pjob;
   int		 wordexperr = 0;
 #endif
 
+  arg2[0] = '\0';
+  arg3[0] = '\0';
+
   if (LOGLEVEL >= 3)
     {
     pair = (struct rqfpair *)GET_NEXT(preq->rq_ind.rq_cpyfile.rq_pair);
@@ -2665,8 +2769,8 @@ void req_cpyfile(
   /* now running as user in the user's home directory */
 
 #if NO_SPOOL_OUTPUT == 1
-  strcpy(homespool,HDir); 
-  strcat(homespool,"/.pbs_spool/");
+  snprintf(homespool,sizeof(homespool),"%s/.pbs_spool/",
+    HDir);
 
   rcstat = stat(homespool,&myspooldir);
 
@@ -2678,8 +2782,57 @@ void req_cpyfile(
     {
     havehomespool = 0;
     }
+#else  /* NO_SPOOL_OUTPUT == 1 */
+  homespool[0]  = '\0';
+  havehomespool = 0;
 #endif /* END NO_SPOOL_OUTPUT == 1 */
 
+  if ((havehomespool == 0) && (TNoSpoolDirList[0] != NULL))
+    {
+    int   dindex;
+
+    char *wdir;
+
+    if ((pjob = find_job(preq->rq_ind.rq_cpyfile.rq_jobid)) == NULL)
+      {
+      wdir = NULL;
+      }
+    else
+      {
+      wdir = get_job_envvar(pjob,"PBS_O_WORKDIR");
+      }
+
+    if (wdir != NULL)
+      {
+      /* check if job's work dir matches the no-spool directory list */
+
+      for (dindex = 0;dindex < TMAX_NSDCOUNT;dindex++)
+        {
+        if (TNoSpoolDirList[dindex] == NULL)
+          break;
+
+        if (!strcasecmp(TNoSpoolDirList[dindex],"$WORKDIR") || 
+            !strcmp(TNoSpoolDirList[dindex],"*"))
+          {
+          havehomespool = 1;
+
+          strncpy(homespool,wdir,sizeof(homespool));
+
+          break;
+          }
+
+        if (!strncmp(TNoSpoolDirList[dindex],wdir,strlen(TNoSpoolDirList[dindex])))
+          {
+          havehomespool = 1;
+
+          strncpy(homespool,wdir,sizeof(homespool));
+      
+          break;
+          }
+        }  /* END for (dindex) */
+      }    /* END if (wdir != NULL) */    
+    }      /* END if ((havehomespool == 0) && (TNoSpoolDirList != NULL)) */
+
 #ifdef HAVE_WORDEXP
   faketmpdir[0] = '\0';
 
@@ -2699,9 +2852,12 @@ void req_cpyfile(
       if (!mkdirtree(faketmpdir,0755))
         {
         char *envstr;
+
         envstr = malloc((strlen("TMPDIR=") + strlen(faketmpdir) + 1) * sizeof(char));
+
         sprintf(envstr,"TMPDIR=%s",
           faketmpdir);
+
         putenv(envstr);
 
         madefaketmpdir = 1;
@@ -2761,13 +2917,36 @@ void req_cpyfile(
       if (pair->fp_flag == STDJOBFILE) 
         {
 #if NO_SPOOL_OUTPUT == 0
+        if (havehomespool == 1)
+          {
+          /* only use spooldir if the job file exists */
 
-        /* stdout | stderr from MOM's spool area (ie, /usr/spool/PBS/spool ) */
+          strcpy(localname_alt,homespool);
+          strcat(localname_alt,"/");
+          strcat(localname_alt,pair->fp_local);
 
-        strcpy(localname,path_spool);
-        strcat(localname,pair->fp_local);  /* from location */
+          rcstat = stat(localname_alt,&myspooldir);
 
-        from_spool = 1;	/* flag as being in spool dir */
+          if ((rcstat == 0) && S_ISREG(myspooldir.st_mode))
+            {
+            strcpy(localname,localname_alt);
+            }
+          else
+            {
+            /* what should be done here??? */
+
+            strcpy(localname,localname_alt);
+            }
+          }
+        else
+          {
+          /* stdout | stderr from MOM's spool area (ie, /var/spool/torque/spool ) */
+
+          strcpy(localname,path_spool);
+          strcat(localname,pair->fp_local);  /* from location */
+
+          from_spool = 1;	/* flag as being in spool dir */
+          }
 #else
         strcpy(localname,pair->fp_local);  /* from location */
 
@@ -2776,17 +2955,24 @@ void req_cpyfile(
           /* only use ~/.pbs_spool if the file actually exists */
 
           strcpy(localname_alt,homespool);
+          strcat(localname_alt,"/");
           strcat(localname_alt,pair->fp_local);
 
           rcstat = stat(localname_alt,&myspooldir);
+
           if ((rcstat == 0) && S_ISREG(myspooldir.st_mode))
             {
+            strcpy(localname,localname_alt);
+            }
+          else
+            {
+            /* what should be done here??? */
+
             strcpy(localname,localname_alt);
             }
           }
-       
 #endif	/* NO_SPOOL_OUTPUT */
-        }
+        }  /* END if (pair->fp_flag == STDJOBFILE) */
 #if MOM_CHECKPOINT == 1
       else if (pair->fp_flag == JOBCKPFILE) 
         {
@@ -2798,11 +2984,10 @@ void req_cpyfile(
         {
         /* user-supplied stage-out file */
 
-        strncpy(localname,pair->fp_local,sizeof(localname)-1);  /* from location */
+        strncpy(localname,pair->fp_local,sizeof(localname) - 1);  /* from location */
         }
 
 #if SRFS
-
       /* Is this file part of $BIGDIR or $FASTDIR ? */
 
       if (!strncmp(localname,"/BIGDIR",7)) 
@@ -2821,7 +3006,6 @@ void req_cpyfile(
 
         strcpy(localname,tmpname);
         }
-
 #endif /* SRFS */
 
       strcpy(arg2,localname);
@@ -2839,7 +3023,7 @@ void req_cpyfile(
         }
 
       strcat(arg3,prmt);
-      } 
+      }  /* END if (dir == STAGE_DIR_OUT) */ 
     else 
       {	
       /* in bound (stage-in) file */
@@ -2859,7 +3043,7 @@ void req_cpyfile(
       strcat(arg2,prmt);
 
       strcpy(arg3,pair->fp_local);
-      }
+      }  /* END else (dir == STAGE_DIR_OUT) */
 
 #ifdef HAVE_WORDEXP
 
@@ -2940,7 +3124,7 @@ void req_cpyfile(
         break;
       }  /* END switch () */
 
-    /* Note: more than one word is only allowed for arg2 (source) */
+    /* NOTE: more than one word is only allowed for arg2 (source) */
 
 
     arg2index = -1;
@@ -3097,11 +3281,12 @@ error:
       }
 
     unlink(rcperr);
+
 #ifdef HAVE_WORDEXP
     if (!wordexperr)
       goto nextword;  /* ugh, it's hard to use a real loop when your feature is #ifdef's out */
 #endif
-    }  /* END for(pair) */
+    }  /* END for (pair) */
 
 #ifdef HAVE_WORDEXP
   if (madefaketmpdir && !usedfaketmpdir)
@@ -3136,7 +3321,7 @@ error:
 
 void req_delfile(
 
-  struct batch_request *preq)
+  struct batch_request *preq)  /* I */
 
   {
   int	 rc;
@@ -3208,14 +3393,12 @@ int mom_checkpoint_job(
   int  abort) /* I */
 
   {
-  static char	id[] = "mom_checkpoint_job";
-  int		hasold = 0;
+/*  int		hasold = 0; */
   int		sesid = -1;
   int		ckerr;
-  int		i;
-  struct stat	statbuf;
+/*  struct stat	statbuf; */
   char		path[MAXPATHLEN + 1];
-  char		oldp[MAXPATHLEN + 1];
+/*  char		oldp[MAXPATHLEN + 1]; */
   char		file[MAXPATHLEN + 1]; 
   char         *name;
   int		filelen;
@@ -3228,6 +3411,8 @@ int mom_checkpoint_job(
   strcat(path,pjob->ji_qs.ji_fileprefix);
   strcat(path,JOB_CKPT_SUFFIX);
 
+#if 0
+  /* Don't mess with existing checkpoints. */
   if (stat(path,&statbuf) == 0) 
     {
     strcpy(oldp,path);   /* file already exists, rename it */
@@ -3241,9 +3426,9 @@ int mom_checkpoint_job(
 
     hasold = 1;
     }
+#endif
 
-  if (mkdir(path,0755) == -1)
-    goto fail;
+  mkdir(path,0755);
 
   filelen = strlen(path);
 
@@ -3312,6 +3497,8 @@ int mom_checkpoint_job(
 
   /* Checkpoint successful */
 
+  pjob->ji_qs.ji_svrflags |= JOB_SVFLG_CHKPT;
+
   job_save(pjob,SAVEJOB_FULL);  /* to save resources_used so far */
 
   sprintf(log_buffer,"checkpointed to %s", 
@@ -3323,8 +3510,10 @@ int mom_checkpoint_job(
     pjob->ji_qs.ji_jobid, 
     log_buffer);
 
+#if 0
   if (hasold) 
     remtree(oldp);
+#endif
 
   return(PBSE_NONE);
 
@@ -3357,7 +3546,7 @@ fail:
     }
 
   /* Clean up files */
-
+#if 0
   remtree(path);
 
   if (hasold) 
@@ -3365,6 +3554,7 @@ fail:
     if (rename(oldp,path) == -1)
       pjob->ji_qs.ji_svrflags &= ~JOB_SVFLG_CHKPT;
     }
+#endif
 
   if (ckerr == EAGAIN)
     {
@@ -3485,31 +3675,49 @@ int start_checkpoint(
   struct batch_request *preq)	/* may be null */
 
   {
-  static char id[] = "start_checkpoint";
+/*  static char id[] = "start_checkpoint"; */
   svrattrl *pal;
-  pid_t     pid;
+/*  pid_t     pid; */
   int       rc;
-  int       sock = -1;
   attribute tmph;
+  char      name_buffer[1024];
 
   if (mom_does_chkpnt() == 0) 	/* no checkpoint, reject request */
     {
     return(PBSE_NOSUP);
     }
 
+    /* Build the name of the checkpoint file before forking to the child */
+
+  sprintf(name_buffer, "ckpt.%s.%d",
+    pjob->ji_qs.ji_jobid,
+    (int)time(0) );
+  decode_str(&pjob->ji_wattr[(int)JOB_ATR_chkptname],NULL,NULL,name_buffer);
+  pjob->ji_wattr[(int)JOB_ATR_chkptname].at_flags =
+    ATR_VFLAG_SET | ATR_VFLAG_MODIFY | ATR_VFLAG_SEND;
+
+
+  if (!(pjob->ji_wattr[(int)JOB_ATR_chkptdir].at_flags & ATR_VFLAG_SET))
+    {
+    /* No dir specified, use the default job checkpoint directory /var/spool/torque/checkpoint/42.host.domain.CK */
+
+    strcpy(name_buffer,path_checkpoint);
+    strcat(name_buffer,pjob->ji_qs.ji_fileprefix);
+    strcat(name_buffer,JOB_CKPT_SUFFIX);
+    decode_str(&pjob->ji_wattr[(int)JOB_ATR_chkptdir],NULL,NULL,name_buffer);
+    }
+
+#if 0
   /* now set up as child of MOM */
 
-  if (preq != NULL)
-    sock = preq->rq_conn;
-
-  pid = fork_me(sock);
+  pid = fork_me((preq == NULL) ? -1 : preq->rq_conn);
 
   if (pid > 0) 
     {
     /* parent, record pid in job for when child terminates */
 
     pjob->ji_momsubt = pid;
-    pjob->ji_mompost = post_chkpt;
+    pjob->ji_mompost = (int (*)())post_chkpt;
 
     if (preq)
       free_br(preq);
@@ -3521,18 +3729,13 @@ int start_checkpoint(
     } 
   else if (pid < 0) 
     {
-    char tmpLine[1024];
-
     /* error on fork */
 
-    sprintf(tmpLine,"cannot fork child process for checkpoint, errno=%d",
-      errno);
-
-    log_err(-1,id,tmpLine);
-
+    log_err(errno,id,"cannot fork child process for checkpoint");
     return(PBSE_SYSTEM);	
     } 
-  else 
+  else
+#endif
     {
     /* child - does the checkpoint */
 
@@ -3565,11 +3768,14 @@ int start_checkpoint(
 
     if ((rc == 0) && (hok == 0))
       rc = site_mom_postchk(pjob,(int)tmph.at_val.at_long);
-
+#if 0
     exit(rc);	/* zero exit tells main chkpnt ok */
+#else
+  return(PBSE_NONE);		/* parent return */
+#endif
     }
 
-  return(0);		/* parent return */
+  return(PBSE_NONE);		/* parent return */
   }  /* END start_checkpoint() */
 
 #endif	/* MOM_CHECKPOINT */
