@@ -143,11 +143,12 @@ sigset_t	allsigs;
 int		alarm_time;
 static char    *logfile = (char *)0;
 static char	path_log[_POSIX_PATH_MAX];
+char	path_acct[_POSIX_PATH_MAX];
 int 		pbs_rm_port;
 
-int	schedreq();
+int             schedreq();
 
-extern int get_4byte(int,int *);
+extern int get_4byte(int,unsigned int *);
 
 
 /*
@@ -251,16 +252,19 @@ static int server_disconnect(
 /*
 **	Got an alarm call.
 */
-void
-toolong(sig)
-int	sig;
-{
-	char	*id = "toolong";
-	struct	stat	sb;
-	pid_t	cpid;
 
-	log_record(PBSEVENT_SYSTEM, PBS_EVENTCLASS_SERVER, id, "alarm call");
-	DBPRT(("alarm call\n"))
+void toolong(
+
+  int sig)
+
+  {
+  char	*id = "toolong";
+  struct	stat	sb;
+  pid_t	cpid;
+
+	log_record(PBSEVENT_SYSTEM, PBS_EVENTCLASS_SERVER, id, 
+                   "scheduling iteration took too long");
+	DBPRT(("scheduling iteration too long\n"))
 
 	if (connector >= 0 && server_disconnect(connector))
 		log_err(errno, id, "server_disconnect");
@@ -312,9 +316,10 @@ int	sig;
 	exit(0);
 }
 
+
+/* sock refers to an opened socket */
 int
-socket_to_conn(sock)
-    int sock;		/* opened socket */
+socket_to_conn(int sock)
 {
 	int     i;
 
@@ -454,7 +459,7 @@ static int read_config(
 
 
 #if !defined(DEBUG) && !defined(NO_SECURITY_CHECK)
-  if (chk_file_sec(file,0,0,S_IWGRP|S_IWOTH,1))
+  if (chk_file_sec(file,0,0,S_IWGRP|S_IWOTH,1,NULL))
     {
     return(-1);
     }
@@ -464,10 +469,13 @@ static int read_config(
  
   mask_num = 0;
 
-	if ((conf = fopen(file, "r")) == NULL) {
-		log_err(errno, id, "cannot open config file");
-		return (-1);
-	}
+  if ((conf = fopen(file,"r")) == NULL) 
+    {
+    log_err(errno,id,"cannot open config file");
+
+    return(-1);
+    }
+
 	while (fgets(line, CONF_LINE_LEN, conf)) {
 
 		if ((line[0] == '#') || (line[0] == '\n'))
@@ -509,8 +517,7 @@ static int read_config(
 }
 
 void
-restart(sig)
-    int	sig;
+restart(int sig)
 {
 	char    *id = "restart";
 
@@ -529,8 +536,7 @@ restart(sig)
 }
 
 void
-badconn(msg)
-    char	*msg;
+badconn(char *msg)
 {
 	static	char	id[] = "badconn";
 	struct	in_addr	addr;
@@ -577,7 +583,8 @@ int server_command()
 
   int           new_socket;
   torque_socklen_t	slen;
-  int		i, cmd;
+  int		i;
+  unsigned int  cmd;
   pbs_net_t	addr;
 
   slen = sizeof(saddr);
@@ -642,7 +649,7 @@ int server_command()
     return(SCH_ERROR);
     }
 
-  return(cmd);
+  return((int)cmd);
   }
 
 
@@ -651,11 +658,11 @@ int server_command()
 
 /*
  * lock_out - lock out other daemons from this directory.
+ *
+ * op is either F_WRLCK or F_UNLCK
  */
 
-static void lock_out(fds, op)
-	int fds;
-	int op;		/* F_WRLCK  or  F_UNLCK */
+static void lock_out(int fds, int op)
 {
 	struct flock flock;
 
@@ -680,26 +687,27 @@ int main(
   char *argv[])
 
   {
-	char		*id = "main";
-	struct	hostent	*hp;
-	int		go, c, errflg = 0;
-	int		lockfds;
-	int		t = 1;
-	pid_t		pid;
-	char		host[100];
-	char		*homedir = PBS_SERVER_HOME;
-	unsigned int	port;
-	char		*dbfile = "sched_out";
-	struct	sigaction	act;
-	sigset_t	oldsigs;
-	caddr_t		curr_brk = 0;
-	caddr_t		next_brk;
-	extern	char	*optarg;
-	extern	int	optind, opterr;
-	extern	int	rpp_fd;
-	fd_set		fdset;
-	int		schedinit A_((int argc, char **argv));
-	int		schedule A_((int com, int connector));
+  char		*id = "main";
+  struct	hostent	*hp;
+  int		go, c, errflg = 0;
+  int		lockfds;
+  int		t = 1;
+  pid_t		pid;
+  char		host[100];
+  char		*homedir = PBS_SERVER_HOME;
+  unsigned int	port;
+  char		*dbfile = "sched_out";
+  struct	sigaction	act;
+  sigset_t	oldsigs;
+  caddr_t	curr_brk = 0;
+  caddr_t	next_brk;
+  extern char	*optarg;
+  extern int	optind, opterr;
+  extern int	rpp_fd;
+  fd_set	fdset;
+
+  int		schedinit A_((int argc, char **argv));
+  int		schedule A_((int com, int connector));
 
 	glob_argv = argv;
 	alarm_time = 180;
@@ -788,8 +796,8 @@ int main(
 
 	(void)sprintf(log_buffer, "%s/sched_priv", homedir);
 #if !defined(DEBUG) && !defined(NO_SECURITY_CHECK)
-	c  = chk_file_sec(log_buffer, 1, 0, S_IWGRP|S_IWOTH, 1);
-	c |= chk_file_sec(PBS_ENVIRON, 0, 0, S_IWGRP|S_IWOTH, 0);
+	c  = chk_file_sec(log_buffer,1,0,S_IWGRP|S_IWOTH,1,NULL);
+	c |= chk_file_sec(PBS_ENVIRON,0,0,S_IWGRP|S_IWOTH,0,NULL);
 	if (c != 0) exit(1);
 #endif  /* not DEBUG and not NO_SECURITY_CHECK */
 	if (chdir(log_buffer) == -1) {
@@ -797,6 +805,7 @@ int main(
 		exit(1);
 	}
 	(void)sprintf(path_log,   "%s/sched_logs", homedir);
+	(void)sprintf(path_acct,   "%s/%s", log_buffer,PBS_ACCT );
 
 
 	/* The following is code to reduce security risks                */
@@ -1007,8 +1016,10 @@ int main(
 	log_record(PBSEVENT_SYSTEM, PBS_EVENTCLASS_SERVER, id, log_buffer);
 
 	FD_ZERO(&fdset);
-	for (go=1; go;) {
-		int	cmd;
+
+  for (go = 1;go;) 
+    {
+    int cmd;
 
 		if (rpp_fd != -1)
 			FD_SET(rpp_fd, &fdset);
@@ -1050,11 +1061,15 @@ int main(
 		}
 		if (sigprocmask(SIG_SETMASK, &oldsigs, NULL) == -1)
 			log_err(errno, id, "sigprocmask(SIG_SETMASK)");
-	}
+    }
 
-	sprintf(log_buffer, "%s normal finish pid %ld", argv[0], (long)pid);
-	log_record(PBSEVENT_SYSTEM, PBS_EVENTCLASS_SERVER, id, log_buffer);
+  sprintf(log_buffer,"%s normal finish pid %ld", 
+    argv[0], 
+    (long)pid);
 
-	(void)close(server_sock);
-	exit(0);
-}
+  log_record(PBSEVENT_SYSTEM,PBS_EVENTCLASS_SERVER,id,log_buffer);
+
+  close(server_sock);
+
+  exit(0);
+  }  /* END main() */
