@@ -125,7 +125,7 @@
 #if !defined(H_ERRNO_DECLARED)
 extern int h_errno;
 #endif
-extern time_t      time_now;
+extern time_t  time_now;
 
 
 /* Global Data */
@@ -138,6 +138,7 @@ extern char	*path_nodes;
 extern char	*path_nodestate;
 extern char	*path_nodenote;
 extern int       LOGLEVEL;
+extern attribute_def  node_attr_def[];   /* node attributes defs */
 
 
 /* Functions in this file
@@ -166,12 +167,14 @@ extern int       LOGLEVEL;
  */
 
 
-#include <work_task.h>
+#include "work_task.h"
 
 extern void ping_nodes(struct work_task *);
 
 
 
+/* us IP address to look up matchin node structure */
+
 struct pbsnode *PGetNodeFromAddr(
 
   pbs_net_t addr)  /* I */
@@ -349,11 +352,6 @@ int addr_ok(
 
 
 /* FIXME: this should all be in a seperate header file */
-typedef struct tree_t {
-       u_long          key;
-       struct pbsnode  *nodep;
-       struct tree_t   *left, *right;
-} tree;
 
 extern void tinsert(const u_long key, struct pbsnode *nodep, tree **rootp);
 extern void *tdelete(const u_long key, tree **rootp);
@@ -753,7 +751,7 @@ static void initialize_pbsnode(
 
   pnode->nd_name    = pname;
   pnode->nd_stream  = -1;
-  pnode->nd_addrs   = pul;	    /*list of host byte order */
+  pnode->nd_addrs   = pul;       /* list of host byte order */
   pnode->nd_ntype   = ntype;
   pnode->nd_nsn     = 0;
   pnode->nd_nsnfree = 0;
@@ -904,8 +902,19 @@ static int process_host_name_part(
   {
   struct hostent *hp;
   struct in_addr  addr;
-  char		 *phostname;     /*caller supplied hostname   */
-  int		  len, i;
+  char		 *phostname;     /* caller supplied hostname   */
+  int             ipcount;
+  int		  len, totalipcount;
+
+  char            tmpHName[1024];
+  char           *hptr;
+
+  static int      NodeSuffixIsSet = 0;
+
+  static char    *NodeSuffix;
+
+  int             hindex;
+  int             size = 0;
 
   len = strlen(objname);
 
@@ -916,13 +925,15 @@ static int process_host_name_part(
 
   phostname = strdup(objname);
 
-  if (phostname == NULL)
+  if ((phostname == NULL) || (pul == NULL))
     {
     return(PBSE_SYSTEM);
     }
 
   *ntype = NTYPE_CLUSTER;
 
+  *pul = NULL;
+
   if ((len >= 3) && !strcmp(&phostname[len - 3],":ts")) 
     {
     phostname[len - 3] = '\0';
@@ -935,6 +946,7 @@ static int process_host_name_part(
       objname);
 
     free(phostname);
+    phostname = NULL;
 
     return(PBSE_UNKNODE);
     }
@@ -958,56 +970,144 @@ static int process_host_name_part(
         errno);
 
       free(phostname);
+      phostname = NULL;
 
       return(PBSE_UNKNODE);
       }
 	
-    hname = (char *)strdup(hp->h_name); /*canonical name in theory*/
+    hname = (char *)strdup(hp->h_name); /* canonical name in theory */
 
     if (hname == NULL) 
       {
       free(phostname);
+      phostname = NULL;
 
       return(PBSE_SYSTEM);
       }
 
-    if ((hp = gethostbyname(hname)) == NULL) 
+    totalipcount = 0;
+
+    if (NodeSuffixIsSet == 0)
       {
-      sprintf(log_buffer,"bad cname %s, h_errno=%d errno=%d",
-        hname,
-        h_errno,errno);
+      if (((server.sv_attr[(int)SRV_ATR_NodeSuffix].at_flags & ATR_VFLAG_SET) != 0) &&
+           (server.sv_attr[(int)SRV_ATR_NodeSuffix].at_val.at_str != NULL))
+        {
+        NodeSuffix = strdup(server.sv_attr[(int)SRV_ATR_NodeSuffix].at_val.at_str);
+        }
 
-      free(hname);
-
-      free(phostname);
-
-      return(PBSE_UNKNODE);
+      NodeSuffixIsSet = 1;
       }
 
-    free(hname);
-    }  /* END if (hp->h_addr_list[1] == NULL) */
+    if (NodeSuffix != NULL)
+      {
+      char *ptr;
 
-  for (i = 0;hp->h_addr_list[i];i++);	              /* count ipaddrs */
+      /* NOTE:  extract outside of loop because hname will be freed */
 
-  *pul = (u_long *)malloc(sizeof(u_long) * (i + 1));  /*null end it*/
+      ptr = strchr(hname,'.');
 
-  if (*pul == NULL) 
-    {
-    free(phostname);
-    }
+      if (ptr != NULL)
+        {
+        *ptr = '\0';
 
-  for (i = 0;hp->h_addr_list[i];i++) 
-    {
-    u_long	ipaddr;
+        snprintf(tmpHName,sizeof(tmpHName),"%s%s.%s",
+          hname,
+          NodeSuffix,
+          ptr + 1);
 
-    memcpy((char *)&addr,hp->h_addr_list[i],hp->h_length);
+        *ptr = '.';
+        }
+      else
+        {
+        snprintf(tmpHName,sizeof(tmpHName),"%s%s",
+          hname,
+          NodeSuffix);
+        }
+      }
 
-    ipaddr = ntohl(addr.s_addr);
+    for (hindex = 0;hindex < 2;hindex++)
+      {
+      if (hindex == 0)
+        {
+        hptr = hname;
+        }
+      else if (NodeSuffix != NULL) 
+        {
+        hptr = tmpHName;
+        }     
+      else
+        {
+        continue;
+        }
 
-    (*pul)[i] = ipaddr;
-    }
+      if ((hp = gethostbyname(hptr)) == NULL) 
+        {
+        sprintf(log_buffer,"bad cname %s, h_errno=%d errno=%d",
+          hptr,
+          h_errno,
+          errno);
 
-  (*pul)[i] = 0;			/* null term array ip addrs */
+        if (hname != NULL)
+          {
+          free(hname);
+          hname = NULL;
+          }
+
+        if (phostname != NULL)
+          {
+          free(phostname);
+          phostname = NULL;
+          }
+
+        return(PBSE_UNKNODE);
+        }
+
+      if (hname != NULL)
+        {
+        free(hname);
+        hname = NULL;
+        }
+
+      /* count host ipaddrs */
+
+      for (ipcount = 0;hp->h_addr_list[ipcount];ipcount++); 
+
+      if (*pul == NULL)
+        {
+        size = sizeof(u_long) * (ipcount + 1);
+
+        *pul = (u_long *)malloc(size);  /* zero-terminate list */
+        }
+      else
+        {
+        size += sizeof(u_long) * ipcount;
+
+        *pul = (u_long *)realloc(*pul,size);
+        }
+
+      if (*pul == NULL) 
+        {
+        if (phostname != NULL)
+          {
+          free(phostname);
+          phostname = NULL;
+          }
+        }
+    
+      for (ipcount = 0;hp->h_addr_list[ipcount];ipcount++,totalipcount++) 
+        {
+        u_long ipaddr;
+
+        memcpy((char *)&addr,hp->h_addr_list[ipcount],hp->h_length);
+
+        ipaddr = ntohl(addr.s_addr);
+
+        (*pul)[totalipcount] = ipaddr;
+        }
+
+      (*pul)[totalipcount] = 0;		/* zero-term array ip addrs */
+      }  /* END for (hindex) */
+    }    /* END if (hp->h_addr_list[1] == NULL) */
 
   *pname = phostname;			/* return node name	    */
 
@@ -1231,7 +1331,7 @@ static struct pbssubn *create_subnode(
   pnode->nd_nsnfree++;
 
   if ((pnode->nd_state & (INUSE_JOB|INUSE_JOBSHARE)) != 0)
-    pnode->nd_state &= ~(INUSE_JOB|INUSE_JOBSHARE);
+    pnode->nd_state = INUSE_FREE;
 
   psubn->allocto = (resource_t)0;
 
@@ -1263,9 +1363,9 @@ int create_pbs_node(
   {
   struct pbsnode  *pnode = NULL;
   struct pbsnode **tmpndlist;
-  int              ntype;	/*node type; time-shared, not */
-  char            *pname;	/*node name w/o any :ts       */
-  u_long          *pul;		/*0 terminated host adrs array*/
+  int              ntype;	/* node type; time-shared, not */
+  char            *pname;	/* node name w/o any :ts       */
+  u_long          *pul;		/* 0 terminated host adrs array*/
   int              rc;
   int              iht;
 
@@ -1495,7 +1595,7 @@ int setup_nodes(void)
   extern char server_name[];
   extern resource_t next_resource_tag;
 
-  sprintf(log_buffer,"%s()\n",
+  sprintf(log_buffer,"%s()",
     id);
 
   log_record(
@@ -1508,7 +1608,7 @@ int setup_nodes(void)
 
   if ((nin = fopen(path_nodes,"r")) == NULL) 
     {
-    sprintf(log_buffer,"cannot open node description file '%s' in setup_nodes()",
+    sprintf(log_buffer,"cannot open node description file '%s' in setup_nodes()\n",
       path_nodes);
 
     log_event(
