@@ -109,7 +109,7 @@
 #include "svrfunc.h"
 #include "job.h"
 #include "log.h"
-#include "pbs_nodes.h"
+#include "pbs_nodes.h" 
 #include "rpp.h"
 #include "dis.h"
 #include "dis_init.h"
@@ -155,6 +155,7 @@ extern char  server_name[];
 
 extern struct server server;
 extern tlist_head svr_newnodes;
+extern attribute_def  node_attr_def[];   /* node attributes defs */
 extern int            SvrNodeCt;
 
 #define	SKIP_NONE	0
@@ -211,14 +212,8 @@ static void funcs_dis() /* The equivalent of DIS_tcp_funcs() */
 **      Modified by Tom Proett <proett@nas.nasa.gov> for PBS.
 */
 
-typedef struct tree_t {
-  u_long		key;
-  struct pbsnode	*nodep;
-  struct tree_t	*left, *right;
-  } tree;
-
-tree	*ipaddrs = NULL;	/* tree of ip addrs */
-tree	*streams = NULL;	/* tree of stream numbers */
+tree *ipaddrs = NULL;	/* tree of ip addrs */
+tree *streams = NULL;	/* tree of stream numbers */
 
 
 
@@ -226,20 +221,20 @@ tree	*streams = NULL;	/* tree of stream numbers */
 
 struct pbsnode *tfind(
 
-  const u_long   key,	/* key to be located */
-  tree         **rootp)	/* address of tree root */
+  const u_long   key,	/* I - key to be located */
+  tree         **rootp)	/* O - address of tree root */
 
   {
   if (rootp == NULL)
     {
-    return NULL;
+    return(NULL);
     }
 
   while (*rootp != NULL) 
     {		/* Knuth's T1: */
     if (key == (*rootp)->key)	/* T2: */
       {
-      return(*rootp)->nodep;	/* we found it! */
+      return((*rootp)->nodep);	/* we found it! */
       }
 
     rootp = (key < (*rootp)->key) ?
@@ -913,7 +908,9 @@ void send_cluster_addrs(
     {
     /* new nodes are still being added... don't bother yet or start over */
 
-    DBPRT(("%s: not sending addrs yet, %d tasks exist\n",id,num_addrnote_tasks));
+    DBPRT(("%s: not sending addrs yet, %d tasks exist\n",
+      id,
+      num_addrnote_tasks));
 
     startcount = 0;
 
@@ -935,6 +932,7 @@ void send_cluster_addrs(
      * otherwise we'll get bogged down.  The skipped nodes will get the
      * updated info when they reconnect. 
      */
+
     if ((np == NULL) || (np->nd_state & INUSE_DELETED) || (np->nd_stream < 0))
       continue;
 
@@ -993,20 +991,28 @@ void send_cluster_addrs(
   else
     {
     /* all nodes have new addr list, so clear the new nodes */
+
     while ((nnew = (new_node *)GET_NEXT(svr_newnodes)) != NULL)
       {
-      np=find_nodebyname(nnew->nn_name);
+      np = find_nodebyname(nnew->nn_name);
+
       if (np != NULL)
         {
         np->nd_state &= ~INUSE_OFFLINE;
         }
+
       delete_link(&nnew->nn_link);
       }
 
       /* reset startcount, as we've sent the updates for all servers */
       startcount = 0;
     }
-  } /* END send_cluster_addrs */
+  }     /* END send_cluster_addrs */
+
+
+
+
+
 
 /*
  *      setup_notification -  Sets up the  mechanism for notifying
@@ -1181,13 +1187,13 @@ int is_stat_get(
         struct attribute nattr;
 
         /* first we decode ret_info into nattr... */
-        if ((node_attr_def + NODE_ATR_np)->at_decode(&nattr,ATTR_NODE_np,NULL,ret_info+6) == 0)
+        if ((node_attr_def + ND_ATR_np)->at_decode(&nattr,ATTR_NODE_np,NULL,ret_info+6) == 0)
           {
           /* ... and if MOM's ncpus is higher than our np... */
           if (nattr.at_val.at_long > np->nd_nsn)
             {
             /* ... then we do the defined magic to create new subnodes */
-            (node_attr_def + NODE_ATR_np)->at_action(&nattr,(void *)np,ATR_ACTION_ALTER);
+            (node_attr_def + ND_ATR_np)->at_action(&nattr,(void *)np,ATR_ACTION_ALTER);
 
             update_nodes_file();
             }
@@ -1302,11 +1308,12 @@ done:
 
 /* EOF on a stream received (either stream or addr must be specified) */
 /* mark node down and remove associated streams */
+/* NOTE: pass in stream = -1 if you wish the stream to be optional */
 
 void stream_eof(
 
   int	 stream,  /* I (optional) */
-  u_long addr,    /* I (optional) */
+  u_long addr,  /* I (optional) */
   int	 ret)     /* I (ignored) */
 
   {
@@ -1348,11 +1355,11 @@ void stream_eof(
 
   /* remove stream from list of valid connections */
 
-  if (stream >= 0)
+  if (np->nd_stream >= 0)
     {
-    np->nd_stream = -1;
+    tdelete((u_long)np->nd_stream,&streams);
 
-    tdelete((u_long)stream,&streams);
+    np->nd_stream = -1;
     }
 
   return;
@@ -1370,6 +1377,9 @@ void stream_eof(
  *      connection to the node.
 */
 
+#define TNODE_PINGCOUNT     256
+#define TNODE_PINGRETRYTIME   3
+
 void ping_nodes(
 
   struct work_task *ptask)  /* I (optional) */
@@ -1396,8 +1406,8 @@ void ping_nodes(
     }
 
   sprintf(log_buffer,"ping attempting to contact %d nodes",
-    (svr_totnodes - startcount > 256) ? 
-      256 : 
+    (svr_totnodes - startcount > TNODE_PINGCOUNT) ? 
+      TNODE_PINGCOUNT : 
       (svr_totnodes - startcount < 0) ? 
         svr_totnodes : 
         svr_totnodes - startcount); /* phew! */
@@ -1414,9 +1424,9 @@ void ping_nodes(
 
   for (i = startcount;i < svr_totnodes;i++) 
     {
-    if (i - startcount > 256)
+    if (i - startcount > TNODE_PINGCOUNT)
       {
-      /* only ping 256 nodes at a time, ping next batch later */
+      /* only ping TNODE_PINGCOUNT nodes at a time, ping next batch later */
 
       break;
       }
@@ -1441,13 +1451,13 @@ void ping_nodes(
 
       if (np->nd_stream == -1) 
         {
-	sprintf(log_buffer,"rpp_open to %s",
-	  np->nd_name);
+        sprintf(log_buffer,"rpp_open to %s",
+          np->nd_name);
 
-	log_err(errno,id,log_buffer);
+        log_err(errno,id,log_buffer);
 
-	continue;
-	}
+        continue;
+        }
 
       tinsert((u_long)np->nd_stream,np,&streams);
       }  /* END if (np->nd_stream < 0) */
@@ -1518,9 +1528,9 @@ void ping_nodes(
 
   if (startcount < svr_totnodes)
     {
-    /* continue outstanding pings in 3 seconds */
+    /* continue outstanding pings in TNODE_PINGRETRYTIME seconds */
 
-    set_task(WORK_Timed,time_now + 3,ping_nodes,NULL); 
+    set_task(WORK_Timed,time_now + TNODE_PINGRETRYTIME,ping_nodes,NULL); 
     }
 
   return;
@@ -1534,7 +1544,7 @@ void ping_nodes(
 
 int add_cluster_addrs(
 
-  int stream)
+  int stream) /* I */
 
   {
   char id[] = "add_cluster_addrs";
@@ -1563,6 +1573,7 @@ int add_cluster_addrs(
     for (j = 0;np->nd_addrs[j];j++)
       {   
       u_long ipaddr = np->nd_addrs[j];                                                             
+
       if (LOGLEVEL >= 8)
         {
         sprintf(log_buffer,"adding node[%d] interface[%d] %ld.%ld.%ld.%ld to hello response",
