@@ -111,6 +111,7 @@
 #include "job.h"
 #include "queue.h"
 #include "server.h"
+#include "pbs_nodes.h"
 #include "net_connect.h"
 #include "credential.h"
 #include "svrfunc.h"
@@ -123,11 +124,14 @@
 #include "batch_request.h"
 #include "pbs_proto.h"
 
+
+#define TSERVER_HA_CHECK_TIME  1  /* 1 second sleep time between checks on the lock file for high availability */
+
 /* external functions called */
 
 extern int  pbsd_init A_((int));
 extern void shutdown_ack ();
-extern void update_nodes_file A_((void));
+extern int  update_nodes_file A_((void));
 extern void tcp_settimeout(long);
 extern void poll_job_task(struct work_task *);
 extern int  schedule_jobs(void);
@@ -143,22 +147,24 @@ extern int RPPConfigure(int,int);
 extern int    svr_chngNodesfile;
 extern int    svr_totnodes;
 
-
 /* Local Private Functions */
 
 static int    get_port A_((char *,unsigned int *,pbs_net_t *));
 static time_t next_task A_(());
 static int    start_hot_jobs();
 static void   lock_out A_((int,int));
+static int   try_lock_out A_((int,int));
 
 /* Global Data Items */
 
+int             high_availability_mode = FALSE;
 char	       *acct_file = NULL;
 char	       *log_file  = NULL;
 char	       *path_home = PBS_SERVER_HOME;
 char	       *path_acct;
 char	        path_log[MAXPATHLEN + 1];
 char	       *path_priv;
+char	       *path_arrays;
 char	       *path_jobs;
 char	       *path_queues;
 char	       *path_spool;
@@ -178,7 +184,7 @@ unsigned int	pbs_mom_port = 0;
 unsigned int	pbs_rm_port;
 pbs_net_t	pbs_scheduler_addr;
 unsigned int	pbs_scheduler_port;
-extern pbs_net_t	pbs_server_addr;
+extern pbs_net_t pbs_server_addr;
 unsigned int	pbs_server_port_dis;
 int		queue_rank = 0;
 struct server	server;		/* the server structure */
@@ -192,18 +198,21 @@ tlist_head	svr_queues;            /* list of queues                   */
 tlist_head	svr_alljobs;           /* list of all jobs in server       */
 tlist_head	svr_newjobs;           /* list of incoming new jobs        */
 tlist_head	svr_newnodes;          /* list of newly created nodes      */
+tlist_head	svr_jobarrays;         /* list of all job arrays           */
 tlist_head	task_list_immed;
 tlist_head	task_list_timed;
 tlist_head	task_list_event;
 
 time_t		time_now = 0;
 
+char           *plogenv = NULL;
 int             LOGLEVEL = 0;
 int             DEBUGMODE = 0;
+int             TDoBackground = 1;  /* background daemon */
 int             TForceUpdate = 0;  /* (boolean) */
 
 char           *ProgName;
-
+char           *NodeSuffix = NULL;
 
 
 
@@ -213,9 +222,9 @@ void DIS_rpp_reset()
   if (dis_getc != rpp_getc) 
     {
     dis_getc    = rpp_getc;
-    dis_puts    = (int (*) A_((int,const char *,size_t)))rpp_write;
-    dis_gets    = (int (*) A_((int,char *,size_t)))rpp_read;
-    disr_skip   = (int (*) A_((int,size_t)))rpp_skip;
+    dis_puts    = (int (*)A_((int,const char *,size_t)))rpp_write;
+    dis_gets    = (int (*)A_((int,char *,size_t)))rpp_read;
+    disr_skip   = (int (*)A_((int,size_t)))rpp_skip;
     disr_commit = rpp_rcommit;
     disw_commit = rpp_wcommit;
     }
@@ -226,9 +235,12 @@ void DIS_rpp_reset()
 
 
 
-/*
-** Read a RPP message from a stream.  Only one kind of message
-** is expected -- Inter Server requests from MOM's.
+/**
+ * Read a RPP message from a stream.  
+ *
+ * NOTE: Only one kind of message is expected -- Inter Server requests from MOM's.
+ *
+ * @param stream (I)
 */
 
 void do_rpp(
@@ -261,10 +273,23 @@ void do_rpp(
 
   if (ret != DIS_SUCCESS) 
     {
+    /* FAILURE */
+
     if (LOGLEVEL >= 1)
       {
-      sprintf(log_buffer,"corrupt rpp request received on stream %d (invalid protocol)",
-        stream);
+      struct pbsnode *node;
+
+      extern tree *streams;        /* tree of stream numbers */
+
+      /* NOTE:  report IP associated w/stream - NYI */
+
+      node = tfind((u_long)stream,&streams);
+
+      sprintf(log_buffer,"corrupt rpp request received on stream %d (node: %s) - invalid protocol - rc=%d (%s)",
+        stream,
+        (node != NULL) ? node->nd_name : "NULL",
+        ret,
+        dis_emsg[ret]);
 
       log_record(
         PBSEVENT_SCHED,
@@ -276,7 +301,7 @@ void do_rpp(
     stream_eof(stream,0,ret);
 
     return;
-    }
+    }  /* END if (ret != DIS_SUCCESS) */
 
   version = disrsi(stream,&ret);
 
@@ -284,8 +309,10 @@ void do_rpp(
     {
     if (LOGLEVEL >= 1)
       {
-      sprintf(log_buffer,"corrupt rpp request received on stream %d (invalid version)",
-        stream);
+      sprintf(log_buffer,"corrupt rpp request received on stream %d - invalid version - rc=%d (%s)",
+        stream,
+        ret,
+        dis_emsg[ret]);
 
       log_record(
         PBSEVENT_SCHED,
@@ -374,11 +401,26 @@ int PBSShowUsage(
   char *EMsg)  /* I (optional) */
 
   {
-  const char *Msg = "[ -A <ACCTFILE> ] [ -a <ATTR> ] [ -d <HOMEDIR> ] [ -L <LOGFILE> ] [ -M <MOMPORT> ]\n  [ -p <SERVERPORT> ] [ -R <RMPORT> ] [ -S <SCHEDULERPORT> ] [ -t <TYPE> ]\n  [ --version|--help ]\n";
+  fprintf(stderr,"Usage: %s\n",
+    ProgName);
 
-  fprintf(stderr,"usage:  %s %s\n",
-    ProgName,
-    Msg);
+  fprintf(stderr,"  -A <INT>  \\\\ Alarm Time\n");
+  fprintf(stderr,"  -a <BOOL> \\\\ Scheduling\n");
+  fprintf(stderr,"  -d <PATH> \\\\ Homedir\n");
+  fprintf(stderr,"  -D        \\\\ Debugmode\n");
+  fprintf(stderr,"  -f        \\\\ Force Overwrite Serverdb\n");
+  fprintf(stderr,"  -h        \\\\ Print Usage\n");
+  fprintf(stderr,"  -H <HOST> \\\\ Daemon Hostname\n");
+  fprintf(stderr,"  -L <PATH> \\\\ Logfile\n");
+  fprintf(stderr,"  -M <PORT> \\\\ MOM Port\n");
+  fprintf(stderr,"  -p <PORT> \\\\ Server Port\n");
+  fprintf(stderr,"  -R <PORT> \\\\ RM Port\n");
+  fprintf(stderr,"  -S <PORT> \\\\ Scheduler Port\n");
+  fprintf(stderr,"  -t <TYPE> \\\\ Startup Type (hot, warm, cold, create)\n");
+  fprintf(stderr,"  -v        \\\\ Version\n");
+  fprintf(stderr,"  --ha      \\\\ High Availability MODE\n");
+  fprintf(stderr,"  --help    \\\\ Print Usage\n");
+  fprintf(stderr,"  --version \\\\ Version\n");
 
   if (EMsg != NULL)
     {
@@ -411,7 +453,7 @@ int main(
   int	 privfd;		/* fd to send is messages */
   uint	 tryport;
   char	 lockfile[MAXPATHLEN + 1];
-  char	*pc;
+  char	*pc = NULL;
   char  *ptr;
   job	*pjob;
   pbs_queue *pque;
@@ -427,6 +469,8 @@ int main(
   void check_nodes A_((struct work_task *));
   void check_log A_((struct work_task *));
 
+  char   EMsg[1024];
+
   static struct {
     char *it_name;
     int   it_type;
@@ -446,19 +490,8 @@ int main(
   ProgName = argv[0];
 
   strcpy(pbs_current_user,"PBS_Server");
-  msg_daemonname=strdup(pbs_current_user);
 
-  /* if we are not running with real and effective uid of 0, forget it */
-
-#ifndef __CYGWIN__
-  if ((getuid() != 0) || (geteuid() != 0)) 
-    {
-    fprintf(stderr,"%s: Must be run by root\n", 
-      ProgName);
-
-    return(1);
-    }
-#endif /* end __CYGWIN__ */
+  msg_daemonname = strdup(pbs_current_user);
 
   /* set standard umask */
 
@@ -472,12 +505,28 @@ int main(
 
   server_host[0] = '\0';
 
-  if (((server_host[0] == '\0') && (gethostname(server_host,PBS_MAXHOSTNAME) == -1)) ||
-       (get_fullhostname(server_host,server_host,PBS_MAXHOSTNAME) == -1)) 
-    {
-    log_err(-1,"pbsd_main","Unable to get my host name");
+  EMsg[0] = '\0';
 
-    return(-1);
+  if (((server_host[0] == '\0') && (gethostname(server_host,PBS_MAXHOSTNAME) == -1)) ||
+       (get_fullhostname(server_host,server_host,PBS_MAXHOSTNAME,EMsg) == -1)) 
+    {
+    /* FAILURE - shutdown */
+
+    if (EMsg[0] != '\0')
+      {
+      char tmpLine[1024];
+
+      snprintf(tmpLine,sizeof(tmpLine),"unable to determine local server hostname - %s",
+        EMsg);
+
+      log_err(-1,"pbsd_main",tmpLine);
+      }
+    else
+      {
+      log_err(-1,"pbsd_main","unable to determine local server hostname");
+      }
+
+    exit(1);
     }
 
   /* initialize service port numbers for self, Scheduler, and MOM */
@@ -550,7 +599,7 @@ int main(
 
   /* parse the parameters from the command line */
 
-  while ((c = getopt(argc,argv,"A:a:d:fh:p:t:L:M:R:S:-:")) != -1) 
+  while ((c = getopt(argc,argv,"A:a:d:DfhH:L:M:p:R:S:t:v-:")) != -1) 
     {
     switch (c) 
       {
@@ -570,33 +619,43 @@ int main(
 
           exit(0);
           }
-        else if (!strcmp(optarg,"about"))
+
+        if (!strcmp(optarg,"about"))
           {
-	  printf("package: %s\n",PACKAGE_STRING);
-	  printf("sourcedir %s\n",PBS_SOURCE_DIR);
-	  printf("configure: %s\n",PBS_CONFIG_ARGS);
+	  printf("package:     %s\n",PACKAGE_STRING);
+	  printf("sourcedir:   %s\n",PBS_SOURCE_DIR);
+	  printf("configure:   %s\n",PBS_CONFIG_ARGS);
 	  printf("buildcflags: %s\n",PBS_CFLAGS);
-	  printf("buildhost: %s\n",PBS_BUILD_HOST);
-	  printf("builddate: %s\n",PBS_BUILD_DATE);
-	  printf("buildir: %s\n",PBS_BUILD_DIR);
-	  printf("builduser: %s\n",PBS_BUILD_USER);
-	  printf("installdir: %s\n",PBS_INSTALL_DIR);
-	  printf("serverhome: %s\n",PBS_SERVER_HOME);
+	  printf("buildhost:   %s\n",PBS_BUILD_HOST);
+	  printf("builddate:   %s\n",PBS_BUILD_DATE);
+	  printf("builddir:    %s\n",PBS_BUILD_DIR);
+	  printf("builduser:   %s\n",PBS_BUILD_USER);
+	  printf("installdir:  %s\n",PBS_INSTALL_DIR);
+	  printf("serverhome:  %s\n",PBS_SERVER_HOME);
+	  printf("version:     %s\n",PACKAGE_VERSION);
+
           exit(0);
           }
-        else if (!strcmp(optarg,"help"))
+
+        if (!strcmp(optarg,"help"))
           {
           PBSShowUsage(NULL);
     
-          exit(1);
+          exit(0);
           }
-        else
-          {
-          PBSShowUsage("invalid command line arg");
 
-          exit(1);
+        if (!strcasecmp(optarg,"ha"))	/* High Availability */
+          {
+          high_availability_mode = TRUE;
+          break;
           }
 
+        PBSShowUsage("invalid command line arg");
+
+        exit(1);
+
+        /*NOTREACHED*/
+
         break;
 
       case 'a':
@@ -610,7 +669,7 @@ int main(
           (void)fprintf(stderr,"%s: bad -a option\n", 
             argv[0]);
 
-          return(1);
+          exit(1);
           }
 
         break;
@@ -621,6 +680,12 @@ int main(
 
         break;
 
+      case 'D':
+
+        TDoBackground = 0;
+
+        break;
+
       case 'f':
 
         TForceUpdate = 1;
@@ -629,16 +694,39 @@ int main(
 
       case 'h':
 
+        PBSShowUsage(NULL);
+  
+        exit(0);
+
+        break;
+
+      case 'H':
+
         /* overwrite locally detected hostname with specified hostname */
         /*  (used for multi-homed hosts) */
 
         strncpy(server_host,optarg,PBS_MAXHOSTNAME);
 
-        if (get_fullhostname(server_host,server_host,PBS_MAXHOSTNAME) == -1)
+        if (get_fullhostname(server_host,server_host,PBS_MAXHOSTNAME,EMsg) == -1)
           {
-          log_err(-1,"pbsd_main","Unable to get host name");
+          /* FAILURE */
 
-          return(1);
+          if (EMsg[0] != '\0')
+            {
+            char tmpLine[1024];
+
+            snprintf(tmpLine,sizeof(tmpLine),"unable to determine full hostname for specified server host '%s' - %s",
+              server_host,
+              EMsg);
+
+            log_err(-1,"pbsd_main",tmpLine);
+            }
+          else
+            {
+            log_err(-1,"pbsd_main","unable to determine full server hostname");
+            }
+
+          exit(1);
           }
 
         strcpy(server_name,server_host);
@@ -660,7 +748,7 @@ int main(
             fprintf(stderr,"%s: -h host too long\n", 
               argv[0]);
 
-            return(1);
+            exit(1);
             }
 
           strcat(server_name,":");
@@ -677,7 +765,7 @@ int main(
             fprintf(stderr, "%s: -h host invalid\n", 
               argv[0]);
 
-            return(1);
+            exit(1);
             }
           }    /* END if (strlen(servicename) > 0) */
 
@@ -692,7 +780,7 @@ int main(
           fprintf(stderr,"%s: -p host:port too long\n",
             argv[0]);
 
-          return(1);
+          exit(1);
           }
 
         strcat(server_name,":");
@@ -703,7 +791,7 @@ int main(
           fprintf(stderr,"%s: -p host:port invalid\n", 
             argv[0]);
 
-          return(1);
+          exit(1);
           }
 
         break;
@@ -725,7 +813,7 @@ int main(
           fprintf(stderr, "%s -t bad recovery type\n",
             argv[0]);
 
-          return(1);
+          exit(1);
           }
 
         break;
@@ -750,7 +838,7 @@ int main(
             argv[0],
             optarg);
 
-          return(1);
+          exit(1);
           }
 
         if (isalpha((int)*optarg)) 
@@ -771,7 +859,7 @@ int main(
             argv[0], 
             optarg);
 
-          return(1);
+          exit(1);
           }
 
         break;
@@ -789,51 +877,97 @@ int main(
             argv[0],
             optarg);
 
-          return(1);
+          exit(1);
           }
 
         break;
 
+      case 'v':
+
+          fprintf(stderr,"version: %s\n",
+            PACKAGE_VERSION);
+
+          exit(0);
+
+          break;
+
       default:
 
-        fprintf(stderr,"%s: unknown option: %c\n",
-          argv[0],
-          c);
+        PBSShowUsage("invalid command line arg");
 
-        return(1);
+        exit(1);
 
         break;
-      }  /* END switch(c) */
+      }  /* END switch (c) */
     }    /* END while (c) */
 
   if (optind < argc) 
     {
-    fprintf(stderr, "%s: invalid operand\n", 
+    fprintf(stderr,"%s: invalid operand\n", 
       argv[0]);
 
+    exit(1);
+    }
+
+  /* if we are not running with real and effective uid of 0, forget it */
+
+  if ((getuid() != 0) || (geteuid() != 0)) 
+    {
+    fprintf(stderr,"%s: must be run by root\n", 
+      ProgName);
+
     return(1);
     }
 
+  i = sysconf(_SC_OPEN_MAX);
+  while (--i > 2)
+    close(i); /* close any file desc left open by parent */
+
+
   /* make sure no other server is running with this home directory */
 
   sprintf(lockfile,"%s/%s/server.lock",
     path_home,
     PBS_SVR_PRIVATE);
 
-  if ((lockfds = open(lockfile,O_CREAT | O_TRUNC | O_WRONLY, 0600)) < 0) 
+  if ((lockfds = open(lockfile,O_CREAT|O_TRUNC|O_WRONLY,0600)) < 0) 
     {
-    sprintf(log_buffer, "%s: unable to open lock file",
-      msg_daemonname);
+    sprintf(log_buffer, "%s: unable to open lock file '%s'",
+      msg_daemonname,
+      lockfile);
 
     fprintf(stderr,"%s\n", 
       log_buffer);
 
     log_err(errno,msg_daemonname,log_buffer);
 
-    return(2);
+    exit(2);
     }
 
-  lock_out(lockfds,F_WRLCK);
+
+  if (high_availability_mode)
+    {
+    /* This will allow multiple instance of the pbs_server to be
+     * running.  This must be done before setting up the client
+     * sockets interface, reading the config file, and contacting
+     * the compute nodes.
+     */
+
+    if (TDoBackground == 1)
+      {
+      if (fork() > 0)
+        {
+        /* parent goes away */
+        exit(0);
+        }
+      }
+    while (try_lock_out(lockfds,F_WRLCK))
+      sleep(TSERVER_HA_CHECK_TIME);	/* Relinquish */
+    }
+  else
+    {
+    lock_out(lockfds,F_WRLCK);
+    }
 	
   server.sv_started = time(&time_now);	/* time server started */
 
@@ -852,7 +986,6 @@ int main(
 
   log_open(log_file,path_log);
 
-
   sprintf(log_buffer,msg_startup1,server_name,server_init_type);
 
   log_event(
@@ -864,14 +997,15 @@ int main(
   /* initialize the server objects and perform specified recovery */
   /* will be left in the server's private directory		*/
 
-  if ((pc = getenv("PBSLOGLEVEL")) != NULL)
+  if ((plogenv = getenv("PBSLOGLEVEL")) != NULL)
     {
-    LOGLEVEL = (int)strtol(pc,NULL,10);
+    LOGLEVEL = (int)strtol(plogenv,NULL,10);
     }
-
+    
   if ((pc = getenv("PBSDEBUG")) != NULL)
     {
     DEBUGMODE = 1;
+    TDoBackground = 0;
     }
 
   /* NOTE:  env cleared in pbsd_init() */
@@ -880,7 +1014,7 @@ int main(
     {
     log_err(-1,msg_daemonname,"pbsd_init failed");
 
-    return(3);
+    exit(3);
     }
 
   /* initialize the network interface */
@@ -902,12 +1036,19 @@ int main(
 
     log_err(-1,msg_daemonname,"init_network failed dis");
 
-    return(3);
+    exit(3);
     }
 
-  net_set_type(Secondary,FromClientDIS);	/* "there" */
+  if (init_network(0,process_request) != 0) 
+    {
+    perror("pbs_server: unix domain socket");
 
-  if (DEBUGMODE == 0)
+    log_err(-1,msg_daemonname,"init_network failed unix domain socket");
+
+    exit(3);
+    }
+
+  if (TDoBackground == 1)
     {
     /* go into the background and become own session/process group */
 
@@ -924,7 +1065,7 @@ int main(
       {
       log_err(errno,msg_daemonname,"setsid failed");
 
-      return(2);
+      exit(2);
       }
 
     lock_out(lockfds,F_WRLCK);
@@ -941,11 +1082,11 @@ int main(
 
     dummyfile = fopen("/dev/null","w");
     assert((dummyfile != 0) && (fileno(dummyfile) == 2));
-    }
+    }  /* END if (TDoBackground == 1) */
   else
     {
-    if (isdigit(pc[0]))
-      LOGLEVEL = (int)strtol(pc,NULL,0);
+    if ((plogenv != NULL) && isdigit(plogenv[0]))
+      LOGLEVEL = (int)strtol(plogenv,NULL,0);
 
     sid = getpid();
   
@@ -956,12 +1097,14 @@ int main(
   sprintf(log_buffer,"%ld\n", 
     (long)sid);
 
-  if (write(lockfds,log_buffer,strlen(log_buffer)) != (ssize_t)strlen(log_buffer))
+  if (write(lockfds,log_buffer,strlen(log_buffer)) != 
+      (ssize_t)strlen(log_buffer))
     {
     log_err(errno,msg_daemonname,"failed to write pid to lockfile");
 
-    return(-1);
+    exit(-1);
     }
+
 #if (PLOCK_DAEMONS & 1)
   plock(PROCLOCK);
 #endif
@@ -970,7 +1113,7 @@ int main(
     {
     log_err(errno,msg_daemonname,"rpp_bind");
 
-    return(1);
+    exit(1);
     }
 
   rpp_fd = -1;		/* force rpp_bind() to get another socket */
@@ -992,7 +1135,7 @@ int main(
     {
     log_err(errno,msg_daemonname,"no privileged ports");
 
-    return(1);
+    exit(1);
     }
 
   if (LOGLEVEL >= 5)
@@ -1004,8 +1147,8 @@ int main(
       "creating rpp and private interfaces");
     }
 
-  add_conn(rppfd,Primary,(pbs_net_t)0,0,rpp_request);
-  add_conn(privfd,Primary,(pbs_net_t)0,0,rpp_request);
+  add_conn(rppfd,Primary,(pbs_net_t)0,0,PBS_SOCK_INET,rpp_request);
+  add_conn(privfd,Primary,(pbs_net_t)0,0,PBS_SOCK_INET,rpp_request);
 
   /* record the fact that we are up and running */
 
@@ -1021,6 +1164,7 @@ int main(
      down unless they have already reported in */
 
   when = server.sv_attr[(int)SRV_ATR_check_rate].at_val.at_long + time_now;
+
   if (svr_totnodes > 1024)
     {
     /* for large systems, give newly reported nodes more time before 
@@ -1035,7 +1179,7 @@ int main(
 
   /* Just check the nodes with check_nodes above and don't ping anymore. */
 
-  set_task(WORK_Timed,time_now + 5,ping_nodes,NULL);
+  set_task(WORK_Timed,time_now + 5,ping_nodes,NULL); 
 
   set_task(WORK_Timed,time_now + 5,check_log,NULL);
 
@@ -1127,16 +1271,12 @@ int main(
       log_err(-1,msg_daemonname,"wait_request failed");
       }
 
-    /* update dynamic loglevel specification */
+    /* qmgr can dynamically set the loglevel specification
+     * we use the new value if PBSLOGLEVEL was not specified
+     */
 
-    /* FIXME: This overrides PBSLOGLEVEL and PBSDEBUG */
-
-    /* if (!getenv("PBSLOGLEVEL") && !getenv("PBSDEBUG"))
-     * (and SRV_ATR_LogLevel's at_action can set LOGLEVEL */
-
-    if (!getenv("PBSLOGLEVEL"))
+   if (plogenv == NULL )
       LOGLEVEL = server.sv_attr[(int)SRV_ATR_LogLevel].at_val.at_long;
-    
 
     /* any running jobs need a status update? */ 
 
@@ -1153,15 +1293,15 @@ int main(
           {
           /* spread these out over the next JobStatRate seconds */
 
-          when = pjob->ji_wattr[(int)JOB_ATR_qrank].at_val.at_long % JobStatRate;
+          when = pjob->ji_wattr[(int)JOB_ATR_qrank].at_val.at_long % 
+                 JobStatRate;
     
           ptask = set_task(WORK_Timed,when + time_now,poll_job_task,pjob);
 
-          if (ptask)
+          if (ptask != NULL)
             {
             append_link(&pjob->ji_svrtask,&ptask->wt_linkobj,ptask);
             }
-
           }
         }
 
@@ -1214,6 +1354,10 @@ int main(
 
   net_close(-1);		/* close all network connections */
 
+#ifdef ENABLE_UNIX_SOCKETS
+  unlink(TSOCK_PATH);
+#endif /* END ENABLE_UNIX_SOCKETS */
+
   log_event(
     PBSEVENT_SYSTEM|PBSEVENT_FORCE,
     PBS_EVENTCLASS_SERVER,
@@ -1224,13 +1368,15 @@ int main(
 
   log_close(1);
 
-  return(0);
+  exit(0);
   }  /* END main() */
 
 
+
+
 void check_log(
 
- struct work_task *ptask)
+ struct work_task *ptask) /* I */
 
  {
  long depth = 1;
@@ -1262,8 +1408,14 @@ void check_log(
         }
       }
     }
- set_task(WORK_Timed,time_now + 300, check_log, NULL);   
- } /* END check_log */
+
+  set_task(WORK_Timed,time_now + PBS_LOG_CHECK_RATE,check_log,NULL);   
+
+  return;
+  } /* END check_log */
+
+
+
 
 
 
@@ -1430,11 +1582,11 @@ static int start_hot_jobs(void)
 
 
 
-/*
- * lock_out - lock out other daemons from this directory.
+/**
+ * Try to lock
+ * @return Zero on success, one on failure
  */
-
-static void lock_out(
+static int try_lock_out(
 
   int fds,
   int op)		/* F_WRLCK  or  F_UNLCK */
@@ -1447,7 +1599,20 @@ static void lock_out(
   flock.l_start  = 0;
   flock.l_len    = 0;
 
-  if (fcntl(fds,F_SETLK,&flock) < 0) 
+  return(fcntl(fds,F_SETLK,&flock) != 0);
+  }
+
+
+/*
+ * lock_out - lock out other daemons from this directory.
+ */
+static void lock_out(
+
+  int fds,
+  int op)		/* F_WRLCK  or  F_UNLCK */
+
+  {
+  if (try_lock_out(fds,op)) 
     {
     strcpy(log_buffer,"pbs_server: another server running\n");
 
@@ -1457,8 +1622,6 @@ static void lock_out(
 
     exit(1);
     }
-
-  return;
   }
 
 /* END pbsd_main.c */
