@@ -105,8 +105,7 @@
 
 /* Private Functions Local to this file */
 
-static int get_hold A_((tlist_head *, char **));
-static void post_hold A_((struct work_task *));
+static void process_hold_reply A_((struct work_task *));
 
 /* Global Data Items: */
 
@@ -118,25 +117,33 @@ extern char	*msg_postmomnojob;
 extern time_t	 time_now;
 
 int chk_hold_priv A_((long val, int perm));
+int get_hold A_((tlist_head *, char **, attribute *));
 
-/* Private Data */
 
-static attribute temphold;
 
 /*
  * chk_hold_priv - check that client has privilege to set/clear hold
  */
 
-int chk_hold_priv(val, perm)
-	long	val;	/* hold bits being changed */
-	int	perm;	/* client privilege */
-{
-	if ( (val & HOLD_s) && ((perm & ATR_DFLAG_MGWR) == 0) )
-		return (PBSE_PERM);
-	if ( (val & HOLD_o) && ((perm & (ATR_DFLAG_MGWR|ATR_DFLAG_OPWR)) == 0) )
-		return (PBSE_PERM);
-	return (PBSE_NONE);
-}
+int chk_hold_priv(
+
+  long val,   /* hold bits being changed */
+  int  perm)  /* client privilege */
+
+  {
+  if ((val & HOLD_s) && ((perm & ATR_DFLAG_MGWR) == 0))
+    {
+    return(PBSE_PERM);
+    }
+
+  if ((val & HOLD_o) && ((perm & (ATR_DFLAG_MGWR|ATR_DFLAG_OPWR)) == 0))
+    {
+    return(PBSE_PERM);
+    }
+
+  return(PBSE_NONE);
+  }  /* END chk_hold_priv() */
+
 
 
 /*
@@ -155,10 +162,11 @@ void req_holdjob(
   int		 newstate;
   int		 newsub;
   long		 old_hold;
-  job		*pjob;
-  char		*pset;
-  int		 rc;
-
+  job    *pjob;
+  char    *pset;
+  int     rc;
+  attribute temphold;
+ 
   pjob = chk_job_request(preq->rq_ind.rq_hold.rq_orig.rq_objname,preq);
 
   if (pjob == NULL)
@@ -168,44 +176,49 @@ void req_holdjob(
 
   /* cannot do anything until we decode the holds to be set */
 
-	if ( (rc=get_hold(&preq->rq_ind.rq_hold.rq_orig.rq_attr, &pset)) != 0) {
-		req_reject(rc,0,preq,NULL,NULL);
-		return;
-	}
-			
-	/* if other than HOLD_u is being set, must have privil */
+  if ( (rc=get_hold(&preq->rq_ind.rq_hold.rq_orig.rq_attr, &pset, 
+                    &temphold)) != 0)
+    {
+    req_reject(rc,0,preq,NULL,NULL);
+    return;
+    }
+      
+  /* if other than HOLD_u is being set, must have privil */
 
-	if ((rc = chk_hold_priv(temphold.at_val.at_long, preq->rq_perm)) != 0) {
-			req_reject(rc,0,preq,NULL,NULL);
-			return;
-	}
+  if ((rc = chk_hold_priv(temphold.at_val.at_long, preq->rq_perm)) != 0)
+    {
+      req_reject(rc,0,preq,NULL,NULL);
+      return;
+    }
 
-	hold_val = &pjob->ji_wattr[(int)JOB_ATR_hold].at_val.at_long;
-	old_hold = *hold_val;
-	*hold_val |= temphold.at_val.at_long;
-	pjob->ji_wattr[(int)JOB_ATR_hold].at_flags |= ATR_VFLAG_SET;
-	(void)sprintf(log_buffer, msg_jobholdset, pset, preq->rq_user,
-		      preq->rq_host);
+  hold_val = &pjob->ji_wattr[(int)JOB_ATR_hold].at_val.at_long;
+  old_hold = *hold_val;
+  *hold_val |= temphold.at_val.at_long;
+  pjob->ji_wattr[(int)JOB_ATR_hold].at_flags |= ATR_VFLAG_SET;
+  sprintf(log_buffer, msg_jobholdset, pset, preq->rq_user,
+          preq->rq_host);
 
-	if ( (pjob->ji_qs.ji_state == JOB_STATE_RUNNING) &&
-	     (pjob->ji_wattr[(int)JOB_ATR_chkpnt].at_val.at_str) &&
-	     (*pjob->ji_wattr[(int)JOB_ATR_chkpnt].at_val.at_str != 'n') ) {
-	     
-		/* have MOM attempt checkpointing */
+  if (pjob->ji_qs.ji_state == JOB_STATE_RUNNING)
+    {
+       
+    /* have MOM attempt checkpointing */
 
-		if ((rc = relay_to_mom(pjob->ji_qs.ji_un.ji_exect.ji_momaddr,
-				       preq, post_hold)) != 0) {
-			*hold_val = old_hold;	/* reset to the old value */
-			req_reject(rc, 0, preq,NULL,NULL);
-		} else {
-			pjob->ji_qs.ji_substate = JOB_SUBSTATE_RERUN;
-			pjob->ji_qs.ji_svrflags |=
-					JOB_SVFLG_HASRUN | JOB_SVFLG_CHKPT;
-			job_save(pjob, SAVEJOB_QUICK);
-			LOG_EVENT(PBSEVENT_JOB, PBS_EVENTCLASS_JOB,
-				  pjob->ji_qs.ji_jobid, log_buffer);
-		}
-	} 
+    if ((rc = relay_to_mom(pjob->ji_qs.ji_un.ji_exect.ji_momaddr,
+               preq, process_hold_reply)) != 0)
+      {
+      *hold_val = old_hold;  /* reset to the old value */
+      req_reject(rc, 0, preq,NULL,NULL);
+      } 
+    else
+      {
+      pjob->ji_qs.ji_substate = JOB_SUBSTATE_RERUN;
+      pjob->ji_qs.ji_svrflags |=
+          JOB_SVFLG_HASRUN | JOB_SVFLG_CHKPT;
+      job_save(pjob, SAVEJOB_QUICK);
+      LOG_EVENT(PBSEVENT_JOB, PBS_EVENTCLASS_JOB,
+          pjob->ji_qs.ji_jobid, log_buffer);
+      }
+    } 
   else 
     {
     /* everything went well, may need to update the job state */
@@ -220,7 +233,7 @@ void req_holdjob(
       {
       /* indicate attributes changed     */
 
-      pjob->ji_modified = 1;	 
+      pjob->ji_modified = 1;   
 
       svr_evaljobstate(pjob,&newstate,&newsub,0);
 
@@ -229,8 +242,6 @@ void req_holdjob(
 
     reply_ack(preq);
     }
-
-  return;
   }  /* END req_holdjob() */
 
 
@@ -239,7 +250,7 @@ void req_holdjob(
 /*
  * req_releasejob - service the Release Job Request 
  *
- *	This request clears one or more holds on a job.
+ *  This request clears one or more holds on a job.
  *	As a result, the job might change state.
  */
 
@@ -254,6 +265,7 @@ void req_releasejob(
   job		*pjob;
   char		*pset;
   int		 rc;
+  attribute      temphold;
 
   pjob = chk_job_request(preq->rq_ind.rq_release.rq_objname,preq);
 
@@ -264,7 +276,7 @@ void req_releasejob(
 
   /* cannot do anything until we decode the holds to be set */
 
-	if ( (rc=get_hold(&preq->rq_ind.rq_hold.rq_orig.rq_attr, &pset)) != 0) {
+	if ( (rc=get_hold(&preq->rq_ind.rq_hold.rq_orig.rq_attr, &pset, &temphold)) != 0) {
 		req_reject(rc, 0, preq,NULL,NULL);
 		return;
 	}
@@ -321,10 +333,12 @@ void req_releasejob(
  *	Decode the hold attribute into temphold.
  */
 
-static int get_hold(
+int get_hold(
 
   tlist_head *phead,
-  char      **pset)  /* O - ptr to hold value */
+  char      **pset,     /* O - ptr to hold value */
+  attribute *temphold   /* O - ptr to attribute to decode value into  */
+  )
 
   {
   int		 have_one = 0;
@@ -358,10 +372,10 @@ static int get_hold(
 
   /* decode into temporary attribute structure */
 
-  clear_attr(&temphold,&job_attr_def[(int)JOB_ATR_hold]);
+  clear_attr(temphold,&job_attr_def[(int)JOB_ATR_hold]);
 
   return(job_attr_def[(int)JOB_ATR_hold].at_decode(
-    &temphold,
+    temphold,
     holdattr->al_name,
     (char *)0,
     holdattr->al_value) );
@@ -373,62 +387,61 @@ static int get_hold(
 
 
 /*
- * post_hold - A round hold in the ground in which a post is placed :-)
+ * process_hold_reply
  *	called when a hold request was sent to MOM and the answer
  *	is received.  Completes the hold request for running jobs.
  */
 
-static void post_hold(
+static void process_hold_reply(
 
   struct work_task *pwt)
-
   {
-  int		      code;
   job		     *pjob;
   struct batch_request *preq;
+  int		 newstate;
+  int		 newsub;
 
   svr_disconnect(pwt->wt_event);	/* close connection to MOM */
 
   preq = pwt->wt_parm1;
+  preq->rq_conn = preq->rq_orgconn;  /* restore client socket */
 
-  code = preq->rq_reply.brp_code;
+  if ((pjob = find_job(preq->rq_ind.rq_hold.rq_orig.rq_objname)) == (job *)0)
+    {
+    LOG_EVENT(PBSEVENT_DEBUG, PBS_EVENTCLASS_JOB,
+        preq->rq_ind.rq_hold.rq_orig.rq_objname,
+        msg_postmomnojob);
+    req_reject(PBSE_UNKJOBID, 0, preq,NULL,msg_postmomnojob);
+    }
+  else if (preq->rq_reply.brp_code != 0)
+    {
+    pjob->ji_qs.ji_substate = JOB_SUBSTATE_RUNNING;  /* reset it */
+    sprintf(log_buffer, msg_mombadhold, preq->rq_reply.brp_code);
+    LOG_EVENT(PBSEVENT_DEBUG, PBS_EVENTCLASS_JOB,
+      pjob->ji_qs.ji_jobid, log_buffer);
+    req_reject(preq->rq_reply.brp_code, 0, preq,NULL,log_buffer);
+    }
+  else
+    {
+    /* record that MOM has a checkpoint file */
 
-  preq->rq_conn = preq->rq_orgconn;	/* restore client socket */
+    /* Stupid PBS_CHKPT_MIGRATE is defined as zero therefore this code will never fire.
+     * And if these flags are not set, start_exec will not try to run the job from
+     * the checkpoint image file.
+     */
 
-	if ((pjob = find_job(preq->rq_ind.rq_hold.rq_orig.rq_objname)) == (job *)0) {
-		LOG_EVENT(PBSEVENT_DEBUG, PBS_EVENTCLASS_JOB,
-			  preq->rq_ind.rq_hold.rq_orig.rq_objname,
-			  msg_postmomnojob);
-		req_reject(PBSE_UNKJOBID, 0, preq,NULL,NULL);
-		return;
-	}
-	if (code != 0) {
-	    pjob->ji_qs.ji_substate = JOB_SUBSTATE_RUNNING;	/* reset it */
-	    if (code != PBSE_NOSUP) {
-		(void)sprintf(log_buffer, msg_mombadhold, code);
-		LOG_EVENT(PBSEVENT_DEBUG, PBS_EVENTCLASS_JOB,
-			  pjob->ji_qs.ji_jobid, log_buffer);
-		req_reject(code, 0, preq,NULL,NULL);
-		return;
-	    }
-	} else if (code == 0) {
+    pjob->ji_qs.ji_svrflags |= JOB_SVFLG_CHKPT;
+    if (preq->rq_reply.brp_auxcode)  /* chkpt can be moved */
+      {
+      pjob->ji_qs.ji_svrflags &= ~JOB_SVFLG_CHKPT;
+      pjob->ji_qs.ji_svrflags |=  JOB_SVFLG_HASRUN | JOB_SVFLG_ChkptMig;
+      }
 
-		/* record that MOM has a checkpoint file */
+    pjob->ji_modified = 1;    /* indicate attributes changed     */
+    svr_evaljobstate(pjob,&newstate,&newsub,0);
+    svr_setjobstate(pjob,newstate,newsub); /* saves job */
 
-		if (preq->rq_reply.brp_auxcode)	/* chkpt can be moved */
-		    pjob->ji_qs.ji_svrflags =
-				(pjob->ji_qs.ji_svrflags & ~JOB_SVFLG_CHKPT) |
-				JOB_SVFLG_HASRUN | JOB_SVFLG_ChkptMig;
-
-		pjob->ji_modified = 1;	  /* indicate attributes changed     */
-		job_save(pjob, SAVEJOB_QUICK);
-
-		/* note in accounting file */
-
-		account_record(PBS_ACCT_CHKPNT, pjob, (char *)0);
-	}
-
-  reply_ack(preq);
-
-  return;
+    account_record(PBS_ACCT_CHKPNT, pjob, (char *)0);  /* note in accounting file */
+    reply_ack(preq);
+    }
   }
