@@ -152,8 +152,8 @@ static struct batch_request *setup_cpyfiles A_((struct batch_request *,job *,cha
 
 static char *setup_from(
 
-  job  *pjob,
-  char *suffix)
+  job  *pjob,   /* I */
+  char *suffix) /* I */
 
   {
   char *from;
@@ -250,7 +250,7 @@ static struct batch_request *setup_cpyfiles(
     pcf = &preq->rq_ind.rq_cpyfile;
     }
  
-   pair = (struct rqfpair *)malloc(sizeof (struct rqfpair));
+   pair = (struct rqfpair *)malloc(sizeof(struct rqfpair));
 
   if (pair == NULL) 
     {
@@ -285,29 +285,39 @@ static struct batch_request *setup_cpyfiles(
 
 static int is_joined(
 
-  job          *pjob,
-  enum job_atr  ati)
+  job          *pjob,  /* I */
+  enum job_atr  ati)   /* I */
 
   {
   char       key;
   attribute *pattr;
-  char	  *pd;
+  char	    *pd;
 
-	if (ati == JOB_ATR_outpath)
-		key = 'o';
-	else if (ati == JOB_ATR_errpath)
-		key = 'e';
-	else
-		return (0);
-	pattr = &pjob->ji_wattr[(int)JOB_ATR_join];
-	if (pattr->at_flags & ATR_VFLAG_SET) {
-		pd = pattr->at_val.at_str;
-		if (pd && *pd && (*pd != 'n')) {
-			/* if not the first letter, and in list - is joined */
-			if ( (*pd != key) && (strchr(pd+1, (int)key)) )
-				return (1);	/* being joined */
-		}
-	}
+  if (ati == JOB_ATR_outpath)
+    key = 'o';
+  else if (ati == JOB_ATR_errpath)
+    key = 'e';
+  else
+    {
+    return(0);
+    }
+
+  pattr = &pjob->ji_wattr[(int)JOB_ATR_join];
+
+  if (pattr->at_flags & ATR_VFLAG_SET) 
+    {
+    pd = pattr->at_val.at_str;
+
+    if ((pd != NULL) && (*pd != '\0') && (*pd != 'n')) 
+      {
+      /* if not the first letter, and in list - is joined */
+
+      if ((*pd != key) && (strchr(pd + 1,(int)key)))
+        {
+        return(1);	/* being joined */
+        }
+      }
+    }
 
   return(0);	/* either the first or not in list */
   }
@@ -594,13 +604,24 @@ int mom_comm(
 
     if (pjob->ji_momhandle < 0) 
       {
+      /* FAILURE */
+
+      if (LOGLEVEL >= 2)
+        {
+        log_event(
+          PBSEVENT_ERROR|PBSEVENT_JOB,
+          PBS_EVENTCLASS_JOB,
+          pjob->ji_qs.ji_jobid,
+          "cannot establish connection with mom for clean-up - will retry later");
+        }
+
       pwt = set_task(
         WORK_Timed,
         (long)(time_now + PBS_NET_RETRY_TIME), 
         func, 
         (void *)pjob);
 
-      if (pwt)
+      if (pwt != NULL)
         {
         /* insure that work task will be removed if job goes away */
 
@@ -609,7 +630,7 @@ int mom_comm(
 
       return(-1);
       }
-    }
+    }    /* END if (pjob->ji_momhandle < 0) */
 
   return(pjob->ji_momhandle);
   }  /* END mom_comm() */
@@ -668,11 +689,14 @@ void on_job_exit(
   {
   int    handle;
   job   *pjob;
+#ifdef VNODETESTING
+  job   *pj;
+#endif
   struct batch_request *preq;
 
   int    IsFaked = 0;
   int	 KeepSeconds = 0;
-  pbs_queue          *pque;
+  pbs_queue *pque;
 
   extern void remove_job_delete_nanny(struct job *);
 
@@ -689,12 +713,51 @@ void on_job_exit(
     pjob = (job *)preq->rq_extra;
     }
 
-  if ((handle = mom_comm(pjob, on_job_exit)) < 0)
+#ifdef VNODETESTING
+/* FIXME: there might be a race with calling on_job_exit after a job has
+ * already been free'd.  This is temp code */
+
+  pj = (job *)GET_NEXT(svr_alljobs);
+  
+  while (pj != NULL)
     {
+    if (pjob == pj)
+      break;
+    
+    pj = (job *)GET_NEXT(pj->ji_alljobs);
+    }
+  
+  if (pj == NULL)
+    {
+    sprintf(log_buffer,"on_job_exit called with INVALID pjob: %p",
+      pjob);
+    }
+  else
+    {
+    sprintf(log_buffer,"on_job_exit valid pjob: %p (substate=%d)",
+      pjob,
+      pjob->ji_qs.ji_substate);
+    }
+
+  log_event(
+    PBSEVENT_JOB,
+    PBS_EVENTCLASS_JOB,
+    pjob->ji_qs.ji_jobid,
+    log_buffer);
+
+  DBPRT(("%s\n",
+    log_buffer));
+#endif /* END VNODETESTING */
+
+  if ((handle = mom_comm(pjob,on_job_exit)) < 0)
+    {
+    /* FAILURE - cannot connect to mom */
+
     return;
     }		
 
   /* MOM has killed everything it can kill, so we can stop the nanny */
+
   remove_job_delete_nanny(pjob);
 
   switch (pjob->ji_qs.ji_substate) 
@@ -702,6 +765,15 @@ void on_job_exit(
     case JOB_SUBSTATE_EXITING:
     case JOB_SUBSTATE_ABORT:
 
+      if (LOGLEVEL >= 2)
+        {
+        log_event(
+          PBSEVENT_JOB,
+          PBS_EVENTCLASS_JOB,
+          pjob->ji_qs.ji_jobid,
+          "JOB_SUBSTATE_EXITING");
+        }
+
       /* see if job has any dependencies */
 
       if (pjob->ji_wattr[(int)JOB_ATR_depend].at_flags & ATR_VFLAG_SET) 
@@ -720,6 +792,15 @@ void on_job_exit(
 
     case JOB_SUBSTATE_STAGEOUT:
 
+      if (LOGLEVEL >= 4)
+        {
+        log_event(
+          PBSEVENT_JOB,
+          PBS_EVENTCLASS_JOB,
+          pjob->ji_qs.ji_jobid,
+          "JOB_SUBSTATE_STAGEOUT");
+        }
+
       IsFaked = 0;
 
       if (ptask->wt_type != WORK_Deferred_Reply) 
@@ -738,10 +819,30 @@ void on_job_exit(
           {
           /* have files to copy */
 
+          if (LOGLEVEL >= 4)
+            {
+            log_event(
+              PBSEVENT_JOB,
+              PBS_EVENTCLASS_JOB,
+              pjob->ji_qs.ji_jobid,
+              "about to copy stdout/stderr/stageout files");
+            }
+
           preq->rq_extra = (void *)pjob;
 
           if (issue_Drequest(handle,preq,on_job_exit,0) == 0) 
             {
+            /* FAILURE */
+
+            if (LOGLEVEL >= 1)
+              {
+              log_event(
+                PBSEVENT_JOB,
+                PBS_EVENTCLASS_JOB,
+                pjob->ji_qs.ji_jobid,
+                "copy request failed");
+              }
+
             /* come back when mom replies */
 
             return;
@@ -766,9 +867,18 @@ void on_job_exit(
             JOB_STATE_EXITING,
             JOB_SUBSTATE_STAGEDEL);
 
+          if (LOGLEVEL >= 4)
+            {
+            log_event(
+              PBSEVENT_JOB,
+              PBS_EVENTCLASS_JOB,
+              pjob->ji_qs.ji_jobid,
+              "no files to copy - deleting job");
+            }
+
           ptask = set_task(WORK_Immed,0,on_job_exit,pjob);
 
-          if (ptask)
+          if (ptask != NULL)
             {
             append_link(&pjob->ji_svrtask,&ptask->wt_linkobj,ptask);
             }
@@ -852,6 +962,15 @@ void on_job_exit(
 
     case JOB_SUBSTATE_STAGEDEL:
 
+      if (LOGLEVEL >= 4)
+        {
+        log_event(
+          PBSEVENT_JOB,
+          PBS_EVENTCLASS_JOB,
+          pjob->ji_qs.ji_jobid,
+          "JOB_SUBSTATE_STAGEDEL");
+        }
+
       if (ptask->wt_type != WORK_Deferred_Reply) 
         { 
         /* first time in */
@@ -874,6 +993,17 @@ void on_job_exit(
 
           if (issue_Drequest(handle,preq,on_job_exit,0) == 0) 
             {
+            /* FAILURE */
+
+            if (LOGLEVEL >= 2)
+              {
+              log_event(
+                PBSEVENT_JOB,
+                PBS_EVENTCLASS_JOB,
+                pjob->ji_qs.ji_jobid,
+                "cannot issue file delete request for staged files");
+              }
+
             /* come back when mom replies */
 
             return;
@@ -957,6 +1087,15 @@ void on_job_exit(
 
     case JOB_SUBSTATE_EXITED:
 
+      if (LOGLEVEL >= 4)
+        {
+        log_event(
+          PBSEVENT_JOB,
+          PBS_EVENTCLASS_JOB,
+          pjob->ji_qs.ji_jobid,
+          "JOB_SUBSTATE_EXITED");
+        }
+
       /* tell mom to delete the job, send final track and purge it */
 
       preq = alloc_br(PBS_BATCH_DeleteJob);
@@ -990,11 +1129,21 @@ void on_job_exit(
     
     case JOB_SUBSTATE_COMPLETE:
 
+      if (LOGLEVEL >= 4)
+        {
+        log_event(
+          PBSEVENT_JOB,
+          PBS_EVENTCLASS_JOB,
+          pjob->ji_qs.ji_jobid,
+          "JOB_SUBSTATE_COMPLETE");
+        }
+
       if ((pque = pjob->ji_qhdr) && (pque->qu_attr != NULL))
         {
-        KeepSeconds = attr_ifelse_long(&pque->qu_attr[(int)QE_ATR_KeepCompleted],
-                                       &server.sv_attr[(int)SRV_ATR_KeepCompleted],
-                                       0);
+        KeepSeconds = attr_ifelse_long(
+          &pque->qu_attr[(int)QE_ATR_KeepCompleted],
+          &server.sv_attr[(int)SRV_ATR_KeepCompleted],
+          0);
         }
 
       if (KeepSeconds <= 0)
@@ -1010,7 +1159,7 @@ void on_job_exit(
 
         ptask = set_task(WORK_Timed,time_now + KeepSeconds,on_job_exit,pjob);
 
-        if (ptask)
+        if (ptask != NULL)
           {
           /* insure that work task will be removed if job goes away */
 
@@ -1068,7 +1217,7 @@ void on_job_rerun(
     pjob = (job *)preq->rq_extra;
     }
 
-  if ((handle = mom_comm(pjob, on_job_rerun)) < 0)
+  if ((handle = mom_comm(pjob,on_job_rerun)) < 0)
     {
     return;
     }
@@ -1387,7 +1536,7 @@ void on_job_rerun(
       if ((pjob->ji_qs.ji_svrflags & JOB_SVFLG_HOTSTART) == 0) 
         {
         /* in case of server shutdown, don't clear exec_host */
-        /* and session_id.  Will use it on hotstart when next comes up */
+        /* or session_id will use it on hotstart when next comes up */
 
         job_attr_def[(int)JOB_ATR_exec_host].at_free(
           &pjob->ji_wattr[(int)JOB_ATR_exec_host]);
@@ -1469,7 +1618,7 @@ static int setrerun(
 
 void req_jobobit(
 
-  struct batch_request *preq)
+  struct batch_request *preq)  /* I */
 
   {
   int		  alreadymailed = 0;
@@ -1484,10 +1633,12 @@ void req_jobobit(
   int		  newsubst;
   char		 *pc;
   job		 *pjob;
+  char        jobid[PBS_MAXSVRJOBID+1];
   struct work_task *ptask;
   svrattrl	 *patlist;
-  unsigned int dummy;
+  unsigned int    dummy;
 
+  strcpy(jobid, preq->rq_ind.rq_jobobit.rq_jid);  /* This will be needed later for logging after preq is freed. */
   pjob = find_job(preq->rq_ind.rq_jobobit.rq_jid);
 
   if ((pjob == NULL) ||
@@ -1519,11 +1670,11 @@ void req_jobobit(
     log_event(
       PBSEVENT_ERROR|PBSEVENT_JOB,
       PBS_EVENTCLASS_JOB,
-      preq->rq_ind.rq_jobobit.rq_jid,
+      jobid,
       log_buffer);
 
     return;
-    }  /* END if (pjob == NULL) */
+    }  /* END if ((pjob == NULL) || ...) */
 
   if (pjob->ji_qs.ji_state != JOB_STATE_RUNNING)  
     {
@@ -1540,14 +1691,14 @@ void req_jobobit(
       /* NOTE:  was logged w/msg_obitnojob */
 
       sprintf(log_buffer,"obit received for job %s from host %s with bad state (state: %s)",
-        preq->rq_ind.rq_jobobit.rq_jid,
+        jobid,
         preq->rq_host, 
         PJobState[pjob->ji_qs.ji_state]);
 
       log_event(
         PBSEVENT_ERROR|PBSEVENT_JOB,
         PBS_EVENTCLASS_JOB,
-        preq->rq_ind.rq_jobobit.rq_jid,
+        jobid,
         log_buffer);
 
       bad = PBSE_BADSTATE;
@@ -1561,7 +1712,7 @@ void req_jobobit(
       NULL);
 
     return;
-    }
+    }  /* END if (pjob->ji_qs.ji_state != JOB_STATE_RUNNING) */
 
   if (pjob->ji_qs.ji_substate == JOB_SUBSTATE_PRERUN) 
     {
@@ -1591,11 +1742,21 @@ void req_jobobit(
   pjob->ji_qs.ji_un.ji_exect.ji_exitstat = exitstatus;
 
   pjob->ji_wattr[(int)JOB_ATR_exitstat].at_val.at_long = exitstatus;
-  pjob->ji_wattr[(int)JOB_ATR_exitstat].at_flags |=ATR_VFLAG_SET;
+  pjob->ji_wattr[(int)JOB_ATR_exitstat].at_flags |= ATR_VFLAG_SET;
 
   patlist = (svrattrl *)GET_NEXT(preq->rq_ind.rq_jobobit.rq_attr);
  
   /* Encode the final resources_used into the job (useful for keep_completed) */
+
+  if (LOGLEVEL >= 2)
+    {
+    log_event(
+      PBSEVENT_ERROR|PBSEVENT_JOB,
+      PBS_EVENTCLASS_JOB,
+      jobid,
+      "obit received - updating final job usage info");
+    }
+
   modify_job_attr(
     pjob,
     patlist,
@@ -1621,8 +1782,11 @@ void req_jobobit(
 
   while (patlist != NULL) 
     {
-    need = strlen(patlist->al_resc) + strlen(patlist->al_value) + 3;
-
+    need = strlen(patlist->al_name) + strlen(patlist->al_value) + 3;
+    if (patlist->al_resc) {
+      need += strlen(patlist->al_resc) + 3;
+    }
+    
     if (need < amt) 
       {
       strcat(acctbuf,"\n");
@@ -1711,6 +1875,25 @@ void req_jobobit(
           /* have mom remove job files, not saving them, and requeue job */ 
 
           pjob->ji_qs.ji_substate = JOB_SUBSTATE_RERUN1;
+
+          /* transient failure detected */
+
+          /* load session id info from prq->rq_ind.rq_jobobit->rq_attr->Session */
+
+          /*
+          memset(&tA,0,sizeof(tA));
+
+          tA.al_name  = "sched_hint";
+          tA.al_resc  = "";
+          tA.al_value = log_buffer;
+          tA.al_op    = SET;
+
+          modify_job_attr(
+            pjob,
+            &tA,                           
+            ATR_DFLAG_MGWR | ATR_DFLAG_SvWR,
+            &bad);
+          */
           }
 
         break;
@@ -1729,6 +1912,15 @@ void req_jobobit(
         /* MOM abort job on init, job has checkpoint file	*/
         /* Requeue it, and thats all folks.			*/
 
+        if (LOGLEVEL >= 1)
+          {
+          log_event(
+            PBSEVENT_JOB_USAGE|PBSEVENT_JOB_USAGE,
+            PBS_EVENTCLASS_JOB,
+            pjob->ji_qs.ji_jobid,
+            "received JOB_EXEC_INITRST, setting job CHKPT flag");
+          }
+
         rel_resc(pjob);
 
         pjob->ji_qs.ji_svrflags |= JOB_SVFLG_HASRUN|JOB_SVFLG_CHKPT;
@@ -1748,7 +1940,7 @@ void req_jobobit(
       case JOB_EXEC_INITRMG:
 
         /* MOM abort job on init, job has migratable checkpoint */
-        /* Must recover output and checkpoint file, do eoj	    */
+        /* Must recover output and checkpoint file, do eoj      */
 
         alreadymailed = setrerun(pjob);
 
@@ -1758,6 +1950,18 @@ void req_jobobit(
       }  /* END switch (exitstatus) */
     }    /* END if (exitstatus < 0) */
 
+  if (LOGLEVEL >= 2)
+    {
+    sprintf(log_buffer,"job exit status %d handled",
+      exitstatus);
+
+    log_event(
+      PBSEVENT_ERROR|PBSEVENT_JOB,
+      PBS_EVENTCLASS_JOB,
+      jobid,
+      log_buffer);
+    }
+
   /* What do we now do with the job... */
 
   if ((pjob->ji_qs.ji_substate != JOB_SUBSTATE_RERUN) &&
@@ -1781,7 +1985,7 @@ void req_jobobit(
 
     /* record accounting and maybe in log */
 
-    account_jobend(pjob, acctbuf);
+    account_jobend(pjob,acctbuf);
 
     if (server.sv_attr[(int)SRV_ATR_log_events].at_val.at_long & PBSEVENT_JOB_USAGE) 
       {
@@ -1806,11 +2010,20 @@ void req_jobobit(
         acctbuf);
       }
 			
-    ptask = set_task(WORK_Immed, 0, on_job_exit, (void *)pjob);
+    ptask = set_task(WORK_Immed,0,on_job_exit,(void *)pjob);
 
-    if (ptask)
+    if (ptask != NULL)
       {
       append_link(&pjob->ji_svrtask,&ptask->wt_linkobj,ptask);
+
+      if (LOGLEVEL >= 4)
+        {
+        log_event(
+          PBSEVENT_ERROR|PBSEVENT_JOB,
+          PBS_EVENTCLASS_JOB,
+          jobid,
+          "on_job_exit task assigned to job");
+        }
       }
 
     /* "on_job_exit()" will be dispatched out of the main loop */
@@ -1832,7 +2045,7 @@ void req_jobobit(
 
       pjob->ji_qs.ji_svrflags |= JOB_SVFLG_HASRUN;
 
-      svr_evaljobstate(pjob,&newstate,&newsubst, 1);
+      svr_evaljobstate(pjob,&newstate,&newsubst,1);
 
       svr_setjobstate(pjob,newstate,newsubst);
 
@@ -1848,12 +2061,33 @@ void req_jobobit(
 
     ptask = set_task(WORK_Immed,0,on_job_rerun,(void *)pjob);
 
-    if (ptask)
+    if (ptask != NULL)
       {
       append_link(&pjob->ji_svrtask,&ptask->wt_linkobj,ptask);
+
+      if (LOGLEVEL >= 4)
+        {
+        log_event(
+          PBSEVENT_ERROR|PBSEVENT_JOB,
+          PBS_EVENTCLASS_JOB,
+          jobid,
+          "on_job_rerun task assigned to job");
+        }
       }
 
     /* "on_job_rerun()" will be dispatched out of the main loop */
+    }  /* END else */
+
+  if (LOGLEVEL >= 4)
+    {
+    sprintf(log_buffer,"job exit status %d handled",
+      exitstatus);
+
+    log_event(
+      PBSEVENT_ERROR|PBSEVENT_JOB,
+      PBS_EVENTCLASS_JOB,
+      jobid,
+      "req_jobobit completed");
     }
 
   return;
