@@ -97,7 +97,7 @@
  *      mgr_node_delete()       - top level function for effectively deleting a node
  *      mgr_node_create()       - top level function for creating a node
  *	req_manager()		- process manager request (top level)
- *	
+ *	disallowed_types_chk()	- at action for queue disallowed_types attribute
  */
 #include <pbs_config.h>   /* the master config generated by configure */
 
@@ -123,6 +123,7 @@
 #include "svrfunc.h"
 #include "job.h"
 #include "pbs_nodes.h"
+#include "work_task.h"
 
 
 #define PERM_MANAGER (ATR_DFLAG_MGWR | ATR_DFLAG_MGRD)
@@ -134,6 +135,7 @@ extern struct server server;
 extern tlist_head     svr_queues;
 extern attribute_def que_attr_def[];
 extern attribute_def svr_attr_def[];
+extern attribute_def  node_attr_def[];   /* node attributes defs */
 extern int  svr_chngNodesfile;
 extern char *msg_attrtype;
 extern char *msg_daemonname;
@@ -509,34 +511,45 @@ int mgr_unset_attr(
 	
   while (plist != NULL) 
     {
-		index = find_attr(pdef, plist->al_name, limit);
-		if ( ((pdef+index)->at_type == ATR_TYPE_RESC) &&
-		     (plist->al_resc != (char *)0) )   {
+    index = find_attr(pdef,plist->al_name,limit);
 
-			/* free resource member, not the attribute */
+    if (((pdef + index)->at_type == ATR_TYPE_RESC) &&
+         (plist->al_resc != NULL))   
+      {
+      /* free resource member, not the attribute */
 
-			prsdef = find_resc_def(svr_resc_def, plist->al_resc,
-					       svr_resc_size);
-			if ((presc = find_resc_entry(pattr+index, prsdef))) {
-			    if ( (pdef->at_parent != PARENT_TYPE_SERVER) ||
-				 (index != (int)SRV_ATR_resource_cost) )   {
-					prsdef->rs_free(&presc->rs_value);
-			    }
-			    delete_link(&presc->rs_link);
-			}
-			free(presc);
+      prsdef = find_resc_def(
+        svr_resc_def, 
+        plist->al_resc,
+        svr_resc_size);
 
-		} else {
+      if ((presc = find_resc_entry(pattr + index,prsdef))) 
+        {
+        if ((pdef->at_parent != PARENT_TYPE_SERVER) ||
+            (index != (int)SRV_ATR_resource_cost))   
+          {
+          prsdef->rs_free(&presc->rs_value);
+          }
 
-			/* free the whole attribute */
+        delete_link(&presc->rs_link);
+        }
 
-			(pdef+index)->at_free(pattr+index);
-			(pattr+index)->at_flags |= ATR_VFLAG_MODIFY;
-		}
-		plist = (svrattrl *)GET_NEXT(plist->al_link);
-	}
-	return (0);
-}
+      free(presc);
+      } 
+    else 
+      {
+      /* free the whole attribute */
+
+      (pdef + index)->at_free(pattr+index);
+
+      (pattr + index)->at_flags |= ATR_VFLAG_MODIFY;
+      }
+
+    plist = (svrattrl *)GET_NEXT(plist->al_link);
+    }  /* END while (plist != NULL) */
+
+  return(0);
+  }  /* END mgr_unset_attr() */
 
 
 
@@ -973,22 +986,45 @@ void mgr_queue_delete(
   pbs_queue *pque;
   int    rc;
 
-	pque = find_queuebyname(preq->rq_ind.rq_manager.rq_objname);
-	if (pque == (pbs_queue *)0) {
-		req_reject(PBSE_UNKQUE, 0, preq,NULL,NULL);
-		return;
-	}
-	if ((rc = que_purge(pque)) != 0)
-		req_reject(rc, 0, preq,NULL,NULL);
-	else {
-		svr_save(&server, SVR_SAVE_QUICK);
-		(void)sprintf(log_buffer, msg_manager, msg_man_del,
-			      preq->rq_user, preq->rq_host);
-		log_event(PBSEVENT_ADMIN, PBS_EVENTCLASS_QUEUE, 
-			  preq->rq_ind.rq_manager.rq_objname, log_buffer);
-		reply_ack(preq);
-	}
-}
+  pque = find_queuebyname(preq->rq_ind.rq_manager.rq_objname);
+
+  if (pque == NULL) 
+    {
+    /* FAILURE */
+
+    req_reject(PBSE_UNKQUE,0,preq,NULL,NULL);
+
+    return;
+    }
+
+  if ((rc = que_purge(pque)) != 0)
+    {
+    /* FAILURE */
+
+    req_reject(rc,0,preq,NULL,NULL);
+
+    return;
+    }
+
+  svr_save(&server,SVR_SAVE_QUICK);
+
+  sprintf(log_buffer,msg_manager, 
+    msg_man_del,
+    preq->rq_user, 
+    preq->rq_host);
+
+  log_event(
+    PBSEVENT_ADMIN, 
+    PBS_EVENTCLASS_QUEUE, 
+    preq->rq_ind.rq_manager.rq_objname, 
+    log_buffer);
+
+  reply_ack(preq);
+
+  /* SUCCESS */
+
+  return;
+  }  /* END mgr_queue_delete() */
 
 
 
@@ -1001,7 +1037,7 @@ void mgr_queue_delete(
 
 void mgr_server_set(
 
-  struct batch_request *preq)
+  struct batch_request *preq) /* I */
 
   {
   int       bad_attr = 0;
@@ -1022,6 +1058,7 @@ void mgr_server_set(
     ATR_ACTION_ALTER);
 
   /* PBSE_BADACLHOST - lets show the user the first bad host in the ACL  */
+
   if (rc == PBSE_BADACLHOST)
     {
     char     *bad_host;
@@ -1034,22 +1071,25 @@ void mgr_server_set(
     int       bhstrlen;
 
     bhstrlen = PBS_MAXHOSTNAME + 17;
-    bad_host = malloc(sizeof(char) * (bhstrlen+1));
+    bad_host = malloc(sizeof(char) * (bhstrlen + 1));
 
-    if (bad_host==NULL)
+    if (bad_host == NULL)
       {
-      reply_badattr(PBSE_BADHOST, bad_attr, plist, preq);
+      reply_badattr(PBSE_BADHOST,bad_attr,plist,preq);
 
       return;
       }
 
     /* look into plist to find an offending attr */
+
     while (plist != NULL)
       {
       /* only concerned about operators and managers */
+
       if (strcmp(plist->al_name,ATTR_managers) && strcmp(plist->al_name,ATTR_operators))
         {
         plist = (svrattrl *)GET_NEXT(plist->al_link);
+
         continue;
         }
 
@@ -1064,17 +1104,21 @@ void mgr_server_set(
     
       /* loop over all hosts in the request and perform same
          check as manager_oper_chk*/
-      for (i = 0; i < pstr->as_usedptr; ++i)
+
+      for (i = 0;i < pstr->as_usedptr;++i)
         {
-        host_entry = strchr(pstr->as_string[i], (int)'@');
+        host_entry = strchr(pstr->as_string[i],(int)'@');
  
         /* if wildcard, we can't check */
+
         if ((host_entry != NULL) && host_entry[1] != '*')
           {
-          if (get_fullhostname(host_entry+1,hostname,PBS_MAXHOSTNAME) ||
-              strncmp(host_entry+1,hostname,PBS_MAXHOSTNAME))
+          if (get_fullhostname(host_entry + 1,hostname,PBS_MAXHOSTNAME,NULL) ||
+              strncmp(host_entry + 1,hostname,PBS_MAXHOSTNAME))
             {
-            snprintf(bad_host,bhstrlen,"First bad host: %s",host_entry+1);
+            snprintf(bad_host,bhstrlen,"First bad host: %s",
+              host_entry + 1);
+
             break;
             }
           }
@@ -1085,6 +1129,7 @@ void mgr_server_set(
 
       /* nothing wrong found in the request, let's try again
          with the server's list */
+
       pstr = server.sv_attr[(int)index].at_val.at_arst;
 
       for (i = 0; i < pstr->as_usedptr; ++i)
@@ -1094,10 +1139,12 @@ void mgr_server_set(
         /* if wildcard, we can't check */
         if ((host_entry != NULL) && host_entry[1] != '*')
           {
-          if (get_fullhostname(host_entry+1,hostname,PBS_MAXHOSTNAME) ||
+          if (get_fullhostname(host_entry+1,hostname,PBS_MAXHOSTNAME,NULL) ||
               strncmp(host_entry+1,hostname,PBS_MAXHOSTNAME))
             {
-            snprintf(bad_host,bhstrlen,"First bad host: %s",host_entry+1);
+            snprintf(bad_host,bhstrlen,"First bad host: %s",
+              host_entry + 1);
+
             break;
             }
           }
@@ -1111,15 +1158,18 @@ void mgr_server_set(
 
     if (bad_host[0] != '\0') /* we found a fully qualified host that was bad */
       {
-      req_reject(PBSE_BADACLHOST, 0, preq, NULL, bad_host); 
+      req_reject(PBSE_BADACLHOST,0,preq,NULL,bad_host); 
       }
-    else /* this shouldn't happen (return PBSE_BADACLHOST, but now we can't find the bad host) */
+    else 
       {
-      reply_badattr(PBSE_BADHOST, bad_attr, plist, preq);
+      /* this shouldn't happen (return PBSE_BADACLHOST, but now we can't 
+         find the bad host) */
+
+      reply_badattr(PBSE_BADHOST,bad_attr,plist,preq);
       }
 
     return;
-    } /* end PBSE_BADACLHOST */
+    } /* END if (rc == PBSE_BADACLHOST) */
   else if (rc != 0)
     {
     reply_badattr(rc,bad_attr,plist,preq);
@@ -1275,6 +1325,7 @@ void mgr_queue_set(
            (void *)pque,
            ATR_ACTION_ALTER);
 
+ 
     if (rc != 0) 
       {
       reply_badattr(rc,bad,plist,preq);
@@ -1449,7 +1500,7 @@ void mgr_node_set(
   if ((*preq->rq_ind.rq_manager.rq_objname == '\0') ||
       (*preq->rq_ind.rq_manager.rq_objname == '@') ||
       ((*preq->rq_ind.rq_manager.rq_objname == ':') &&   
-      (*(preq->rq_ind.rq_manager.rq_objname+1) != '\0')))    
+      (*(preq->rq_ind.rq_manager.rq_objname + 1) != '\0')))    
     {
     /*In this instance the set node req is to apply to all */
     /*nodes at the local ('\0')  or specified ('@') server */
@@ -1458,11 +1509,11 @@ void mgr_node_set(
       {
       pnode = *pbsndlist;
       if ((*preq->rq_ind.rq_manager.rq_objname == ':') && 
-          (strcmp(preq->rq_ind.rq_manager.rq_objname+1,"ALL") != 0))
+          (strcmp(preq->rq_ind.rq_manager.rq_objname + 1,"ALL") != 0))
         {
         propnodes = 1;
         nodename = preq->rq_ind.rq_manager.rq_objname;
-        props.name = nodename+1;
+        props.name = nodename + 1;
         props.mark = 1;
         props.next = NULL;
         }
@@ -1614,46 +1665,60 @@ void mgr_node_set(
     need_todo &= ~(WRITE_NEW_NODESFILE);	 /*successful on update*/
     }
 
-	if (allnodes || propnodes) {          /*modification was for all nodes  */ 
+  if (allnodes || propnodes) 
+    {
+    /* modification was for all nodes */ 
 
-	   if ( problem_cnt ) {  /*one or more problems encountered*/
+    if (problem_cnt) 
+      {  
+      /* one or more problems encountered */
 		
-		for( len=0, i=0; i<problem_cnt; i++ )
-		     len += strlen( problem_nodes[i]->nd_name ) + 3;
+      for (len = 0,i = 0;i < problem_cnt;i++)
+        len += strlen(problem_nodes[i]->nd_name) + 3;
 
-		len += strlen (pbse_to_txt (PBSE_GMODERR));
+      len += strlen (pbse_to_txt(PBSE_GMODERR));
 
-		if ((problem_names = malloc (len))) {
+      if ((problem_names = malloc(len))) 
+        {
+        strcpy(problem_names,pbse_to_txt(PBSE_GMODERR));
 
-		    strcpy (problem_names, pbse_to_txt (PBSE_GMODERR));
-		    for (i=0; i<problem_cnt; i++) {
-			 if (i)
-			     strcat (problem_names, ", ");
-			 strcat (problem_names, problem_nodes[i]->nd_name);
-		    }
+        for (i = 0;i < problem_cnt;i++) 
+          {
+          if (i)
+            strcat(problem_names,", ");
 
-		    reply_text (preq, PBSE_GMODERR, problem_names);
-		    free (problem_names);
-		} else {
-		    reply_text (preq, PBSE_GMODERR, pbse_to_txt (PBSE_GMODERR));
-		}
-	    }
+          strcat(problem_names,problem_nodes[i]->nd_name);
+          }
 
-	    if (problem_nodes)
-		free (problem_nodes);
+        reply_text(preq,PBSE_GMODERR,problem_names);
 
-	    if (problem_cnt) {		/*reply has already been sent  */
-	 	recompute_ntype_cnts ();
-	        return ;
-	    }
-	}
+        free(problem_names);
+        } 
+      else 
+        {
+        reply_text(preq,PBSE_GMODERR,pbse_to_txt(PBSE_GMODERR));
+        }
+      }
 
-  recompute_ntype_cnts ();
+    if (problem_nodes != NULL)
+      free(problem_nodes);
+
+    if (problem_cnt) 
+      {
+      /* reply has already been sent */
+
+      recompute_ntype_cnts();
+
+      return;
+      }
+    }    /* END if (allnodes || propnodes) */
+
+  recompute_ntype_cnts();
 
   reply_ack(preq);		/*request completely successful*/
 
   return;
-  }
+  }  /* END void mgr_node_set() */
 
 
 
@@ -2146,7 +2211,7 @@ int manager_oper_chk(
 
       /* if not wild card, must be fully qualified host */
 
-      if (get_fullhostname(entry,hostname,PBS_MAXHOSTNAME) ||
+      if (get_fullhostname(entry,hostname,PBS_MAXHOSTNAME,NULL) ||
           strncmp(entry,hostname,PBS_MAXHOSTNAME)) 
         {
         if (actmode == ATR_ACTION_RECOV) 
@@ -2184,9 +2249,9 @@ int manager_oper_chk(
 
 int servername_chk(
 
-  attribute *pattr,
-  void      *pobject,
-  int	     actmode)
+  attribute *pattr,   /* I */
+  void      *pobject, /* I */
+  int	     actmode) /* I */
 
   {
   int			 err = 0;
@@ -2198,7 +2263,7 @@ int servername_chk(
   if (actmode == ATR_ACTION_FREE)
     {
     if ((gethostname(hostname,PBS_MAXHOSTNAME) == -1) ||
-       (get_fullhostname(hostname,hostname,PBS_MAXHOSTNAME) == -1))
+       (get_fullhostname(hostname,hostname,PBS_MAXHOSTNAME,NULL) == -1))
       {
       return(PBSE_BADHOST);
       }
@@ -2217,7 +2282,7 @@ int servername_chk(
 
   /* must be fully qualified host */
 
-  if (get_fullhostname(pstr,hostname,PBS_MAXHOSTNAME) ||
+  if (get_fullhostname(pstr,hostname,PBS_MAXHOSTNAME,NULL) ||
       strncmp(pstr,hostname,PBS_MAXHOSTNAME)) 
     {
     if (actmode == ATR_ACTION_RECOV) 
@@ -2242,6 +2307,51 @@ int servername_chk(
   return(err);
   }  /* END server_name_chk() */
 
+
+/*
+ * extra_resc_chk() called when extra_resc server attribute is changed
+ *     It's purpose is to re-init the resource definitions by calling
+ *     init_resc_defs().  Unfortunately, it gets call before the
+ *     change is actually committed; so we setup a work task to call
+ *     it async asap.
+ */
+void on_extra_resc(
+
+  struct work_task *ptask) 
+  {
+  init_resc_defs();
+  }
+int extra_resc_chk(
+
+  attribute *pattr,   /* I */
+  void      *pobject, /* I */
+  int	     actmode) /* I */
+
+  {
+  struct work_task *ptask;
+
+  /* Is there anything to validate?  Maybe check for all alphanum? */
+  /* the new resource is at pattr->at_val.at_str */
+  ptask=set_task(WORK_Immed,0,on_extra_resc,NULL);
+
+  return (ptask == NULL);
+}
+
+/*
+ * free_extraresc() makes sure that the init_resc_defs() is called after
+ * the list has changed by 'unset'.
+ */
+void free_extraresc(
+
+  struct attribute *attr)
+
+  {
+  set_task(WORK_Immed,0,on_extra_resc,NULL);
+
+  free_arst(attr);
+}
+
+
 /*
  * disallowed_types_chk -
  *      This is the at_action() routine for the queue disallowed_type attribute
@@ -2301,7 +2411,8 @@ int disallowed_types_chk(
 
   return(0);
   } /* END disallowed_types_chk() */
-  
+
+
 static int mgr_long_action_helper(
 
   attribute *pattr,
@@ -2353,30 +2464,28 @@ int schiter_chk(
   }  /* END schiter_chk() */
 
 /* nextjobnum_action - makes sure value is sane (>=0)
-   actually updates server.sv_qs.sv_jobidnumber
-   unsets attribute so it is "hidden" in qmgr */
+   Note that there must be special code in svr_recov
+   to prevent the attribute value from clobbering
+   the internal counter server.sv_qs.sv_jobidnumber.
+ */
 int nextjobnum_chk(
 
   attribute *pattr,
   void      *pobject,
   int        actmode)
   {
-
-  if (pattr->at_val.at_long >= 0)
-    {
-    server.sv_qs.sv_jobidnumber = pattr->at_val.at_long;
-    pattr->at_flags &= ~ATR_VFLAG_SET;
-    svr_save(&server,SVR_SAVE_FULL);
-    return(PBSE_NONE);
-    }
-  else if (pattr->at_val.at_long > PBS_SEQNUMTOP)
+  if (pattr->at_val.at_long > PBS_SEQNUMTOP)
     {
     return(PBSE_EXLIMIT);
     }
+  else if (pattr->at_val.at_long >= 0)
+    {
+    return(PBSE_NONE);
+    }
   else
     {
     return(PBSE_BADATVAL);
-    }
+    } 
   }
 
 int set_nextjobnum(
@@ -2386,28 +2495,34 @@ int set_nextjobnum(
   enum batch_op op)
 
   {
-  /* this is almost identical to set_l, but we need to grab the
-     current value of server.sv_qs.sv_jobidnumber before we INCR/DECR the value of
+  /* this is almost identical to set_l, but we need to grab the 
+     current value of server.sv_qs.sv_jobidnumber before we INCR/DECR the value of 
      attr->at_val.at_long.  In fact, it probably should be moved to Libattr/ */
 
+  /* The special handling of INCR is to allow setting of a job number range.
+   * If the job numbers are already in the range, it does not alter the number.
+   * Otherwise, if the number is in the default range, it sets it to the new range.
+   */
 
-  switch (op)
+  switch (op) 
     {
     case SET:   attr->at_val.at_long = new->at_val.at_long;
                 break;
 
-    case INCR:  attr->at_val.at_long = server.sv_qs.sv_jobidnumber += new->at_val.at_long;
+    /*case INCR:  attr->at_val.at_long = MAX(server.sv_qs.sv_jobidnumber, new->at_val.at_long);*/
+    case INCR:  attr->at_val.at_long += new->at_val.at_long;
                 break;
 
-    case DECR:  attr->at_val.at_long = server.sv_qs.sv_jobidnumber -= new->at_val.at_long;
+    case DECR:  attr->at_val.at_long -= new->at_val.at_long;
                 break;
-
     default: return(PBSE_SYSTEM);
     }
   attr->at_flags |= ATR_VFLAG_SET | ATR_VFLAG_MODIFY;
+
+  server.sv_qs.sv_jobidnumber = attr->at_val.at_long;
+  svr_save(&server,SVR_SAVE_QUICK);
+
   return 0;
   }
-
-
 /* END req_manager.c */
 
