@@ -211,7 +211,8 @@ void req_modifyjob(
 
   /* if job is running, special checks must be made */
 
-  if (pjob->ji_qs.ji_state == JOB_STATE_RUNNING) 
+  if (pjob->ji_qs.ji_state == JOB_STATE_RUNNING ||
+      pjob->ji_qs.ji_state == JOB_STATE_HELD) 
     {
     while (plist != NULL) 
       {
@@ -259,6 +260,10 @@ void req_modifyjob(
 
         sendmom = 1;
         }
+      else if (i == (int)JOB_ATR_chkptname) 
+        {
+        sendmom = 1;
+        }
 
       plist = (svrattrl *)GET_NEXT(plist->al_link);
       }
@@ -311,7 +316,7 @@ void req_modifyjob(
 
   /* if a resource limit changed for a running job, send to MOM */
 
-  if (sendmom) 
+  if (sendmom)
     {
     if ((rc = relay_to_mom(
            pjob->ji_qs.ji_un.ji_exect.ji_momaddr,
@@ -372,7 +377,7 @@ int modify_job_attr(
     perm,         /* I */
     bad);         /* O */
 
-  /* if resource limits are being changing ... */
+  /* if resource limits are being changed ... */
 
   if ((rc == 0) &&
       (newattr[(int)JOB_ATR_resource].at_flags & ATR_VFLAG_SET))
@@ -419,29 +424,37 @@ int modify_job_attr(
 
   if (rc == 0) 
     {
-	    for (i=0; i<JOB_ATR_LAST; i++) {
-		if (newattr[i].at_flags & ATR_VFLAG_MODIFY) {
-			if (job_attr_def[i].at_action) {
-				rc = job_attr_def[i].at_action(&newattr[i],
-					        pjob, ATR_ACTION_ALTER);
-				if (rc)
-					break;
-			}
-		}
-	    }
-	    if ((rc == 0) &&
-	        ((newattr[(int)JOB_ATR_userlst].at_flags & ATR_VFLAG_MODIFY) ||
-                 (newattr[(int)JOB_ATR_grouplst].at_flags & ATR_VFLAG_MODIFY))){
-		/* Need to reset execution uid and gid */
-		rc = set_jobexid(pjob, newattr);
-	    }
-			
+    for (i = 0;i < JOB_ATR_LAST;i++) 
+      {
+      if (newattr[i].at_flags & ATR_VFLAG_MODIFY) 
+        {
+        if (job_attr_def[i].at_action) 
+          {
+          rc = job_attr_def[i].at_action(
+            &newattr[i],
+            pjob, 
+            ATR_ACTION_ALTER);
+
+          if (rc)
+            break;
+          }
+        }
+      }    /* END for (i) */
+
+    if ((rc == 0) &&
+       ((newattr[(int)JOB_ATR_userlst].at_flags & ATR_VFLAG_MODIFY) ||
+        (newattr[(int)JOB_ATR_grouplst].at_flags & ATR_VFLAG_MODIFY)))
+      {
+      /* need to reset execution uid and gid */
+
+      rc = set_jobexid(pjob,newattr,NULL);
+      }
     }  /* END if (rc == 0) */
 
   if (rc != 0) 
     {
     for (i = 0;i < JOB_ATR_LAST;i++)
-      job_attr_def[i].at_free(newattr+i);
+      job_attr_def[i].at_free(newattr + i);
 
     /* FAILURE */
 
