@@ -90,8 +90,6 @@
  *	req_commit()
  */
 
-/* #define __TNW */
-
 #include <pbs_config.h>   /* the master config generated by configure */
 
 #include <stdio.h>
@@ -119,10 +117,10 @@
 #include "log.h"
 #include "svrfunc.h"
 
-#ifdef PBS_MOM
-#include <pwd.h>
-#include "mom_func.h"
-#endif	/* PBS_MOM */
+#include "work_task.h"
+extern void  job_clone_wt A_((struct work_task *));
+extern int setup_array_struct(job *pjob);
+
 
 /* External Functions Called: */
 
@@ -134,12 +132,11 @@ extern int  job_route A_((job *));
 
 /* Global Data Items: */
 
-#ifndef PBS_MOM
 extern char *path_spool;
 extern struct server server;
 extern char  server_name[];
 extern int   queue_rank;
-#endif	/* !PBS_MOM */
+extern tlist_head	svr_jobarrays; 
 
 extern const char *PJobSubState[];
 
@@ -181,15 +178,15 @@ static char *user_account_default A_((char *));
 static int user_account_read_user A_((char *));
 #endif /* PNOT */
 
-#ifndef PBS_MOM
 static char *pbs_o_que = "PBS_O_QUEUE=";
-#endif
 
 
 
 
 /*
  * req_quejob - Queue Job Batch Request processing routine
+ *  NOTE:  calls svr_chkque() to validate queue access
+ *
  */
 
 void req_quejob(
@@ -210,7 +207,6 @@ void req_quejob(
   int		 rc;
   int		 sock = preq->rq_conn;
 
-#ifndef PBS_MOM
   int		 i;
   char		 buf[256];
   int		 fds;
@@ -220,13 +216,11 @@ void req_quejob(
   char		*qname;
   attribute	 tempattr;
   char           EMsg[1024];
-#endif /* server */
 
   /* set basic (user) level access permission */
 
   resc_access_perm = ATR_DFLAG_USWR|ATR_DFLAG_Creat;
 
-#ifndef PBS_MOM		/* server */
 
   /*
    * if the job id is supplied, the request had better be 
@@ -255,20 +249,23 @@ void req_quejob(
     } 
   else 
     {
-    /* assign it a job id */
+    char host_server[PBS_MAXSERVERNAME + 1];
 
     created_here = JOB_SVFLG_HERE;
 
-#ifdef JOBARRAYTESTING
-    sprintf(jidbuf,"%d-%d.",
-      server.sv_qs.sv_jobidnumber,1);
-#else
     sprintf(jidbuf,"%d.",
       server.sv_qs.sv_jobidnumber);
-#endif
+ 
+    memset(host_server,0,sizeof(host_server));
 
-
-    strcat(jidbuf,server_name);
+    if (get_fullhostname(pbs_default(),host_server,PBS_MAXSERVERNAME,NULL) == 0)
+      {
+      strcat(jidbuf,host_server);
+      }
+    else
+      {
+      strcat(jidbuf,server_name);
+      }
 
     jid = jidbuf;
 		
@@ -277,6 +274,11 @@ void req_quejob(
     if (++server.sv_qs.sv_jobidnumber > PBS_SEQNUMTOP)
       server.sv_qs.sv_jobidnumber = 0;	/* wrap it */
 
+    /* Make the current job number visible in qmgr print server commnad. */
+    server.sv_attr[(int)SRV_ATR_NextJobNumber].at_val.at_long = server.sv_qs.sv_jobidnumber;
+    server.sv_attr[(int)SRV_ATR_NextJobNumber].at_flags |= ATR_VFLAG_SET|ATR_VFLAG_MODIFY;
+
+
     if (svr_save(&server,SVR_SAVE_QUICK)) 
       {
       /* FAILURE */
@@ -287,29 +289,6 @@ void req_quejob(
       }
     }
 
-#else /* PBS_MOM */
-
-  if (preq->rq_fromsvr) 
-    {		
-    /* from another server - accept the extra attributes */
-
-    resc_access_perm |= ATR_DFLAG_MGWR|ATR_DFLAG_SvWR|ATR_DFLAG_MOM;
-
-    jid = preq->rq_ind.rq_queuejob.rq_jid;
-    } 
-  else 
-    {
-    /* request must be from server */
-
-    log_err(errno,id,"request not from server");
-
-    req_reject(PBSE_IVALREQ,0,preq,NULL,"request not received from server");
-
-    return;
-    }
-
-#endif /* PBS_MOM */
-
   /* does job already exist, check both old and new jobs */
 
   if ((pj = find_job(jid)) == NULL) 
@@ -325,8 +304,6 @@ void req_quejob(
       }
     }
 
-#ifndef PBS_MOM	/* server */
-
   if (pj != NULL) 
     {
     /* server will reject queue request if job already exists */
@@ -363,7 +340,7 @@ void req_quejob(
     {
     /* FAILURE */
 
-    log_err(errno,id,"requested queue not found");
+    log_err(-1,id,"requested queue not found");
 
     req_reject(rc,0,preq,NULL,"cannot locate queue"); /* not there   */
 
@@ -372,8 +349,8 @@ void req_quejob(
 
   /*
    * make up job file name, it is based on the jobid, however the
-   * minimun file name is only 14 character in POSIX, so we may have
-   * to "hash" the name slightly
+   * minimun acceptable file name limit is only 14 character in POSIX, 
+   * so we may have to "hash" the name slightly
    */
 
   strncpy(basename,jid,PBS_JOBBASE);
@@ -382,8 +359,8 @@ void req_quejob(
   do {
     strcpy(namebuf,path_jobs);
     strcat(namebuf,basename);
-    strcat(namebuf,JOB_FILE_SUFFIX);
-
+    strcat(namebuf, JOB_FILE_SUFFIX);
+    
     fds = open(namebuf,O_CREAT|O_EXCL|O_WRONLY,0600);
 
     if (fds < 0) 
@@ -440,111 +417,6 @@ void req_quejob(
     return;
     }
 
-#else /* PBS_MOM */
-
-  /*
-   * New job ...
-   *
-   * for MOM - rather than make up a hashname, we use the name sent
-   * to us by the server as an attribute.
-   */
-
-  psatl = (svrattrl *)GET_NEXT(preq->rq_ind.rq_queuejob.rq_attr);
-
-  while (psatl != NULL) 
-    {
-    if (!strcmp(psatl->al_name,ATTR_hashname)) 
-      {
-      strcpy(basename,psatl->al_value);
-
-      break;
-      }
-
-    psatl = (svrattrl *)GET_NEXT(psatl->al_link);
-    }
-
-  if (pj != NULL) 
-    {
-    /* newly queued job already exists */
-
-    if (pj->ji_qs.ji_substate == JOB_SUBSTATE_RUNNING) 
-      {
-      /* FAILURE - job exists and is running */
-
-      log_err(errno,id,"cannot queue new job, job exists and is running");
-
-      req_reject(PBSE_JOBEXIST,0,preq,NULL,"job is running");
-
-      return;
-      }
-
-    /* if checkpointed, then keep old and skip rest of process */
-
-    if (pj->ji_qs.ji_svrflags & JOB_SVFLG_CHKPT) 
-      {
-      pj->ji_qs.ji_substate = JOB_SUBSTATE_TRANSIN;
-
-      if (reply_jobid(preq,pj->ji_qs.ji_jobid,BATCH_REPLY_CHOICE_Queue) == 0) 
-        {
-        delete_link(&pj->ji_alljobs);
-
-        append_link(&svr_newjobs,&pj->ji_alljobs,pj);
-
-        pj->ji_qs.ji_un_type = JOB_UNION_TYPE_NEW;
-        pj->ji_qs.ji_un.ji_newt.ji_fromsock = sock;
-        pj->ji_qs.ji_un.ji_newt.ji_fromaddr = get_connectaddr(sock);
-        pj->ji_qs.ji_un.ji_newt.ji_scriptsz = 0;
-        } 
-      else 
-        {
-        close_conn(sock);
-        }
-
-      return;
-      }  /* END if (pj->ji_qs.ji_svrflags & JOB_SVFLG_CHKPT) */
-
-    /* unlink job from svr_alljobs since it will be placed on newjobs */
-
-    delete_link(&pj->ji_alljobs);
-    }  /* END if (pj != NULL) */ 
-  else 
-    {
-    /* if not already here, allocate job struct */
-
-    if ((pj = job_alloc()) == NULL) 
-      {
-      /* FAILURE */
-
-      req_reject(PBSE_SYSTEM,0,preq,NULL,"cannot allocate new job structure");
-
-      return;
-      }
-
-    strcpy(namebuf,path_jobs);      /* job directory path */
-    strcat(namebuf,basename);
-    strcat(namebuf,JOB_TASKDIR_SUFFIX);
-
-    if ((mkdir(namebuf,0700) == -1) && (errno != EEXIST))
-      {
-      /* FAILURE */
-
-      char tmpLine[1024];
-
-      sprintf(tmpLine,"cannot create directory '%s'",
-        namebuf);
-
-      log_err(errno,tmpLine,msg_init_abt);
-
-      job_purge(pj);
-
-      req_reject(PBSE_SYSTEM,0,preq,NULL,tmpLine);
-
-      return;
-      }
-    }    /* END else (pj != NULL) */
-
-#endif  /* PBS_MOM */
-
   strcpy(pj->ji_qs.ji_jobid,jid);
   strcpy(pj->ji_qs.ji_fileprefix,basename);
 
@@ -568,17 +440,7 @@ void req_quejob(
 
       /* didn`t recognize the name */
 
-#ifndef PBS_MOM
       index = JOB_ATR_UNKN;	/* keep as "unknown" for now */
-#else	/* is PBS_MOM */
-      /* FAILURE */
-
-      job_purge(pj);   /* CRI - 12/20/2004 */
-
-      reply_badattr(PBSE_NOATTR,1,psatl,preq);
-
-      return;
-#endif	/* PBS_MOM */
       }
 
     pdef = &job_attr_def[index];
@@ -604,8 +466,6 @@ void req_quejob(
       psatl->al_resc, 
       psatl->al_value);
 
-#ifndef PBS_MOM
-
     if (rc != 0) 
       {
       if (rc == PBSE_UNKRESC) 
@@ -641,56 +501,9 @@ void req_quejob(
         }
       }    /* END if (rc != 0) */
 
-#else	/* PBS_MOM */
-
-    if (rc != 0) 
-      {
-      /* FAILURE */
-
-      /* all errors are fatal for MOM */
-
-      job_purge(pj);
-
-      reply_badattr(rc,1,psatl,preq);
-
-      return;
-      }
-
-    if (psatl->al_op == DFLT) 
-      {
-      if (psatl->al_resc) 
-        {
-        resource     *presc;
-        resource_def *prdef;
-
-        prdef = find_resc_def(svr_resc_def,psatl->al_resc,svr_resc_size);
-
-        if (prdef == NULL) 
-          {
-          job_purge(pj);
-
-          reply_badattr(rc,1,psatl,preq);
-
-          return;
-          }
-
-        presc = find_resc_entry(&pj->ji_wattr[index],prdef);
-
-        if (presc != NULL)
-          presc->rs_value.at_flags |= ATR_VFLAG_DEFLT;
-        } 
-      else 
-        {
-        pj->ji_wattr[index].at_flags |= ATR_VFLAG_DEFLT;
-        }
-      }    /* END if (psatl->al_op == DFLT) */
-#endif     /* PBS_MOM */
-
     psatl = (svrattrl *)GET_NEXT(psatl->al_link);
     }      /* END while (psatl != NULL) */
 
-#ifndef PBS_MOM
-
   /* perform any at_action routine declared for the attributes */
 
   for (i = 0;i < JOB_ATR_LAST;++i) 
@@ -838,6 +651,17 @@ void req_quejob(
       return;
       }
 
+    /* If queue has checkpoint directory name specified, propagate it to the job. */
+
+    if (!(pj->ji_wattr[(int)JOB_ATR_chkptdir].at_flags & ATR_VFLAG_SET) &&
+         (pque->qu_attr[(int)QE_ATR_ChkptDir].at_flags & ATR_VFLAG_SET))
+      {
+      job_attr_def[(int)JOB_ATR_chkptdir].at_set(
+        &pj->ji_wattr[(int)JOB_ATR_chkptdir],
+        &pque->qu_attr[(int)QE_ATR_ChkptDir],
+        SET);
+      }
+
 #ifdef PNOT
     /*************************************************************
      * Argonne National Laboratory Account/Project enforcement patch
@@ -948,8 +772,6 @@ void req_quejob(
   pj->ji_wattr[(int)JOB_ATR_substate].at_val.at_long = JOB_SUBSTATE_TRANSIN;
   pj->ji_wattr[(int)JOB_ATR_substate].at_flags |= ATR_VFLAG_SET;
 
-#endif  /* !PBS_MOM */
-
   /* set remaining job structure elements */
 
   pj->ji_qs.ji_state =    JOB_STATE_TRANSIT;
@@ -1010,17 +832,13 @@ void req_jobcredential(
     return;
     }
 
-#ifndef PBS_MOM
-
-  if (svr_authorize_jobreq(preq, pj) == -1) 
+  if (svr_authorize_jobreq(preq,pj) == -1) 
     {
-    req_reject(PBSE_PERM,0,preq,NULL,NULL);
+    req_reject(PBSE_PERM,0,preq,NULL,"job request not authorized");
 
     return;
     }
 
-#endif	/* PBS_MOM */
-
   reply_ack(preq);
 
   return;
@@ -1045,16 +863,7 @@ void req_jobscript(
   int	 fds;
   char	 namebuf[MAXPATHLEN];
   job	*pj;
-#ifdef PBS_MOM
-  int	 filemode = 0700;
-  extern char mom_host[];
-#else	/* server */
   int	 filemode = 0600;
-#endif
-
-#ifdef __TNW
-  int    DoRetry = 1;
-#endif /* __TNW */
 
   errno = 0;
 
@@ -1090,57 +899,26 @@ void req_jobscript(
 
     log_err(errno,id,log_buffer);
 
-#ifdef PBS_MOM
-
-    /* NOTE:  state is out of sync between pbs_server and pbs_mom */
-    /*        pbs_server believes job is idle, pbs_mom does not */
-    /*        MOM should check local job state, and if not idle, purge the job */
-    /*        w/job_purge(pj) */
-
-    req_reject(PBSE_IVALREQ,0,preq,mom_host,log_buffer);
-#else
     req_reject(PBSE_IVALREQ,0,preq,NULL,log_buffer);
-#endif /* PBS_MOM */
 
     return;
     }
 
-#ifndef PBS_MOM
-
   if (svr_authorize_jobreq(preq,pj) == -1) 
     {
     /* FAILURE */
 
     log_err(errno,id,"cannot authorize request");
 
-    req_reject(PBSE_PERM,0,preq,NULL,NULL);
+    req_reject(PBSE_PERM,0,preq,NULL,"cannot receive job script");
 
     return;
     }
 
-#else
-
-  /* mom - if job has been checkpointed, discard script,already have it */
-
-  if (pj->ji_qs.ji_svrflags & JOB_SVFLG_CHKPT) 
-    {
-    /* SUCCESS - do nothing, ignore script */
-
-    reply_ack(preq);
-
-    return;
-    }
-
-#endif	/* PBS_MOM */
-
   strcpy(namebuf,path_jobs);  
   strcat(namebuf,pj->ji_qs.ji_fileprefix);
   strcat(namebuf,JOB_SCRIPT_SUFFIX);
 
-#ifdef __TNW
-retry:
-#endif
-
   if (pj->ji_qs.ji_un.ji_newt.ji_scriptsz == 0) 
     {
     /* NOTE:  fail is job script already exists */
@@ -1161,37 +939,13 @@ retry:
       errno,
       strerror(errno));
 
-#ifdef __TNW
-
-    if (errno == EEXIST)
-      {
-      if (DoRetry == 1)
-        {
-        /* stale script detected - remove it and retry */
-
-        remove(namebuf);
-  
-        log_err(errno,id,"duplicate script file located and removed");
-
-        DoRetry = 0;
-
-        goto retry;
-        }
-      }
-
-#endif /* __TNW */
-
     /* FAILURE */
 
     /* NOTE: log_err may modify errno */
 
     log_err(errno,id,msg_script_open);
 
-#ifdef PBS_MOM
-    req_reject(PBSE_INTERNAL,0,preq,mom_host,tmpLine);
-#else
     req_reject(PBSE_INTERNAL,0,preq,NULL,tmpLine);
-#endif /* PBS_MOM */
 
     return;
     }
@@ -1205,11 +959,7 @@ retry:
 
     log_err(errno,id,msg_script_write);
 
-#ifdef PBS_MOM
-    req_reject(PBSE_INTERNAL,0,preq,mom_host,"cannot write job command file");
-#else
     req_reject(PBSE_INTERNAL,0,preq,NULL,"cannot write job command file");
-#endif /* PBS_MOM */
 
     close(fds);
 
@@ -1235,7 +985,6 @@ retry:
 
 
 
-#ifndef PBS_MOM	
 /* the following is for the server only, MOM has her own version below */
 
 /*
@@ -1360,99 +1109,6 @@ void req_mvjobfile(  /* NOTE:  routine for server only - mom code follows this r
   return;
   }  /* END req_mvjobfile() */
 
-#else /* PBS_MOM */
-
-/*
- * req_mvjobfile - move the specifled job standard files 
- *	This is MOM's version.  The files are owned by the user and placed
- *	in either the spool area or the user's home directory depending
- *	on the compile option, see std_file_name().
- */
-
-void req_mvjobfile(  /* routine for MOM only - server routine listed above */
-
-  struct batch_request *preq)  /* I */
-
-  {
-  int	         fds;
-  enum job_file  jft;
-  int	         oflag;
-  job	        *pj;
-  struct passwd *pwd;
-
-  jft = (enum job_file)preq->rq_ind.rq_jobfile.rq_type;
-
-  if (preq->rq_ind.rq_jobfile.rq_sequence == 0)
-    oflag = O_CREAT | O_WRONLY | O_TRUNC;
-  else
-    oflag = O_CREAT | O_WRONLY | O_APPEND;
-
-  pj = locate_new_job(preq->rq_conn,NULL);
-
-  if (pj == NULL)
-    pj = find_job(preq->rq_ind.rq_jobfile.rq_jobid);
-
-  if (pj == NULL) 
-    {
-    snprintf(log_buffer,1024,"cannot find job %s for move of %s file",
-      preq->rq_ind.rq_jobfile.rq_jobid,
-      TJobFileType[jft]);
-
-    log_err(-1,"req_mvjobfile",log_buffer);
-
-    req_reject(PBSE_UNKJOBID,0,preq,NULL,NULL);
-
-    return;
-    }
-
-  if ((pj->ji_grpcache == NULL) && (check_pwd(pj) == NULL))
-    {
-    req_reject(PBSE_UNKJOBID,0,preq,NULL,NULL);
-
-    return;
-    }
-    
-  if (((pwd = getpwnam(pj->ji_wattr[(int)JOB_ATR_euser].at_val.at_str)) == NULL) || 
-      ((fds = open_std_file(pj,jft,oflag,pwd->pw_gid)) < 0)) 
-    {
-    req_reject(PBSE_MOMREJECT,0,preq,NULL,NULL);
-
-    return;
-    }
-
-  if (write(
-       fds,
-       preq->rq_ind.rq_jobfile.rq_data, 
-       preq->rq_ind.rq_jobfile.rq_size) != preq->rq_ind.rq_jobfile.rq_size)
-    {
-    req_reject(PBSE_SYSTEM,0,preq,NULL,NULL);
-    }
-  else
-    {
-    reply_ack(preq);	
-    }
-
-  close(fds);
-
-  if (LOGLEVEL >= 6)
-    {
-    sprintf(log_buffer,"successfully moved %s file for job '%s'",
-      TJobFileType[jft],
-      preq->rq_ind.rq_jobfile.rq_jobid);
-
-    log_record(
-      PBSEVENT_JOB,
-      PBS_EVENTCLASS_JOB,
-      (pj != NULL) ? pj->ji_qs.ji_jobid : "NULL",
-      log_buffer);
-    }
-
-  return;
-  }  /* END req_mvjobfile() */
-
-
-#endif /* PBS_MOM */
-
 
 
 
@@ -1466,7 +1122,7 @@ void req_mvjobfile(  /* routine for MOM only - server routine listed above */
 
 void req_rdytocommit(
 
-  struct batch_request *preq)
+  struct batch_request *preq)  /* I */
 
   {
   job *pj;
@@ -1476,6 +1132,8 @@ void req_rdytocommit(
   int  OrigSState;
   char OrigSChar;
   long OrigFlags;
+  
+  char namebuf[MAXPATHLEN+1];
 
   pj = locate_new_job(sock,preq->rq_ind.rq_rdytocommit);
 
@@ -1510,8 +1168,6 @@ void req_rdytocommit(
     return;
     }
 
-#ifndef PBS_MOM
-
   if (svr_authorize_jobreq(preq,pj) == -1) 
     {
     req_reject(PBSE_PERM,0,preq,NULL,"cannot authorize jobreq");
@@ -1521,8 +1177,6 @@ void req_rdytocommit(
     return;
     }
 
-#endif /* !PBS_MOM */
-
   OrigState  = pj->ji_qs.ji_state;
   OrigSState = pj->ji_qs.ji_substate;
   OrigSChar  = pj->ji_wattr[(int)JOB_ATR_state].at_val.at_char;
@@ -1532,6 +1186,17 @@ void req_rdytocommit(
   pj->ji_qs.ji_substate = JOB_SUBSTATE_TRANSICM;
   pj->ji_wattr[(int)JOB_ATR_state].at_val.at_char = 'T';
   pj->ji_wattr[(int)JOB_ATR_state].at_flags |= ATR_VFLAG_SET;
+  
+  if (pj->ji_wattr[(int)JOB_ATR_job_array_request].at_flags & ATR_VFLAG_SET)
+    {
+    pj->ji_isparent = TRUE;
+          
+    strcpy(namebuf,path_jobs);
+    strcat(namebuf,pj->ji_qs.ji_fileprefix);
+    strcat(namebuf, JOB_FILE_SUFFIX);
+    unlink(namebuf);
+        
+    }
 
   if (job_save(pj,SAVEJOB_NEW) == -1) 
     {
@@ -1607,12 +1272,10 @@ void req_commit(
   {
   job	  *pj;
 
-#ifndef PBS_MOM	/* SERVER only */
   int	   newstate;
   int	   newsub;
   pbs_queue *pque;
   int	   rc;
-#endif /* SERVER only */
 
   pj = locate_new_job(preq->rq_conn,preq->rq_ind.rq_commit);
 
@@ -1641,91 +1304,6 @@ void req_commit(
     return;
     }
 
-#ifdef PBS_MOM	/* MOM only */
-
-  /* move job from new job list to "all" job list, set to running state */
-
-  delete_link(&pj->ji_alljobs);
-  append_link(&svr_alljobs,&pj->ji_alljobs,pj);
-
-  /*
-  ** Set JOB_SVFLG_HERE to indicate that this is Mother Superior.
-  */
-
-  pj->ji_qs.ji_svrflags |= JOB_SVFLG_HERE;
-
-  pj->ji_qs.ji_state = JOB_STATE_RUNNING;
-  pj->ji_qs.ji_substate = JOB_SUBSTATE_PRERUN;
-  pj->ji_qs.ji_un_type = JOB_UNION_TYPE_MOM;
-  pj->ji_qs.ji_un.ji_momt.ji_svraddr = get_connectaddr(preq->rq_conn);
-  pj->ji_qs.ji_un.ji_momt.ji_exitstat = 0;
-
-  /* For MOM - start up the job (blocks) */
-
-  if (LOGLEVEL >= 6)
-    {
-    log_record(
-      PBSEVENT_JOB,
-      PBS_EVENTCLASS_JOB,
-      (pj != NULL) ? pj->ji_qs.ji_jobid : "NULL",
-      "starting job execution");
-    }
-
-  start_exec(pj);
-
-  if (LOGLEVEL >= 6)
-    {
-    log_record(
-      PBSEVENT_JOB,
-      PBS_EVENTCLASS_JOB,
-      (pj != NULL) ? pj->ji_qs.ji_jobid : "NULL",
-      "job execution started");
-    }
-	
-  /* if start request fails, reply with failure string */
-
-  if (pj->ji_qs.ji_substate == JOB_SUBSTATE_EXITING)
-    {
-    char tmpLine[1024];
-
-    if ((pj->ji_hosts != NULL) && 
-        (pj->ji_nodekill >= 0) && 
-        (pj->ji_hosts[pj->ji_nodekill].hn_host != NULL))
-      {
-      sprintf(tmpLine,"start failed on node %s",
-        pj->ji_hosts[pj->ji_nodekill].hn_host);
-      }
-    else
-      {
-      sprintf(tmpLine,"start failed on unknown node");
-      }
-
-    if (LOGLEVEL >= 6)
-      {
-      log_record(
-        PBSEVENT_JOB,
-        PBS_EVENTCLASS_JOB,
-        (pj != NULL) ? pj->ji_qs.ji_jobid : "NULL",
-        tmpLine);
-      }
-
-    reply_text(preq,0,tmpLine);
-    }
-  else
-    {	
-    reply_jobid(preq,pj->ji_qs.ji_jobid,BATCH_REPLY_CHOICE_Commit);
-    }
-
-  job_save(pj,SAVEJOB_FULL);
-
-  /* NOTE: we used to flag JOB_ATR_errpath, JOB_ATR_outpath,
-   * JOB_ATR_session_id, and JOB_ATR_altid as modified at this point to make sure
-   * pbs_server got these attr values.  This worked fine before TORQUE modified
-   * job launched into an async process.  At 2.0.0p6, a new attribute "SEND" flag
-   * was added to handle this process. */
-
-#else	/* PBS_SERVER */
-
   if (svr_authorize_jobreq(preq,pj) == -1) 
     {
     req_reject(PBSE_PERM,0,preq,NULL,NULL);
@@ -1746,6 +1324,20 @@ void req_commit(
 
   delete_link(&pj->ji_alljobs);
 
+  /* job array, setup cloning work task and reply with placeholder job id
+     *** job array under development */
+  if (pj->ji_wattr[(int)JOB_ATR_job_array_request].at_flags & ATR_VFLAG_SET)
+    {	
+    if (setup_array_struct(pj))
+      {
+      req_reject(PBSE_BAD_ARRAY_REQ,0,preq,NULL,NULL);
+      return;
+      }	
+    
+    reply_jobid(preq,pj->ji_qs.ji_jobid,BATCH_REPLY_CHOICE_Commit);
+    return;
+    }  /* end if (pj->ji_wattr[(int)JOB_ATR_job_array_request].at_flags & ATR_VFLAG_SET) */
+
   svr_evaljobstate(pj,&newstate,&newsub,1);
 
   svr_setjobstate(pj,newstate,newsub);
@@ -1848,7 +1440,6 @@ void req_commit(
 
     issue_track(pj);
     }
-#endif		/* PBS_SERVER */
 
   return;
   }  /* END req_commit() */
@@ -1904,15 +1495,6 @@ static job *locate_new_job(
 
         break;
         }
-#ifdef __TNW
-      else if ((pj->ji_qs.ji_substate != JOB_SUBSTATE_TRANSIN) || 
-               (time_now - pj->ji_wattr[(int)JOB_ATR_mtime].at_val.at_long > 9000))
-        {
-        /* ignore 'stale' recovered job */
-
-        /* NO-OP */
-        }
-#endif /* __TNW */
       else
         {
         /* matching job slot located */
@@ -2040,7 +1622,7 @@ user_account_default_done:
     log_buffer);
 
   return(rc);
-  }
+  }  /* END user_account_default() */
 
 
 
@@ -2062,7 +1644,7 @@ int user_account_read_user(
   {
   char  proj_file[] = AcctFile;
   int   fd;
-  char  s_buf[64*1024];
+  char  s_buf[128*1024];
   int   readsize;
   int   scanmode = AcctScanUser;
   int   i, j;
@@ -2085,54 +1667,95 @@ int user_account_read_user(
     return(0);
     }
 
-        readsize = read(fd, s_buf, sizeof(s_buf));
-        close(fd);
-       if (readsize < 1 || readsize > sizeof(s_buf))
-           return(0);                       /* Bail if not sane */
+  readsize = read(fd,s_buf,sizeof(s_buf));
 
-       for (i=0; i<readsize; ++i) {
-           /* First,  handle comments and whitespace */
-           if (scanmode == AcctScanLine) {  /* Looking for new line */
-               if (s_buf[i] == '\n')        /* Found it */
-                   scanmode = AcctScanUser;
-               continue;
-           }
-           if (isspace(s_buf[i]))            /* Skip spaces */
-                continue;
-           if (s_buf[i] == '#') {            /* Comment found */
-               scanmode = AcctScanLine;
-                continue;
-            }
+  close(fd);
 
-           /* Next, handle user and account scanning */
-            if (scanmode == AcctScanUser) {
-               if ((i+arguserlen) > readsize) /* Past the end */
-                   return(0);
-               if (strncmp(&s_buf[i], arguser, arguserlen)) {
-                   scanmode = AcctScanLine;   /* Not arguser, next line */
-                    continue;
-                }
-               if (isspace(s_buf[i+arguserlen]) ||
-                           s_buf[i+arguserlen] == ':') {
-                  i += arguserlen;            /* Is arguser */
-                  scanmode = AcctScanAcct;
-                } else {                       /* Whatever, ignore it */
-                  scanmode = AcctScanLine;
-                  continue;
-               }
-           } else {                           /* scanmode == AcctScanAcct */
-              if (s_buf[i] == ':' || isspace(s_buf[i]))
-                  continue;
-               for (j=i; j<readsize; j++) {
-                  if (isspace(s_buf[j])) {
-                      strncpy(UserAcct.ActRaw, &s_buf[i], j-i);
-                      UserAcct.ActRaw[j-i] = '\0';
-                      goto have_account;
-                   }
-             }
-              return(0);
-           }
-       }
+  if ((readsize < 1) || (readsize > sizeof(s_buf)))
+    {
+    /* bail if not sane */
+
+    return(0);   
+    }
+
+  for (i = 0;i < readsize;++i) 
+    {
+    /* First, handle comments and whitespace */
+
+    if (scanmode == AcctScanLine) 
+      {  
+      /* Looking for new line */
+
+      if (s_buf[i] == '\n')        /* Found it */
+        scanmode = AcctScanUser;
+
+      continue;
+      }
+
+    if (isspace(s_buf[i]))            /* Skip spaces */
+      continue;
+
+    if (s_buf[i] == '#') 
+      {
+      /* Comment found */
+
+      scanmode = AcctScanLine;
+
+      continue;
+      }
+
+    /* Next, handle user and account scanning */
+
+    if (scanmode == AcctScanUser) 
+      {
+      if ((i + arguserlen) > readsize) /* Past the end */
+        {
+        return(0);
+        }
+
+      if (strncmp(&s_buf[i],arguser,arguserlen)) 
+        {
+        scanmode = AcctScanLine;   /* Not arguser, next line */
+
+        continue;
+        }
+
+      if (isspace(s_buf[i + arguserlen]) ||
+          s_buf[i + arguserlen] == ':') 
+        {
+        i += arguserlen;            /* Is arguser */
+
+        scanmode = AcctScanAcct;
+        } 
+      else 
+        {
+        /* Whatever, ignore it */
+
+        scanmode = AcctScanLine;
+
+        continue;
+        }
+      } 
+    else 
+      {                           
+      /* scanmode == AcctScanAcct */
+
+      if ((s_buf[i] == ':') || isspace(s_buf[i]))
+        continue;
+
+      for (j = i;j < readsize;j++) 
+        {
+        if (isspace(s_buf[j])) 
+          {
+          strncpy(UserAcct.ActRaw, &s_buf[i], j-i);
+          UserAcct.ActRaw[j-i] = '\0';
+          goto have_account;
+          }
+        }
+
+      return(0);
+      }
+    }    /* END for (i) */
 
   return(0);                             /* Never found it */
 
