@@ -85,12 +85,19 @@
 
 #include <stdlib.h>
 #include <ctype.h>
-#include "pbs_ifl.h" 
+#include <stdio.h>
+#include <stdio.h>
 #include <string.h>
+#include <sys/types.h>
+#include "pbs_ifl.h" 
 #include "list_link.h" 
 #include "attribute.h" 
 #include "resource.h" 
 #include "pbs_error.h"
+#include "server_limits.h"
+#include "server.h"
+
+extern struct server server;
 
 /*
  * The entries for each attribute are (see attribute.h):
@@ -108,9 +115,11 @@
 
 static int decode_nodes A_((struct attribute *, char *, char *, char *));
 static int set_node_ct A_((resource *, attribute *, int actmode));
+static int set_tokens A_((struct attribute *attr, struct attribute *new, enum batch_op actmode));
 
+resource_def *svr_resc_def;
 
-resource_def svr_resc_def[] = {
+resource_def svr_resc_def_const[] = {
 
     {	"arch",				/* system architecture type */
 	decode_str,
@@ -549,90 +558,101 @@ resource_def svr_resc_def[] = {
 	NO_USER_SET | ATR_DFLAG_MOM | ATR_DFLAG_RMOMIG,
 	ATR_TYPE_STR
     },
-    /* Cray specific */
+    { ATTR_tokens,                               /* tokens required to run */
+	decode_tokens,
+	encode_str,
+	set_tokens,
+	comp_str,
+	free_str,
+	NULL_FUNC,
+	READ_WRITE,
+	ATR_TYPE_STR
+    },
+
+    /* Cray XT */
     { "size", decode_l, encode_l, set_l, comp_l, free_null, NULL_FUNC, READ_WRITE | ATR_DFLAG_MOM, ATR_TYPE_LONG },
     { "cpapartitionid", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
     { "cpaalloccookie", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
     { "cpaadmincookie", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
-    {	"mppwidth",     /* Number mpp PEs (processing elements) */
-	decode_l,    /* PE = ALPS launched binary invocation on compute node */
-	encode_l,
-	set_l,
-	comp_l,
-	free_null,
-	NULL_FUNC,
-	READ_WRITE,
-	ATR_TYPE_LONG
+    {   "mppwidth",     /* Number mpp PEs (processing elements) */
+        decode_l,    /* PE = ALPS launched binary invocation on compute node */
+        encode_l,
+        set_l,
+        comp_l,
+        free_null,
+        NULL_FUNC,
+        READ_WRITE,
+        ATR_TYPE_LONG
     },
-    {	"mppdepth",    /* Number of threads per PE */
-	decode_l,
-	encode_l,
-	set_l,
-	comp_l,
-	free_null,
-	NULL_FUNC,
-	READ_WRITE,
-	ATR_TYPE_LONG
+    {   "mppdepth",    /* Number of threads per PE */
+        decode_l,
+        encode_l,
+        set_l,
+        comp_l,
+        free_null,
+        NULL_FUNC,
+        READ_WRITE,
+        ATR_TYPE_LONG
     },
-    {	"mppnppn",    /* number of PEs per Node */
-	decode_l,     /* applies to multi-core systems only */
-	encode_l,
-	set_l,
-	comp_l,
-	free_null,
-	NULL_FUNC,
-	READ_WRITE,
-	ATR_TYPE_LONG
+    {   "mppnppn",    /* number of PEs per Node */
+        decode_l,     /* applies to multi-core systems only */
+        encode_l,
+        set_l,
+        comp_l,
+        free_null,
+        NULL_FUNC,
+        READ_WRITE,
+        ATR_TYPE_LONG
     },
-    {	"mppnodes",    /* node list (can be of the form 1,3-7) */
-	decode_str,    
-	encode_str,
-	set_str,
-	comp_str,
-	free_str,
-	NULL_FUNC,
-	READ_WRITE,
-	ATR_TYPE_STR
+    {   "mppnodes",    /* node list (can be of the form 1,3-7) */
+        decode_str,
+        encode_str,
+        set_str,
+        comp_str,
+        free_str,
+        NULL_FUNC,
+        READ_WRITE,
+        ATR_TYPE_STR
     },
-    {	"mpplabels",    /* compute node features */
-	decode_str,    
-	encode_str,
-	set_str,
-	comp_str,
-	free_str,
-	NULL_FUNC,
-	READ_WRITE,
-	ATR_TYPE_STR
+    {   "mpplabels",    /* compute node features */
+        decode_str,
+        encode_str,
+        set_str,
+        comp_str,
+        free_str,
+        NULL_FUNC,
+        READ_WRITE,
+        ATR_TYPE_STR
     },
-    {	"mpptimeMPP",   /* NYI */
-	decode_time,
-	encode_time,
-	set_l,
-	comp_l,
-	free_null,
-	NULL_FUNC,
-	READ_WRITE,
-	ATR_TYPE_LONG
+    {   "mpptimeMPP",   /* NYI */
+        decode_time,
+        encode_time,
+        set_l,
+        comp_l,
+        free_null,
+        NULL_FUNC,
+        READ_WRITE,
+        ATR_TYPE_LONG
     },
-    {	"mpphostMPP",
-	decode_str,
-	encode_str,
-	set_str,
-	comp_str,
-	free_str,
-	NULL_FUNC,
-	READ_WRITE,
-	ATR_TYPE_STR
+    {   "mpphostMPP",
+        decode_str,
+        encode_str,
+        set_str,
+        comp_str,
+        free_str,
+        NULL_FUNC,
+        READ_WRITE,
+        ATR_TYPE_STR
     },
-    {	"mpparchMPP",    /* node architecture (XT3, XT4, etc.) */
-	decode_str,
-	encode_str,
-	set_str,
-	comp_str,
-	free_str,
-	NULL_FUNC,
-	READ_WRITE,
-	ATR_TYPE_STR
+    {   "mpparchMPP",    /* node architecture (XT3, XT4, etc.) */
+        decode_str,
+        encode_str,
+        set_str,
+        comp_str,
+        free_str,
+        NULL_FUNC,
+        READ_WRITE,
+        ATR_TYPE_STR
     },
 
     /* support external resource manager extensions */
@@ -645,17 +665,20 @@ resource_def svr_resc_def[] = {
     { "ddisk", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
     { "dmem", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
     { "feature", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
-    { "jobflags", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
+    { "flags", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
     { "gattr", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
     { "geometry", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
     { "gmetric", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
-    { "gres", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
+    { "gres", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE|ATR_DFLAG_MOM|ATR_DFLAG_RMOMIG, ATR_TYPE_STR },
     { "hostlist", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
+    { "image", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
     { "jgroup", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
+    { "jobflags", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },  /* same as flags */
     { "latency", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
     { "loglevel", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
     { "minprocspeed", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
     { "minpreempttime", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
+    { "minwclimit", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
     { "naccesspolicy", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
     { "nallocpolicy", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
     { "nodeset", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
@@ -663,15 +686,22 @@ resource_def svr_resc_def[] = {
     { "os", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
     { "partition", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
     { "pref", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
+    { "procs", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
     { "qos", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
     { "queuejob", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
+    { "reqattr", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
+    { "retrycount", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
+    { "retrycc", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
     { "rmtype", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
+    { "select", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
     { "sid", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
     { "signal", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
     { "stagein", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
     { "spriority", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
-    { "gres", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
+    { "subnode", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
+    { "subnode_list", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
     { "taskdistpolicy", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
+    { "template", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
     { "termsig", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
     { "termtime", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
     { "tid", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
@@ -679,6 +709,7 @@ resource_def svr_resc_def[] = {
     { "trig", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
     { "trl", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
     { "var", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
+    { "wcrequeue", decode_str, encode_str, set_str, comp_str, free_str, NULL_FUNC, READ_WRITE, ATR_TYPE_STR },
 
     /* the definition for the "unknown" resource MUST be last */
 
@@ -687,7 +718,93 @@ resource_def svr_resc_def[] = {
     /* DO NOT ADD DEFINITIONS AFTER "unknown", ONLY BEFORE */
 };
 
-int svr_resc_size = sizeof(svr_resc_def) / sizeof(resource_def);
+int svr_resc_size = 0;
+
+
+int init_resc_defs(void)
+
+  {
+  resource_def *tmpresc = NULL;
+  int rindex = 0, dindex = 0, unkindex = 0;
+#ifndef PBS_MOM
+  struct array_strings *resc_arst = NULL;
+  char                 *extra_resc;
+  int			resc_num=0;
+#endif
+
+  svr_resc_size = sizeof(svr_resc_def_const) / sizeof(resource_def);
+
+#ifndef PBS_MOM
+  /* build up a temporary list of string resources */
+  if (server.sv_attr[(int)SRV_ATR_ExtraResc].at_flags & ATR_VFLAG_SET)
+    {
+    resc_arst = server.sv_attr[(int)SRV_ATR_ExtraResc].at_val.at_arst;
+
+    tmpresc = calloc(resc_arst->as_usedptr + 1,sizeof(resource_def));
+
+    if (tmpresc == NULL)
+      {
+      return(-1);
+      }
+
+    for (resc_num = 0;resc_num < resc_arst->as_usedptr;resc_num++)
+      {
+      extra_resc = resc_arst->as_string[resc_num];
+
+      (tmpresc+resc_num)->rs_name=strdup(extra_resc);
+      (tmpresc+resc_num)->rs_decode=decode_str;
+      (tmpresc+resc_num)->rs_encode=encode_str;
+      (tmpresc+resc_num)->rs_set=set_str;
+      (tmpresc+resc_num)->rs_comp=comp_str;
+      (tmpresc+resc_num)->rs_free=free_str;
+      (tmpresc+resc_num)->rs_action=NULL_FUNC;
+      (tmpresc+resc_num)->rs_flags=READ_WRITE;
+      (tmpresc+resc_num)->rs_type=ATR_TYPE_STR;
+
+      dindex++;
+
+      }
+    }
+#endif
+
+  svr_resc_def=calloc(svr_resc_size+dindex,sizeof(resource_def));
+
+  /* copy all const resources, except for the last "unknown" */
+  for (rindex=0; rindex < (svr_resc_size-1); rindex++)
+    {
+    memcpy(svr_resc_def+rindex,svr_resc_def_const+rindex,sizeof(resource_def));
+    }
+  unkindex=rindex;
+
+  /* copy our dynamic resources */
+  if (tmpresc)
+    {
+    for (dindex=0; (tmpresc+dindex)->rs_decode; dindex++)
+      {
+      if (find_resc_def(svr_resc_def,(tmpresc+dindex)->rs_name,rindex) == NULL)
+        {
+        memcpy(svr_resc_def+rindex,tmpresc+dindex,sizeof(resource_def));
+        rindex++;
+        }
+      }
+
+    free(tmpresc);
+    }
+  
+  /* copy the last "unknown" resource */
+  memcpy(svr_resc_def+rindex,svr_resc_def_const+unkindex,sizeof(resource_def));
+
+  svr_resc_size = rindex + 1;
+  
+/* uncomment if you feel like debugging this
+  for (rindex=0; rindex<svr_resc_size; rindex++)
+    {
+    fprintf(stderr,"resource: %s (%d)\n",(svr_resc_def+rindex)->rs_name,rindex);
+    }
+*/
+  return 0;
+  }
+    
 
 
 
@@ -854,7 +971,7 @@ static int set_node_ct(
 
   pnct->rs_value.at_flags |= ATR_VFLAG_SET;
 
-  /* Set "neednodes" to "nodes", may be altered by Scheduler */
+  /* Set "neednodes" to "nodes", may be altered by scheduler */
 
   pndef = find_resc_def(svr_resc_def,"neednodes",svr_resc_size);
 
@@ -885,3 +1002,52 @@ static int set_node_ct(
   }  /* END set_node_ct() */
 
 
+
+
+
+/*
+ * set_tokens = set node count
+ *
+ */
+
+static int set_tokens(
+
+  struct attribute *attr,
+  struct attribute *new,
+  enum batch_op     op)
+
+  {
+  char  *colon = NULL;
+  float  count = 0;
+
+  int ret = 0;
+
+  if (new != NULL)
+    {
+    colon = strchr(new->at_val.at_str,(int)':');
+
+    if (colon == NULL)
+      {
+      ret = PBSE_BADATVAL;
+      } 
+    else 
+      {
+      colon++;
+      count = atof(colon);
+
+      if ((count <= 0) || (count > 1000))
+        {
+        ret = PBSE_BADATVAL;
+        }
+      }
+    }
+
+  if (ret == 0)
+    {
+    ret = set_str(attr,new,op);
+    }
+
+  return(ret);
+  }  /* END set_tokens() */
+
+
