@@ -138,6 +138,7 @@ static void eval_chkpnt A_((attribute *j, attribute *q));
 extern struct server server;
 
 extern tlist_head svr_alljobs;
+extern tlist_head svr_jobarrays;
 extern char  *msg_badwait;		/* error message */
 extern char  *msg_daemonname;
 extern char  *pbs_o_host;
@@ -149,6 +150,7 @@ extern int    svr_do_schedule;
 extern int    LOGLEVEL;
 
 extern time_t time_now;
+extern int    DEBUGMODE;
 
 int           SvrNodeCt = 0;  /* cfg nodes or num nodes specified via resources_available */
 
@@ -471,7 +473,9 @@ void svr_dequejob(
     if (--server.sv_jobstates[pjob->ji_qs.ji_state] < 0)
       bad_ct = 1;
     }
+    
 
+    
   if ((pque = pjob->ji_qhdr) != (pbs_queue *)0) 
     {
     if (is_linked(&pque->qu_jobs, &pjob->ji_jobque)) 
@@ -533,7 +537,7 @@ void svr_dequejob(
   svr_do_schedule = SCH_SCHEDULE_TERM;
 
   return;
-  }  /* END svr_dequeuejob() */
+  }  /* END svr_dequejob() */
 
 
 
@@ -751,15 +755,15 @@ char *get_variable(
  * compare the job resource limit against the system limit
  * unless a queue limit exists, it takes priority
  *
- * returns number of .gt. and .lt. comparision in comp_resc_gt and comp_resc_lt
+ * returns number of .gt. and .lt. comparison in comp_resc_gt and comp_resc_lt
  *	does not make use of comp_resc_eq or comp_resc_nc
  */
 
 static void chk_svr_resc_limit(
 
-  attribute *jobatr,
-  attribute *queatr,
-  attribute *svratr,
+  attribute *jobatr, /* I */
+  attribute *queatr, /* I */
+  attribute *svratr, /* I */
   int	     qtype,  /* I */
   char      *EMsg)   /* O (optional,minsize=1024) */
 
@@ -777,20 +781,38 @@ static void chk_svr_resc_limit(
   int       LimitIsFromQueue;
   char     *LimitName;
 
-  static resource_def *noderesc = NULL;
-  static resource_def *needresc = NULL;
-  static resource_def *nodectresc = NULL;
+  /* NOTE:  support Cray-specific evaluation */
+
+  int       MPPWidth = 0;
+  int       PPN = 0;
+
+  static resource_def *noderesc     = NULL;
+  static resource_def *needresc     = NULL;
+  static resource_def *nodectresc   = NULL;
+  static resource_def *mppwidthresc = NULL;
+
+  static time_t UpdateTime = 0;
+  static time_t now;
 
   /* NOTE:  server limits are specified with server.resources_available */
 
   if (EMsg != NULL)
     EMsg[0] = '\0';
 
-  if (noderesc == NULL) 
+  time(&now);
+
+  if ((noderesc == NULL) || (now > UpdateTime + 30))
     {
-    noderesc = find_resc_def(svr_resc_def,"nodes",svr_resc_size);
-    needresc = find_resc_def(svr_resc_def,"neednodes",svr_resc_size);
-    nodectresc = find_resc_def(svr_resc_def,"nodect",svr_resc_size);
+    UpdateTime = now;
+
+    /* NOTE:  to optimize, only update once per 30 seconds */
+
+    noderesc     = find_resc_def(svr_resc_def,"nodes",svr_resc_size);
+    needresc     = find_resc_def(svr_resc_def,"neednodes",svr_resc_size);
+    nodectresc   = find_resc_def(svr_resc_def,"nodect",svr_resc_size);
+    mppwidthresc = find_resc_def(svr_resc_def,"mppwidth",svr_resc_size);
+
+    SvrNodeCt = 0;
 
     if (nodectresc != NULL)
       {
@@ -814,7 +836,7 @@ static void chk_svr_resc_limit(
         svrc = (resource *)GET_NEXT(svrc->rs_link);
         } /* END while (svrc != NULL) */
       }   /* END if (nodectresc != NULL) */
-    }     /* END if (noderesc == NULL) */
+    }     /* END if ((noderesc == NULL) || ...) */
 
   /* return values via global comp_resc_gt and comp_resc_lt */
 
@@ -860,6 +882,16 @@ static void chk_svr_resc_limit(
 
         jbrc_nodes = jbrc;
         }
+#ifdef NERSCDEV
+      else if (jbrc->rs_defin == mppwidthresc)
+        {
+
+        if (jbrc->rs_value.at_flags & ATR_VFLAG_SET)
+          {
+          MPPWidth = jbrc->rs_value.at_val.at_long;
+          }
+        }
+#endif /* NERSCDEV */
       else if ((cmpwith != NULL) && (jbrc->rs_defin != needresc)) 
         {
         /* don't check neednodes */
@@ -932,6 +964,17 @@ static void chk_svr_resc_limit(
         comp_resc_lt++;
         }
       }
+    }    /* END if (jbrc_nodes != NULL) */
+
+  if (MPPWidth > 0)
+    {
+    /* NYI */
+
+    if (PPN == 0)
+      {
+      if (EMsg != NULL)
+        strcpy(EMsg,"MPP Width Detected");
+      }
     }
 
   return;
@@ -956,6 +999,9 @@ int chk_resc_limits(
   {
   /* NOTE:  comp_resc_gt and comp_resc_lt are global ints */
 
+  if (EMsg != NULL)
+    EMsg[0] = '\0';
+
   /* first check against queue minimum */
 
   if ((comp_resc2(
@@ -1001,8 +1047,10 @@ int chk_resc_limits(
  *
  *	returns 0 for yes, 1 for no
  *
- * Note: the following fields must be set in the job struture before
- *	 calling svr_chkque(): 	ji_wattr[JOB_ATR_job_owner]
+ * NOTE: the following fields must be set in the job struture before
+ *   calling svr_chkque():  ji_wattr[JOB_ATR_job_owner]
+ * NOTE: calls chk_resc_limits() to validate resource request
+ *
  * Also if the job is being considered for an execution queue, then
  * set_jobexid() will be called.
  */
@@ -1022,7 +1070,7 @@ int svr_chkque(
   int user_jobs;
   job *pj;
   struct array_strings *pas;
-  int j=0;
+  int j = 0;
 
   if (EMsg != NULL)
     EMsg[0] = '\0';
@@ -1039,7 +1087,7 @@ int svr_chkque(
 
     if (!(pjob->ji_wattr[(int)JOB_ATR_euser].at_flags & ATR_VFLAG_SET))
       {
-      if ((i = set_jobexid(pjob,pjob->ji_wattr)) != 0)
+      if ((i = set_jobexid(pjob,pjob->ji_wattr,EMsg)) != 0)
         {
         return(i);  /* PBSE_BADUSER or GRP */
         }
@@ -1067,52 +1115,53 @@ int svr_chkque(
       {
       return(PBSE_NOATTR);
       }
-      
+
     /* 1e. check queue's disallowed_types */
+
     if (pque->qu_attr[QA_ATR_DisallowedTypes].at_flags & ATR_VFLAG_SET)
       {
-      
       for (i = 0; 
            i < (pque->qu_attr[QA_ATR_DisallowedTypes]).at_val.at_arst->as_usedptr;
            i++)
         {
-	
+	/* if job is interactive...*/
+
         if ((pjob->ji_wattr[(int)JOB_ATR_interactive].at_flags & ATR_VFLAG_SET) &&
             (pjob->ji_wattr[(int)JOB_ATR_interactive].at_val.at_long > 0))
           {
           if (strcmp(Q_DT_interactive, 
                 pque->qu_attr[QA_ATR_DisallowedTypes].at_val.at_arst->as_string[i]) == 0)
             {
-            return (PBSE_NOINTERACTIVE);
+            return(PBSE_NOINTERACTIVE);
             }
           }
-        else
+        else /* else job is batch... */
           {
-	   if (strcmp(Q_DT_batch, 
-                 pque->qu_attr[QA_ATR_DisallowedTypes].at_val.at_arst->as_string[i]) == 0)
-             {
-             return (PBSE_NOBATCH);
-             }
+          if (strcmp(Q_DT_batch, 
+               pque->qu_attr[QA_ATR_DisallowedTypes].at_val.at_arst->as_string[i]) == 0)
+            {
+            return(PBSE_NOBATCH);
+            }
           }
-
-        if (strcmp(Q_DT_rerunable,
-              pque->qu_attr[QA_ATR_DisallowedTypes].at_val.at_arst->as_string[i]) == 0
+	  
+	if (strcmp(Q_DT_rerunable,
+	      pque->qu_attr[QA_ATR_DisallowedTypes].at_val.at_arst->as_string[i]) == 0
             && (pjob->ji_wattr[(int)JOB_ATR_rerunable].at_flags & ATR_VFLAG_SET &&
-                pjob->ji_wattr[(int)JOB_ATR_rerunable].at_val.at_long > 0))
-          {
-          return (PBSE_NORERUNABLE);
-          }
+	        pjob->ji_wattr[(int)JOB_ATR_rerunable].at_val.at_long > 0))
+	  {
+	  return(PBSE_NORERUNABLE);
+	  }
+
 	if (strcmp(Q_DT_nonrerunable,
 	      pque->qu_attr[QA_ATR_DisallowedTypes].at_val.at_arst->as_string[i]) == 0
             && (!(pjob->ji_wattr[(int)JOB_ATR_rerunable].at_flags & ATR_VFLAG_SET) ||
 	        pjob->ji_wattr[(int)JOB_ATR_rerunable].at_val.at_long == 0))
 	  {
-	  return (PBSE_NONONRERUNABLE);
+	  return(PBSE_NONONRERUNABLE);
 	  }
-
         }	
-      }
-      			
+      }    /* END if (pque->qu_attr[QA_ATR_DisallowedTypes].at_flags & ATR_VFLAG_SET) */
+		
     /* 1f. if enabled, check the queue's group ACL */
 
     if (pque->qu_attr[QA_ATR_AclGroupEnabled].at_val.at_long)
@@ -1154,7 +1203,8 @@ int svr_chkque(
         /* fetch the groups in the ACL and look for matching user membership */
 
 	pas = pque->qu_attr[QA_ATR_AclGroup].at_val.at_arst;
-	for (i=0;i<pas->as_usedptr;i++) 
+
+	for (i = 0;i < pas->as_usedptr;i++) 
 	  {
           if ((grp = getgrnam(pas->as_string[i])) == NULL) 
             continue;
@@ -1164,6 +1214,7 @@ int svr_chkque(
             if (!strcmp(grp->gr_mem[j],uname))
               {
               rc = 1;
+
               break;
               }
             }
@@ -1171,7 +1222,7 @@ int svr_chkque(
           if (rc == 1)
             break;
           }
-        }    /* END if (rc == 0) && AclGroupSloppy...) */
+        }    /* END if (rc == 0) && slpygrp && ...) */
 
       if (rc == 0)
         {
@@ -1199,8 +1250,8 @@ int svr_chkque(
             }
           }
         }
-      }
-    }
+      }    /* END if (pque->qu_attr[QA_ATR_AclGroupEnabled].at_val.at_long) */
+    }      /* END if (pque->qu_qs.qu_type == QTYPE_Execution) */
 
   /* checks 2 and 3 are bypassed for a move by manager or qorder */
 
@@ -1298,7 +1349,7 @@ int svr_chkque(
         }
       }
 
-    /* 5.5. If failed user and group acls, fail */
+    /* 5.5. if failed user and group acls, fail */
 
     if (failed_group_acl && failed_user_acl)
       {
@@ -1562,19 +1613,26 @@ char *prefix_std_file(
  *	The "to" buffer must be large enought (PBS_MAXUSER+1).
  */
 
-void get_jobowner(from, to)
-	char *from;
-	char *to;
-{
-	int i;
+void get_jobowner(
 
-	for (i=0; i<PBS_MAXUSER; ++i) {
-		if ( (*(from+i) == '@') || (*(from+i) == '\0') )
-			break;
-		*(to+i) = *(from+i);
-	}
-	*(to+i) = '\0';
-}
+  char *from,
+  char *to)
+
+  {
+  int i;
+
+  for (i = 0;i < PBS_MAXUSER;++i) 
+    {
+    if ((*(from + i) == '@') || (*(from + i) == '\0'))
+      break;
+
+    *(to + i) = *(from + i);
+    }
+
+  *(to + i) = '\0';
+
+  return;
+  }
 
 
 
@@ -1695,6 +1753,8 @@ void set_resc_deflt(
 
 
 
+
+
 /*
  * set_statechar - set the job state attribute to the letter that correspondes
  *	to its current state.
@@ -1725,7 +1785,9 @@ void set_statechar(
       pjob->ji_wattr[JOB_ATR_state].at_val.at_char = 'U'; /* Unknown */
       }
     }
-  }
+
+  return;
+  }  /* END set_statechar() */
 
 
 
@@ -1738,30 +1800,44 @@ void set_statechar(
  *	to the queue min time. 
  */
 
-static void eval_chkpnt(jobckp, queckp)
-	attribute *jobckp;	/* job's checkpoint attribute */
-	attribute *queckp;	/* queue's checkpoint attribute */
-{
-	int jobs;
-	char queues[30];
-	char *pv;
-	
-	if ( ((jobckp->at_flags & ATR_VFLAG_SET) == 0)  ||
-	     ((queckp->at_flags & ATR_VFLAG_SET) == 0) )
-		return;		/* need do nothing */
+static void eval_chkpnt(
 
-	pv = jobckp->at_val.at_str;
-	if (*pv++ == 'c') {
-		if (*pv == '=')
-			pv++;
-		jobs = atoi(pv);
-		if (jobs < queckp->at_val.at_long) {
-			(void)sprintf(queues, "c=%ld", queckp->at_val.at_long);
-			free_str(jobckp);
-			(void)decode_str(jobckp, 0, 0, queues);
-		}
-	}
-}
+  attribute *jobckp,	/* job's checkpoint attribute */
+  attribute *queckp)	/* queue's checkpoint attribute */
+
+  {
+  int jobs;
+  char queues[30];
+  char *pv;
+	
+  if (((jobckp->at_flags & ATR_VFLAG_SET) == 0) ||
+      ((queckp->at_flags & ATR_VFLAG_SET) == 0))
+    {
+    return;		/* need do nothing */
+    }
+
+  pv = jobckp->at_val.at_str;
+
+  if (*pv++ == 'c') 
+    {
+    if (*pv == '=')
+      pv++;
+
+    jobs = atoi(pv);
+
+    if (jobs < queckp->at_val.at_long) 
+      {
+      sprintf(queues,"c=%ld", 
+        queckp->at_val.at_long);
+
+      free_str(jobckp);
+
+      decode_str(jobckp,0,0,queues);
+      }
+    }
+
+  return;
+  }  /* END eval_chkpnt() */
 
 
 
