@@ -76,13 +76,17 @@
 * This license will be governed by the laws of the Commonwealth of Virginia,
 * without reference to its choice of law rules.
 */
-/*
- * svr_recov.c - contains functions to save server state and recover
+
+/**
+ * @file svr_recov.c
+ *
+ * contains functions to save server state and recover
  *
  * Included functions are:
  *	svr_recov()
  *	svr_save()
  */
+
 #include <pbs_config.h>   /* the master config generated by configure */
 
 #include <sys/types.h>
@@ -112,15 +116,19 @@ extern attribute_def svr_attr_def[];
 extern char	    *path_svrdb;
 extern char	    *path_svrdb_new;
 extern char	    *path_priv;
-extern time_t	     time_now;
+extern time_t    time_now;
 extern char	    *msg_svdbopen;
 extern char	    *msg_svdbnosv;
 
 
 
 
-/*
- * svr_recov() - recover server state from server database
+/**
+ * Recover server state from server database.
+ *
+ * @param ps     A pointer to the server state structure.
+ * @param mode   This is either SVR_SAVE_QUICK or SVR_SAVE_FULL.
+ * @return       Zero on success or -1 on error.
  */
 
 int svr_recov(
@@ -128,6 +136,7 @@ int svr_recov(
   char *svrfile)  /* I */
 
   {
+  static char *id = "svr_recov";
   int i;
   int sdb;
 
@@ -137,7 +146,19 @@ int svr_recov(
 
   if (sdb < 0) 
     {
-    log_err(errno,"svr_recov",msg_svdbopen);
+    if (errno == ENOENT)
+      {
+      char tmpLine[LOG_BUF_SIZE];
+
+      snprintf(tmpLine,sizeof(tmpLine),"cannot locate server database '%s' - use 'pbs_server -t create' to create new database if database has not been initialized.",
+        svrfile);
+
+      log_err(errno,id,tmpLine);
+      }
+    else
+      {
+      log_err(errno,id,msg_svdbopen);
+      }
 
     return(-1);
     }
@@ -149,15 +170,18 @@ int svr_recov(
   if (i != sizeof(struct server_qs)) 
     {
     if (i < 0) 
-      log_err(errno,"svr_recov","read of serverdb failed");
+      log_err(errno,id,"read of serverdb failed");
     else
-      log_err(errno,"svr_recov","short read of serverdb");
+      log_err(errno,id,"short read of serverdb");
 
     close(sdb);
 
     return(-1);
     }
-	
+
+  /* Save the sv_jobidnumber field in case it is set by the attributes. */
+  i = server.sv_qs.sv_jobidnumber;
+ 	
   /* read in server attributes */
 
   if (recov_attr(
@@ -168,13 +192,19 @@ int svr_recov(
 	(int)SRV_ATR_LAST,
         0) != 0 ) 
     {
-    log_err(errno,"svr_recov","error on recovering server attr");
+    log_err(errno,id,"error on recovering server attr");
 
     close(sdb);
 
     return(-1);
     }
 
+  /* Restore the current job number and make it visible in qmgr print server commnad. */
+  server.sv_qs.sv_jobidnumber = i;
+  server.sv_attr[(int)SRV_ATR_NextJobNumber].at_val.at_long = i;
+  server.sv_attr[(int)SRV_ATR_NextJobNumber].at_flags |= ATR_VFLAG_SET|ATR_VFLAG_MODIFY;
+
+
   close(sdb);
 	
   /* recover the server various acls from their own files */
@@ -206,8 +236,12 @@ int svr_recov(
 
 
 
-/*
- * svr_save - save the state of the server (server structure)
+/**
+ * Save the state of the server (server structure).
+ *
+ * @param ps     A pointer to the server state structure.
+ * @param mode   This is either SVR_SAVE_QUICK or SVR_SAVE_FULL.
+ * @return       Zero on success or -1 on error.
  */
 
 int svr_save(
@@ -216,11 +250,11 @@ int svr_save(
   int            mode)
 
   {
+  static char *this_function_name = "svr_save";
   int i;
   int sdb;
   int save_acl A_((attribute *,attribute_def *,char *,char *));
 
-  char *my_id = "svr_save";
 
   if (mode == SVR_SAVE_QUICK) 
     {
@@ -228,7 +262,7 @@ int svr_save(
 
     if (sdb < 0) 
       {
-      log_err(errno,"svr_recov",msg_svdbopen);
+      log_err(errno, this_function_name, msg_svdbopen);
 
       return(-1);
       }
@@ -241,7 +275,7 @@ int svr_save(
       if ((i == -1) && (errno == EINTR))
         continue;
 
-      log_err(errno,my_id,msg_svdbnosv);
+      log_err(errno, this_function_name, msg_svdbnosv);
 
       return(-1);
       }
@@ -256,7 +290,7 @@ int svr_save(
 
     if (sdb < 0) 
       {
-      log_err(errno,"svr_recov",msg_svdbopen);
+      log_err(errno, this_function_name, msg_svdbopen);
 
       return(-1);
       }
@@ -270,7 +304,7 @@ int svr_save(
       snprintf(log_buffer,1024,"cannot save data into server db, errno=%d",
         errno);
 
-      log_err(errno,"svr_recov",log_buffer);
+      log_err(errno, this_function_name, log_buffer);
 
       close(sdb);
 
@@ -301,7 +335,7 @@ int svr_save(
       snprintf(log_buffer,1024,"cannot move new database to default database location, errno=%d",
         errno);
 
-      log_err(errno,"svr_recov",log_buffer);
+      log_err(errno, this_function_name, log_buffer);
       }
     else
       {
@@ -315,8 +349,7 @@ int svr_save(
       {
       if (ps->sv_attr[i].at_type == ATR_TYPE_ACL)
         save_acl(&ps->sv_attr[i],&svr_attr_def[i],
-
-      PBS_SVRACL,svr_attr_def[i].at_name);
+          PBS_SVRACL,svr_attr_def[i].at_name);
       }
     }    /* END else (mode == SVR_SAVE_QUICK) */
 
@@ -326,18 +359,25 @@ int svr_save(
 
 
 
-/*
- * save_acl() - save an Access Control List to its file 
+/**
+ * Save an Access Control List to its file.
+ *
+ * @param attr   A pointer to an acl (access control list) attribute.
+ * @param pdef   A pointer to the attribute definition structure.
+ * @param subdir The sub-directory path specifying where to write the file.
+ * @param name   The parent object name which in this context becomes the file name.
+ * @return       Zero on success (File may not be written if attribute is clean) or -1 on error.
  */
 
 int save_acl(
 
-  attribute     *attr,	/* acl attribute */
-  attribute_def *pdef,	/* attribute def structure */
-  char          *subdir,	/* sub-directory path */
-  char          *name)	/* parent object name = file name */
+  attribute     *attr,	 /* acl attribute */
+  attribute_def *pdef,	 /* attribute def structure */
+  char          *subdir, /* sub-directory path */
+  char          *name)	 /* parent object name = file name */
 
   {
+  static char *this_function_name = "save_acl";
   int  fds;
   char filename1[MAXPATHLEN];
   char filename2[MAXPATHLEN];
@@ -369,42 +409,65 @@ int save_acl(
   strcpy(filename2,filename1);
   strcat(filename2,".new");
 
-	fds = open(filename2, O_WRONLY|O_CREAT|O_TRUNC|O_Sync, 0600);
-	if (fds < 0) {
-		log_err(errno, "save_acl", "unable to open acl file");
-		return (-1);
-	}
-	
-	CLEAR_HEAD(head);
-	i = pdef->at_encode(attr,&head,pdef->at_name,(char *)0,ATR_ENCODE_SAVE);
-	if (i < 0) {
-		log_err(-1, "save_acl", "unable to encode acl");
-		(void)close(fds);
-		(void)unlink(filename2);
-		return (-1);
-	}
-	
-	pentry = (svrattrl *)GET_NEXT(head);
-	if (pentry != (svrattrl *)0) {
-		/* write entry, but without terminating null */
-		while ((i = write(fds, pentry->al_value, pentry->al_valln - 1))
-		      != pentry->al_valln - 1) {
-			if ((i == -1) && (errno == EINTR))
-				continue;
-			log_err(errno,"save_acl","wrote incorrect amount");	
-			(void)close(fds);
-			(void)unlink(filename2);
-			return (-1);
-		}
-		(void)free(pentry);
-	}
+  fds = open(filename2,O_WRONLY|O_CREAT|O_TRUNC|O_Sync,0600);
 
-	(void)close(fds);
-	(void)unlink(filename1);
-	if (link(filename2, filename1) < 0) {
-		log_err(errno, "save_acl", "unable to relink file");
-		return (-1);
-	}
+  if (fds < 0) 
+    {
+    snprintf(log_buffer,1024,"unable to open acl file '%s'",
+      filename2);
+
+    log_err(errno, this_function_name, log_buffer);
+
+    return(-1);
+    }
+	
+  CLEAR_HEAD(head);
+
+  i = pdef->at_encode(attr,&head,pdef->at_name,(char *)0,ATR_ENCODE_SAVE);
+
+  if (i < 0) 
+    {
+    log_err(-1, this_function_name, "unable to encode acl");
+
+    close(fds);
+
+    unlink(filename2);
+
+    return(-1);
+    }
+	
+  pentry = (svrattrl *)GET_NEXT(head);
+
+  if (pentry != NULL) 
+    {
+    /* write entry, but without terminating null */
+
+    while ((i = write(fds,pentry->al_value,pentry->al_valln - 1)) != pentry->al_valln - 1)
+      {
+      if ((i == -1) && (errno == EINTR))
+        continue;
+
+      log_err(errno, this_function_name, "wrote incorrect amount");	
+
+      close(fds);
+
+      unlink(filename2);
+
+      return(-1);
+      }
+
+    free(pentry);
+    }
+
+  close(fds);
+  unlink(filename1);
+
+  if (link(filename2,filename1) < 0) 
+    {
+    log_err(errno, this_function_name, "unable to relink file");
+
+    return(-1);
+    }
 
   unlink(filename2);
 
@@ -414,11 +477,14 @@ int save_acl(
   }
 
 
-
-
-
-/*
- * recov_acl() - reload an Access Control List from its file 
+/**
+ * Reload an Access Control List from its file.
+ *
+ * @param attr   A pointer to an acl (access control list) attribute.
+ * @param pdef   A pointer to the attribute definition structure.
+ * @param subdir The sub-directory path specifying where to read the file.
+ * @param name   The parent object name which in this context is the file name.
+ * @return       Zero on success (File may not be written if attribute is clean) or -1 on error.
  */
 
 void recov_acl(
@@ -429,6 +495,7 @@ void recov_acl(
   char	        *name)	/* parent object name = file name */
 
   {
+  static char *this_function_name = "recov_acl";
   char *buf;
   int   fds;
   char  filename1[MAXPATHLEN];
@@ -456,7 +523,7 @@ void recov_acl(
       sprintf(log_buffer, "unable to open acl file %s",
         filename1);
 
-      log_err(errno,"recov_acl",log_buffer);
+      log_err(errno, this_function_name, log_buffer);
       }
 
     return;
@@ -483,7 +550,7 @@ void recov_acl(
 
   if (read(fds,buf,(unsigned int)sb.st_size) != (int)sb.st_size) 
     {
-    log_err(errno,"recov_acl","unable to read acl file");
+    log_err(errno, this_function_name, "unable to read acl file");
 
     close(fds);
     free(buf);
@@ -502,14 +569,14 @@ void recov_acl(
     sprintf(log_buffer,"decode of acl %s failed",
       pdef->at_name);
 
-    log_err(errno,"recov_acl",log_buffer);
+    log_err(errno, this_function_name, log_buffer);
     } 
   else if (pdef->at_set(pattr,&tempat,SET) != 0) 
     {
     sprintf(log_buffer,"set of acl %s failed",
       pdef->at_name);
 
-    log_err(errno,"recov_acl",log_buffer);
+    log_err(errno, this_function_name, log_buffer);
     }
 
   pdef->at_free(&tempat);
